@@ -39,42 +39,9 @@ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 #ifdef TIMING
 #include <sys/time.h>
 #endif
-using namespace std ;
-viewport viewport(1000, 1000) ;
-/*
- *   This is a "renderer" that is just stubs, for performance testing.
- */
-class nullrender : public liferender {
-public:
-   nullrender() {}
-   virtual ~nullrender() {}
-   virtual void killrect(int, int, int, int) {}
-   virtual void pixblit(int, int, int, int, char*, int) {}
-   virtual void getcolors(unsigned char** r, unsigned char** g, unsigned char** b) {
-      static unsigned char dummy[256];
-      *r = *g = *b = dummy;
-   }
-} ;
 
-nullrender renderer ;
-/*
- *   This is a "lifeerrors" that we can use to test some edge
- *   conditions.  In this case the only real thing we use
- *   it for is to check rendering during a progress dialog.
- */
-class nullerrors : public lifeerrors {
-public:
-   nullerrors() {}
-   virtual void fatal(const char *) {}
-   virtual void warning(const char *) {}
-   virtual void status(const char *) {}
-   virtual void beginprogress(const char *s) { abortprogress(0, s) ; }
-   virtual bool abortprogress(double, const char *) ;
-   virtual void endprogress() { abortprogress(1, "") ; }
-   virtual const char* getuserrules() { return "" ; }
-   virtual const char* getrulesdir() { return "Rules/" ; }
-} ;
-nullerrors nullerror ;
+using namespace std ;
+
 #ifdef TIMING
 double start ;
 double timestamp() {
@@ -86,30 +53,78 @@ double timestamp() {
    return r ;
 }
 #endif
+
+viewport viewport(1000, 1000) ;
+lifealgo *imp = 0 ;
+
 /*
- *   This is a "lifeerrors" that we can use to test some edge
- *   conditions.  In this case the only real thing we use
- *   it for is to check rendering during a progress dialog.
+ *   This is a "renderer" that is just stubs, for performance testing.
  */
-class verbosestatus : public lifeerrors {
+class nullrender : public liferender {
 public:
-   verbosestatus() {}
-   virtual void fatal(const char *) {}
-   virtual void warning(const char *) {}
+   nullrender() {}
+   virtual ~nullrender() {}
+   virtual void pixblit(int, int, int, int, unsigned char*, int) {}
+   virtual void getcolors(unsigned char** r, unsigned char** g, unsigned char** b,
+                          unsigned char* dead_alpha, unsigned char* live_alpha) {
+      static unsigned char dummy[256];
+      *r = *g = *b = dummy;
+      *dead_alpha = *live_alpha = 255;
+   }
+} ;
+nullrender renderer ;
+
+// the RuleLoader algo looks for .rule files in the user_rules directory
+// then in the supplied_rules directory
+char* user_rules = (char *)"";              // can be changed by -s or --search
+char* supplied_rules = (char *)"Rules/";
+
+/*
+ *   This lifeerrors is used to check rendering during a progress dialog.
+ */
+class progerrors : public lifeerrors {
+public:
+   progerrors() {}
+   virtual void fatal(const char *s) { cout << "Fatal error: " << s << endl ; exit(10) ; }
+   virtual void warning(const char *s) { cout << "Warning: " << s << endl ; }
+   virtual void status(const char *s) { cout << s << endl ; }
+   virtual void beginprogress(const char *s) { abortprogress(0, s) ; }
+   virtual bool abortprogress(double, const char *) ;
+   virtual void endprogress() { abortprogress(1, "") ; }
+   virtual const char* getuserrules() { return user_rules ; }
+   virtual const char* getrulesdir() { return supplied_rules ; }
+} ;
+progerrors progerrors_instance ;
+
+bool progerrors::abortprogress(double, const char *) {
+   imp->draw(viewport, renderer) ;
+   return 0 ;
+}
+
+/*
+ *   This is our standard lifeerrors.
+ */
+class stderrors : public lifeerrors {
+public:
+   stderrors() {}
+   virtual void fatal(const char *s) { cout << "Fatal error: " << s << endl ; exit(10) ; }
+   virtual void warning(const char *s) { cout << "Warning: " << s << endl ; }
    virtual void status(const char *s) {
 #ifdef TIMING
       cout << timestamp() << " " << s << endl ;
+#else
+      cout << s << endl ;
 #endif
    }
    virtual void beginprogress(const char *) {}
    virtual bool abortprogress(double, const char *) { return 0 ; }
    virtual void endprogress() {}
-   virtual const char* getuserrules() { return "" ; }
-   virtual const char* getrulesdir() { return "Rules/" ; }
+   virtual const char* getuserrules() { return user_rules ; }
+   virtual const char* getrulesdir() { return supplied_rules ; }
 } ;
-verbosestatus verbosestatus_instance ;
+stderrors stderrors_instance ;
+
 char *filename ;
-lifealgo *imp = 0 ;
 struct options {
   const char *shortopt ;
   const char *longopt ;
@@ -138,6 +153,7 @@ options options[] = {
   { "-2", "--exponential", "Use exponentially increasing steps", 'b', &hyper },
   { "-q", "--quiet", "Don't show population; twice, don't show anything", 'b', &quiet },
   { "-r", "--rule", "Life rule to use", 's', &liferule },
+  { "-s", "--search", "Search directory for .rule files", 's', &user_rules },
   { "-h", "--hashlife", "Use Hashlife algorithm", 'b', &hashlife },
   { "-a", "--algorithm", "Select algorithm by name", 's', &algoName },
   { "-o", "--output", "Output file (*.rle, *.mc, *.rle.gz, *.mc.gz)", 's',
@@ -156,6 +172,7 @@ options options[] = {
   { "",   "--exec", "Run testing script", 's', &testscript },
   { 0, 0, 0, 0, 0 }
 } ;
+
 int endswith(const char *s, const char *suff) {
    int off = (int)(strlen(s) - strlen(suff)) ;
    if (off <= 0)
@@ -167,6 +184,7 @@ int endswith(const char *s, const char *suff) {
    numberoffset = off ;
    return 1 ;
 }
+
 void usage(const char *s) {
   fprintf(stderr, "Usage:  bgolly [options] patternfile\n") ;
   for (int i=0; options[i].shortopt; i++)
@@ -176,6 +194,7 @@ void usage(const char *s) {
     lifefatal(s) ;
   exit(0) ;
 }
+
 #define STRINGIFY(ARG) STR2(ARG)
 #define STR2(ARG) #ARG
 #define MAXRLE 1000000000
@@ -204,6 +223,7 @@ void writepat(int fc) {
       lifewarning(err) ;
    cerr << ")" << flush ;
 }
+
 const int MAXCMDLENGTH = 2048 ;
 struct cmdbase {
    cmdbase(const char *cmdarg, const char *argsarg) {
@@ -300,7 +320,9 @@ struct cmdbase {
    virtual ~cmdbase() {}
    static cmdbase *list ;
 } ;
+
 cmdbase *cmdbase::list = 0 ;
+
 struct loadcmd : public cmdbase {
    loadcmd() : cmdbase("load", "s") {}
    virtual void doit() {
@@ -312,8 +334,17 @@ struct loadcmd : public cmdbase {
 struct stepcmd : public cmdbase {
    stepcmd() : cmdbase("step", "b") {}
    virtual void doit() {
-      imp->setIncrement(barg) ;
-      imp->step() ;
+      if (imp->gridwd > 0 || imp->gridht > 0) {
+         // bounded grid, so must step by 1
+         imp->setIncrement(1) ;
+         if (!imp->CreateBorderCells()) exit(10) ;
+         imp->step() ;
+         if (!imp->DeleteBorderCells()) exit(10) ;
+      } else {
+         imp->setIncrement(barg) ;
+         imp->step() ;
+      }
+      if (timeline) imp->extendtimeline() ;
       cout << imp->getGeneration().tostring() << ": " ;
       cout << imp->getPopulation().tostring() << endl ;
    }
@@ -406,6 +437,7 @@ struct showcutcmd : public cmdbase {
          cout << cutbuf[i].first << " " << cutbuf[i].second << endl ;
    }
 } showcut_inst ;
+
 lifealgo *createUniverse() {
    if (algoName == 0) {
      if (hashlife)
@@ -426,6 +458,7 @@ lifealgo *createUniverse() {
    imp->setMaxMemory(maxmem) ;
    return imp ;
 }
+
 struct newcmd : public cmdbase {
    newcmd() : cmdbase("new", "") {}
    virtual void doit() {
@@ -463,10 +496,7 @@ struct edgescmd : public cmdbase {
       cout << " " << b.tostring() << endl ;
    }
 } edges_inst ;
-bool nullerrors::abortprogress(double, const char *) {
-   imp->draw(viewport, renderer) ;
-   return 0 ;
-}
+
 void runtestscript(const char *testscript) {
    FILE *cmdfile = 0 ;
    if (strcmp(testscript, "-") != 0)
@@ -488,8 +518,9 @@ void runtestscript(const char *testscript) {
    }
    exit(0) ;
 }
+
 int main(int argc, char *argv[]) {
-   cout << "This is bgolly " STRINGIFY(VERSION) " Copyright 2013 The Golly Gang."
+   cout << "This is bgolly " STRINGIFY(VERSION) " Copyright 2016 The Golly Gang."
         << endl << flush ;
    qlifealgo::doInitializeAlgoInfo(staticAlgoInfo::tick()) ;
    hlifealgo::doInitializeAlgoInfo(staticAlgoInfo::tick()) ;
@@ -535,12 +566,12 @@ case 's':
         }
       }
       if (!hit)
-        usage("Bad option given") ;
+         usage("Bad option given") ;
    }
    if (argc < 2 && !testscript)
-     usage("No pattern argument given") ;
+      usage("No pattern argument given") ;
    if (argc > 2)
-     usage("Extra stuff after pattern argument") ;
+      usage("Extra stuff after pattern argument") ;
    if (outfilename) {
       if (endswith(outfilename, ".rle")) {
       } else if (endswith(outfilename, ".mc")) {
@@ -559,33 +590,39 @@ case 's':
          lifefatal("Output filename too long") ;
    }
    if (timeline && hyper)
-      lifefatal("Cannot use both timeline and hyperthreading") ;
+      lifefatal("Cannot use both timeline and exponentially increasing steps") ;
    imp = createUniverse() ;
    if (progress)
-     lifeerrors::seterrorhandler(&nullerror) ;
-   else if (verbose) {
-     lifeerrors::seterrorhandler(&verbosestatus_instance) ;
-     hlifealgo::setVerbose(1) ;
+      lifeerrors::seterrorhandler(&progerrors_instance) ;
+   else
+      lifeerrors::seterrorhandler(&stderrors_instance) ;
+   if (verbose) {
+      hlifealgo::setVerbose(1) ;
    }
    imp->setMaxMemory(maxmem) ;
 #ifdef TIMING
    timestamp() ;
 #endif
    if (testscript) {
-     if (argc > 1) {
-       filename = argv[1] ;
-       const char *err = readpattern(argv[1], *imp) ;
-       if (err)
-         lifefatal(err) ;
-     }
-     runtestscript(testscript) ;
+      if (argc > 1) {
+         filename = argv[1] ;
+         const char *err = readpattern(argv[1], *imp) ;
+         if (err) lifefatal(err) ;
+      }
+      runtestscript(testscript) ;
    }
    filename = argv[1] ;
    const char *err = readpattern(argv[1], *imp) ;
-   if (err)
-      lifefatal(err) ;
-   if (liferule)
-      imp->setrule(liferule) ;
+   if (err) lifefatal(err) ;
+   if (liferule) {
+      err = imp->setrule(liferule) ;
+      if (err) lifefatal(err) ;
+   }
+   bool boundedgrid = (imp->gridwd > 0 || imp->gridht > 0) ;
+   if (boundedgrid) {
+      hyper = 0 ;
+      inc = 1 ;     // only step by 1
+   }
    if (inc != 0)
       imp->setIncrement(inc) ;
    if (timeline) {
@@ -602,9 +639,9 @@ case 's':
       if (quiet < 2) {
          cout << imp->getGeneration().tostring() ;
          if (!quiet)
-           cout << ": " << imp->getPopulation().tostring() << endl ;
+            cout << ": " << imp->getPopulation().tostring() << endl ;
          else
-           cout << endl ;
+            cout << endl ;
       }
       if (popcount)
          imp->getPopulation() ;
@@ -622,7 +659,10 @@ case 's':
          diff <<= bs ;
          imp->setIncrement(diff) ;
       }
+      if (boundedgrid && !imp->CreateBorderCells()) break ;
       imp->step() ;
+      if (boundedgrid && !imp->DeleteBorderCells()) break ;
+      if (timeline) imp->extendtimeline() ;
       if (maxgen < 0 && outfilename != 0)
          writepat(fc++) ;
       if (timeline && imp->getframecount() + 2 > MAX_FRAME_COUNT)
