@@ -428,7 +428,7 @@ int bigint::mod_smallint(int a) {
    if (v.i & 1)
       return (((v.i >> 1) % a) + a) % a ;
    int pos = v.p[0] ;
-   int mm = (1 << 31) % a ;
+   int mm = (2 * ((1 << 30) % a) % a) ;
    int r = 0 ;
    while (pos > 0) {
       r = (mm * r + v.p[pos]) % a ;
@@ -773,3 +773,9 @@ const bigint bigint::two(2) ;
 const bigint bigint::three(3) ;
 const bigint bigint::maxint(INT_MAX) ;
 const bigint bigint::minint(INT_MIN) ;
+
+// most editing operations are limited to absolute coordinates <= 10^9,
+// partly because getcell and setcell only take int parameters, but mostly
+// to avoid ridiculously long cut/copy/paste/rotate/etc operations
+const bigint bigint::min_coord(-1000000000) ;
+const bigint bigint::max_coord(+1000000000) ;
