@@ -518,7 +518,6 @@ void ghashbase::step() {
       root = newroot ;
    }
    depth = ghnode_depth(root) ;
-   extendtimeline() ;
 }
 void ghashbase::setcurrentstate(void *n) {
    if (root != (ghnode *)n) {
@@ -662,8 +661,8 @@ ghnode *ghashbase::setbit(ghnode *n, int x, int y, int newstate, int depth) {
       return (ghnode *)l ;
    } else {
       unsigned int w = 0, wh = 0 ;
-      if (depth >= 31) {
-         if (depth == this->depth)
+      if (depth > 31) {
+         if (depth == 32)
             wh = 0x80000000 ;
       } else {
          w = 1 << depth ;
@@ -735,8 +734,8 @@ int ghashbase::getbit(ghnode *n, int x, int y, int depth) {
            return l->ne ;
    } else {
       unsigned int w = 0, wh = 0 ;
-      if (depth >= 31) {
-         if (depth == this->depth)
+      if (depth >= 32) {
+         if (depth == 32)
             wh = 0x80000000 ;
       } else {
          w = 1 << depth ;
@@ -1286,6 +1285,9 @@ const bigint &ghashbase::getPopulation() {
       if (inGC) {
         needPop = 1 ;
         return negone ;
+      } else if (poller->isCalculating()) {
+        // AKT: avoid calling poller->bailIfCalculating
+        return negone ;
       } else {
         calcPopulation(root) ;
         popValid = 1 ;
@@ -1351,11 +1353,9 @@ const char *ghashbase::readmacrocell(char *line) {
             ind[indlen++] = 0 ;
       }
       if (line[0] == '#') {
-         switch (line[1]) {
          char *p, *pp ;
-
-         // AKT: need to check for explicit rule
          const char *err ;
+         switch (line[1]) {
          case 'R':
             p = line + 2 ;
             while (*p && *p <= ' ') p++ ;
