@@ -624,7 +624,6 @@ void hlifealgo::step() {
       root = newroot ;
    }
    depth = node_depth(root) ;
-   extendtimeline() ;
 }
 void hlifealgo::setcurrentstate(void *n) {
    if (root != (node *)n) {
@@ -794,8 +793,8 @@ node *hlifealgo::setbit(node *n, int x, int y, int newstate, int depth) {
       return (node *)l ;
    } else {
       unsigned int w = 0, wh = 0 ;
-      if (depth >= 31) {
-         if (depth == this->depth)
+      if (depth >= 32) {
+         if (depth == 32)
             wh = 0x80000000 ;
       } else {
          w = 1 << depth ;
@@ -869,8 +868,8 @@ int hlifealgo::getbit(node *n, int x, int y, int depth) {
       return 0 ;
    } else {
       unsigned int w = 0, wh = 0 ;
-      if (depth >= 31) {
-         if (depth == this->depth)
+      if (depth >= 32) {
+         if (depth == 32)
             wh = 0x80000000 ;
       } else {
          w = 1 << depth ;
@@ -1422,6 +1421,9 @@ const bigint &hlifealgo::getPopulation() {
       if (inGC) {
         needPop = 1 ;
         return negone ;
+      } else if (poller->isCalculating()) {
+        // AKT: avoid calling poller->bailIfCalculating
+        return negone ;
       } else {
         calcPopulation(root) ;
         popValid = 1 ;
@@ -1516,9 +1518,9 @@ default:       return "Illegal character in readmacrocell." ;
          clearstack() ;
          ind[i++] = (node *)find_leaf(lnw, lne, lsw, lse) ;
       } else if (line[0] == '#') {
-         switch (line[1]) {
          char *p, *pp ;
          const char *err ;
+         switch (line[1]) {
          case 'R':
             p = line + 2 ;
             while (*p && *p <= ' ') p++ ;
