@@ -108,11 +108,14 @@ bool SaveStartingPattern()
     currlayer->startsel = currlayer->currsel;
 
     if ( !currlayer->savestart ) {
-        // no need to save pattern; ResetPattern will load currfile
-        currlayer->startfile.clear();
+        // no need to save pattern (use currlayer->currfile as the starting pattern)
+        if (currlayer->currfile.empty())
+            Warning("Bug in SaveStartingPattern: currfile is empty!");
         return true;
     }
 
+    currlayer->currfile = currlayer->tempstart;     // ResetPattern will load tempstart
+
     // save starting pattern in tempstart file
     if ( currlayer->algo->hyperCapable() ) {
         // much faster to save pattern in a macrocell file
@@ -147,7 +150,6 @@ bool SaveStartingPattern()
         }
     }
 
-    currlayer->startfile = currlayer->tempstart;   // ResetPattern will load tempstart
     return true;
 }
 
@@ -206,7 +208,7 @@ void ResetPattern(bool resetundo)
         return;
     }
 
-    if (currlayer->startfile.length() == 0 && currlayer->currfile.length() == 0) {
+    if (currlayer->currfile.empty()) {
         // if this happens then savestart logic is wrong
         Warning("Starting pattern cannot be restored!");
         return;
@@ -221,13 +223,7 @@ void ResetPattern(bool resetundo)
     currlayer->algtype = currlayer->startalgo;
 
     // restore starting pattern
-    if ( currlayer->startfile.length() == 0 ) {
-        // restore pattern from currfile
-        LoadPattern(currlayer->currfile.c_str(), "");
-    } else {
-        // restore pattern from startfile
-        LoadPattern(currlayer->startfile.c_str(), "");
-    }
+    LoadPattern(currlayer->currfile.c_str(), "");
 
     if (currlayer->algo->getGeneration() != currlayer->startgen) {
         // LoadPattern failed to reset the gen count to startgen
@@ -235,11 +231,11 @@ void ResetPattern(bool resetundo)
         // so best to clear the pattern and reset the gen count
         CreateUniverse();
         currlayer->algo->setGeneration(currlayer->startgen);
+        std::string msg = "Failed to reset pattern from this file:\n";
+        msg += currlayer->currfile;
+        Warning(msg.c_str());
     }
 
-    // ensure savestart flag is correct
-    currlayer->savestart = currlayer->startfile.length() != 0;
-
     // restore settings saved by SaveStartingPattern
     RestoreRule(currlayer->startrule.c_str());
     currlayer->currname = currlayer->startname;
@@ -306,10 +302,12 @@ void RestorePattern(bigint& gen, const char* filename,
         LoadPattern(filename, "");
 
         if (currlayer->algo->getGeneration() != gen) {
-            // filename could not be loaded for some reason,
-            // so best to clear the pattern and set the expected gen count
+            // best to clear the pattern and set the expected gen count
             CreateUniverse();
             currlayer->algo->setGeneration(gen);
+            std::string msg = "Could not restore pattern from this file:\n";
+            msg += filename;
+            Warning(msg.c_str());
         }
 
         // restore step size and set increment
@@ -410,601 +408,6 @@ const char* ChangeGenCount(const char* genstring, bool inundoredo)
 
 // -----------------------------------------------------------------------------
 
-static void JoinTwistedEdges(lifealgo* curralgo)
-{
-    // set grid edges
-    int gl = curralgo->gridleft.toint();
-    int gt = curralgo->gridtop.toint();
-    int gr = curralgo->gridright.toint();
-    int gb = curralgo->gridbottom.toint();
-
-    // border edges are 1 cell outside grid edges
-    int bl = gl - 1;
-    int bt = gt - 1;
-    int br = gr + 1;
-    int bb = gb + 1;
-
-    if (curralgo->htwist && curralgo->vtwist) {
-        // cross-surface
-        //  eg. :C4,3
-        //  a l k j i d
-        //  l A B C D i
-        //  h E F G H e
-        //  d I J K L a
-        //  i d c b a l
-
-        for (int x = gl; x <= gr; x++) {
-            int twistedx = gr - x + gl;
-            int state = curralgo->getcell(twistedx, gt);
-            if (state > 0) curralgo->setcell(x, bb, state);
-            state = curralgo->getcell(twistedx, gb);
-            if (state > 0) curralgo->setcell(x, bt, state);
-        }
-
-        for (int y = gt; y <= gb; y++) {
-            int twistedy = gb - y + gt;
-            int state = curralgo->getcell(gl, twistedy);
-            if (state > 0) curralgo->setcell(br, y, state);
-            state = curralgo->getcell(gr, twistedy);
-            if (state > 0) curralgo->setcell(bl, y, state);
-        }
-
-        // copy grid's corner cells to SAME corners in border
-        // (these cells are topologically different to non-corner cells)
-        curralgo->setcell(bl, bt, curralgo->getcell(gl, gt));
-        curralgo->setcell(br, bt, curralgo->getcell(gr, gt));
-        curralgo->setcell(br, bb, curralgo->getcell(gr, gb));
-        curralgo->setcell(bl, bb, curralgo->getcell(gl, gb));
-
-    } else if (curralgo->htwist) {
-        // Klein bottle with top and bottom edges twisted 180 degrees
-        //  eg. :K4*,3
-        //  i l k j i l
-        //  d A B C D a
-        //  h E F G H e
-        //  l I J K L i
-        //  a d c b a d
-
-        for (int x = gl; x <= gr; x++) {
-            int twistedx = gr - x + gl;
-            int state = curralgo->getcell(twistedx, gt);
-            if (state > 0) curralgo->setcell(x, bb, state);
-            state = curralgo->getcell(twistedx, gb);
-            if (state > 0) curralgo->setcell(x, bt, state);
-        }
-
-        for (int y = gt; y <= gb; y++) {
-            // join left and right edges with no twist
-            int state = curralgo->getcell(gl, y);
-            if (state > 0) curralgo->setcell(br, y, state);
-            state = curralgo->getcell(gr, y);
-            if (state > 0) curralgo->setcell(bl, y, state);
-        }
-
-        // do corner cells
-        curralgo->setcell(bl, bt, curralgo->getcell(gl, gb));
-        curralgo->setcell(br, bt, curralgo->getcell(gr, gb));
-        curralgo->setcell(bl, bb, curralgo->getcell(gl, gt));
-        curralgo->setcell(br, bb, curralgo->getcell(gr, gt));
-
-    } else { // curralgo->vtwist
-        // Klein bottle with left and right edges twisted 180 degrees
-        //  eg. :K4,3*
-        //  d i j k l a
-        //  l A B C D i
-        //  h E F G H e
-        //  d I J K L a
-        //  l a b c d i
-
-        for (int x = gl; x <= gr; x++) {
-            // join top and bottom edges with no twist
-            int state = curralgo->getcell(x, gt);
-            if (state > 0) curralgo->setcell(x, bb, state);
-            state = curralgo->getcell(x, gb);
-            if (state > 0) curralgo->setcell(x, bt, state);
-        }
-
-        for (int y = gt; y <= gb; y++) {
-            int twistedy = gb - y + gt;
-            int state = curralgo->getcell(gl, twistedy);
-            if (state > 0) curralgo->setcell(br, y, state);
-            state = curralgo->getcell(gr, twistedy);
-            if (state > 0) curralgo->setcell(bl, y, state);
-        }
-
-        // do corner cells
-        curralgo->setcell(bl, bt, curralgo->getcell(gr, gt));
-        curralgo->setcell(br, bt, curralgo->getcell(gl, gt));
-        curralgo->setcell(bl, bb, curralgo->getcell(gr, gb));
-        curralgo->setcell(br, bb, curralgo->getcell(gl, gb));
-    }
-}
-
-// -----------------------------------------------------------------------------
-
-static void JoinTwistedAndShiftedEdges(lifealgo* curralgo)
-{
-    // set grid edges
-    int gl = curralgo->gridleft.toint();
-    int gt = curralgo->gridtop.toint();
-    int gr = curralgo->gridright.toint();
-    int gb = curralgo->gridbottom.toint();
-
-    // border edges are 1 cell outside grid edges
-    int bl = gl - 1;
-    int bt = gt - 1;
-    int br = gr + 1;
-    int bb = gb + 1;
-
-    if (curralgo->hshift != 0) {
-        // Klein bottle with shift by 1 on twisted horizontal edge (with even number of cells)
-        //  eg. :K4*+1,3
-        //  j i l k j i
-        //  d A B C D a
-        //  h E F G H e
-        //  l I J K L i
-        //  b a d c b a
-
-        int state, twistedx, shiftedx;
-
-        for (int x = gl; x <= gr; x++) {
-            // join top and bottom edges with a twist and then shift by 1
-            twistedx = gr - x + gl;
-            shiftedx = twistedx - 1; if (shiftedx < gl) shiftedx = gr;
-            state = curralgo->getcell(shiftedx, gb);
-            if (state > 0) curralgo->setcell(x, bt, state);
-
-            state = curralgo->getcell(shiftedx, gt);
-            if (state > 0) curralgo->setcell(x, bb, state);
-        }
-
-        for (int y = gt; y <= gb; y++) {
-            // join left and right edges with no twist or shift
-            state = curralgo->getcell(gl, y);
-            if (state > 0) curralgo->setcell(br, y, state);
-            state = curralgo->getcell(gr, y);
-            if (state > 0) curralgo->setcell(bl, y, state);
-        }
-
-        // do corner cells
-        shiftedx = gl - 1; if (shiftedx < gl) shiftedx = gr;
-        curralgo->setcell(bl, bt, curralgo->getcell(shiftedx, gb));
-        curralgo->setcell(bl, bb, curralgo->getcell(shiftedx, gt));
-        shiftedx = gr - 1; if (shiftedx < gl) shiftedx = gr;
-        curralgo->setcell(br, bt, curralgo->getcell(shiftedx, gb));
-        curralgo->setcell(br, bb, curralgo->getcell(shiftedx, gt));
-
-    } else { // curralgo->vshift != 0
-        // Klein bottle with shift by 1 on twisted vertical edge (with even number of cells)
-        //  eg. :K3,4*+1
-        //  f j k l d
-        //  c A B C a
-        //  l D E F j
-        //  i G H I g
-        //  f J K L d
-        //  c a b c a
-
-        int state, twistedy, shiftedy;
-
-        for (int x = gl; x <= gr; x++) {
-            // join top and bottom edges with no twist or shift
-            state = curralgo->getcell(x, gt);
-            if (state > 0) curralgo->setcell(x, bb, state);
-            state = curralgo->getcell(x, gb);
-            if (state > 0) curralgo->setcell(x, bt, state);
-        }
-
-        for (int y = gt; y <= gb; y++) {
-            // join left and right edges with a twist and then shift by 1
-            twistedy = gb - y + gt;
-            shiftedy = twistedy - 1; if (shiftedy < gt) shiftedy = gb;
-            state = curralgo->getcell(gr, shiftedy);
-            if (state > 0) curralgo->setcell(bl, y, state);
-
-            state = curralgo->getcell(gl, shiftedy);
-            if (state > 0) curralgo->setcell(br, y, state);
-        }
-
-        // do corner cells
-        shiftedy = gt - 1; if (shiftedy < gt) shiftedy = gb;
-        curralgo->setcell(bl, bt, curralgo->getcell(gr, shiftedy));
-        curralgo->setcell(br, bt, curralgo->getcell(gl, shiftedy));
-        shiftedy = gb - 1; if (shiftedy < gt) shiftedy = gb;
-        curralgo->setcell(bl, bb, curralgo->getcell(gr, shiftedy));
-        curralgo->setcell(br, bb, curralgo->getcell(gl, shiftedy));
-    }
-}
-
-// -----------------------------------------------------------------------------
-
-static void JoinShiftedEdges(lifealgo* curralgo,
-                             int gwd, int ght,        // grid wd and ht
-                             int hshift, int vshift)  // horizontal and vertical shifts
-{
-    // set grid edges
-    int gl = curralgo->gridleft.toint();
-    int gt = curralgo->gridtop.toint();
-    int gr = curralgo->gridright.toint();
-    int gb = curralgo->gridbottom.toint();
-
-    // border edges are 1 cell outside grid edges
-    int bl = gl - 1;
-    int bt = gt - 1;
-    int br = gr + 1;
-    int bb = gb + 1;
-
-    if (hshift != 0) {
-        // torus with horizontal shift
-        //  eg. :T4+1,3
-        //  k l i j k l
-        //  d A B C D a
-        //  h E F G H e
-        //  l I J K L i
-        //  a b c d a b
-
-        int state, shiftedx;
-
-        for (int x = gl; x <= gr; x++) {
-            // join top and bottom edges with a horizontal shift
-            shiftedx = x - hshift;
-            if (shiftedx < gl) shiftedx += gwd; else if (shiftedx > gr) shiftedx -= gwd;
-            state = curralgo->getcell(shiftedx, gb);
-            if (state > 0) curralgo->setcell(x, bt, state);
-
-            shiftedx = x + hshift;
-            if (shiftedx < gl) shiftedx += gwd; else if (shiftedx > gr) shiftedx -= gwd;
-            state = curralgo->getcell(shiftedx, gt);
-            if (state > 0) curralgo->setcell(x, bb, state);
-        }
-
-        for (int y = gt; y <= gb; y++) {
-            // join left and right edges with no shift
-            state = curralgo->getcell(gl, y);
-            if (state > 0) curralgo->setcell(br, y, state);
-
-            state = curralgo->getcell(gr, y);
-            if (state > 0) curralgo->setcell(bl, y, state);
-        }
-
-        // do corner cells
-        shiftedx = gr - hshift;
-        if (shiftedx < gl) shiftedx += gwd; else if (shiftedx > gr) shiftedx -= gwd;
-        curralgo->setcell(bl, bt, curralgo->getcell(shiftedx, gb));
-        shiftedx = gl - hshift;
-        if (shiftedx < gl) shiftedx += gwd; else if (shiftedx > gr) shiftedx -= gwd;
-        curralgo->setcell(br, bt, curralgo->getcell(shiftedx, gb));
-        shiftedx = gr + hshift;
-        if (shiftedx < gl) shiftedx += gwd; else if (shiftedx > gr) shiftedx -= gwd;
-        curralgo->setcell(bl, bb, curralgo->getcell(shiftedx, gt));
-        shiftedx = gl + hshift;
-        if (shiftedx < gl) shiftedx += gwd; else if (shiftedx > gr) shiftedx -= gwd;
-        curralgo->setcell(br, bb, curralgo->getcell(shiftedx, gt));
-
-    } else { // vshift != 0
-        // torus with vertical shift
-        //  eg. :T4,3+1
-        //  h i j k l a
-        //  l A B C D e
-        //  d E F G H i
-        //  h I J K L a
-        //  l a b c d e
-
-        int state, shiftedy;
-
-        for (int x = gl; x <= gr; x++) {
-            // join top and bottom edges with no shift
-            state = curralgo->getcell(x, gt);
-            if (state > 0) curralgo->setcell(x, bb, state);
-
-            state = curralgo->getcell(x, gb);
-            if (state > 0) curralgo->setcell(x, bt, state);
-        }
-
-        for (int y = gt; y <= gb; y++) {
-            // join left and right edges with a vertical shift
-            shiftedy = y - vshift;
-            if (shiftedy < gt) shiftedy += ght; else if (shiftedy > gb) shiftedy -= ght;
-            state = curralgo->getcell(gr, shiftedy);
-            if (state > 0) curralgo->setcell(bl, y, state);
-
-            shiftedy = y + vshift;
-            if (shiftedy < gt) shiftedy += ght; else if (shiftedy > gb) shiftedy -= ght;
-            state = curralgo->getcell(gl, shiftedy);
-            if (state > 0) curralgo->setcell(br, y, state);
-        }
-
-        // do corner cells
-        shiftedy = gb - vshift;
-        if (shiftedy < gt) shiftedy += ght; else if (shiftedy > gb) shiftedy -= ght;
-        curralgo->setcell(bl, bt, curralgo->getcell(gr, shiftedy));
-        shiftedy = gb + vshift;
-        if (shiftedy < gt) shiftedy += ght; else if (shiftedy > gb) shiftedy -= ght;
-        curralgo->setcell(br, bt, curralgo->getcell(gl, shiftedy));
-        shiftedy = gt - vshift;
-        if (shiftedy < gt) shiftedy += ght; else if (shiftedy > gb) shiftedy -= ght;
-        curralgo->setcell(bl, bb, curralgo->getcell(gr, shiftedy));
-        shiftedy = gt + vshift;
-        if (shiftedy < gt) shiftedy += ght; else if (shiftedy > gb) shiftedy -= ght;
-        curralgo->setcell(br, bb, curralgo->getcell(gl, shiftedy));
-    }
-}
-
-// -----------------------------------------------------------------------------
-
-static void JoinAdjacentEdges(lifealgo* curralgo,
-                              int pt, int pl, int pb, int pr)    // pattern edges
-{
-    // set grid edges
-    int gl = curralgo->gridleft.toint();
-    int gt = curralgo->gridtop.toint();
-    int gr = curralgo->gridright.toint();
-    int gb = curralgo->gridbottom.toint();
-
-    // border edges are 1 cell outside grid edges
-    int bl = gl - 1;
-    int bt = gt - 1;
-    int br = gr + 1;
-    int bb = gb + 1;
-
-    // sphere
-    //  eg. :S3
-    //  a a d g c
-    //  a A B C g
-    //  b D E F h
-    //  c G H I i
-    //  g c f i i
-
-    // copy live cells in top edge to left border
-    for (int x = pl; x <= pr; x++) {
-        int state;
-        int skip = curralgo->nextcell(x, gt, state);
-        if (skip < 0) break;
-        x += skip;
-        if (state > 0) curralgo->setcell(bl, gt + (x - gl), state);
-    }
-
-    // copy live cells in left edge to top border
-    for (int y = pt; y <= pb; y++) {
-        // no point using nextcell() here -- edge is only 1 cell wide
-        int state = curralgo->getcell(gl, y);
-        if (state > 0) curralgo->setcell(gl + (y - gt), bt, state);
-    }
-
-    // copy live cells in bottom edge to right border
-    for (int x = pl; x <= pr; x++) {
-        int state;
-        int skip = curralgo->nextcell(x, gb, state);
-        if (skip < 0) break;
-        x += skip;
-        if (state > 0) curralgo->setcell(br, gt + (x - gl), state);
-    }
-
-    // copy live cells in right edge to bottom border
-    for (int y = pt; y <= pb; y++) {
-        // no point using nextcell() here -- edge is only 1 cell wide
-        int state = curralgo->getcell(gr, y);
-        if (state > 0) curralgo->setcell(gl + (y - gt), bb, state);
-    }
-
-    // copy grid's corner cells to SAME corners in border
-    curralgo->setcell(bl, bt, curralgo->getcell(gl, gt));
-    curralgo->setcell(br, bt, curralgo->getcell(gr, gt));
-    curralgo->setcell(br, bb, curralgo->getcell(gr, gb));
-    curralgo->setcell(bl, bb, curralgo->getcell(gl, gb));
-}
-
-// -----------------------------------------------------------------------------
-
-static void JoinEdges(lifealgo* curralgo,
-                      int gwd, int ght,                  // grid wd and ht
-                      int pt, int pl, int pb, int pr)    // pattern edges
-{
-    // set grid edges
-    int gl = curralgo->gridleft.toint();
-    int gt = curralgo->gridtop.toint();
-    int gr = curralgo->gridright.toint();
-    int gb = curralgo->gridbottom.toint();
-
-    // border edges are 1 cell outside grid edges
-    int bl = gl - 1;
-    int bt = gt - 1;
-    int br = gr + 1;
-    int bb = gb + 1;
-
-    if (ght > 0) {
-        // copy live cells in top edge to bottom border
-        for (int x = pl; x <= pr; x++) {
-            int state;
-            int skip = curralgo->nextcell(x, gt, state);
-            if (skip < 0) break;
-            x += skip;
-            if (state > 0) curralgo->setcell(x, bb, state);
-        }
-        // copy live cells in bottom edge to top border
-        for (int x = pl; x <= pr; x++) {
-            int state;
-            int skip = curralgo->nextcell(x, gb, state);
-            if (skip < 0) break;
-            x += skip;
-            if (state > 0) curralgo->setcell(x, bt, state);
-        }
-    }
-
-    if (gwd > 0) {
-        // copy live cells in left edge to right border
-        for (int y = pt; y <= pb; y++) {
-            // no point using nextcell() here -- edge is only 1 cell wide
-            int state = curralgo->getcell(gl, y);
-            if (state > 0) curralgo->setcell(br, y, state);
-        }
-        // copy live cells in right edge to left border
-        for (int y = pt; y <= pb; y++) {
-            // no point using nextcell() here -- edge is only 1 cell wide
-            int state = curralgo->getcell(gr, y);
-            if (state > 0) curralgo->setcell(bl, y, state);
-        }
-    }
-
-    if (gwd > 0 && ght > 0) {
-        // copy grid's corner cells to opposite corners in border
-        curralgo->setcell(bl, bt, curralgo->getcell(gr, gb));
-        curralgo->setcell(br, bt, curralgo->getcell(gl, gb));
-        curralgo->setcell(br, bb, curralgo->getcell(gl, gt));
-        curralgo->setcell(bl, bb, curralgo->getcell(gr, gt));
-    }
-}
-
-// -----------------------------------------------------------------------------
-
-bool CreateBorderCells(lifealgo* curralgo)
-{
-    // no need to do anything if there is no pattern or if the grid is a bounded plane
-    if (curralgo->isEmpty() || curralgo->boundedplane) return true;
-
-    int gwd = curralgo->gridwd;
-    int ght = curralgo->gridht;
-
-    bigint top, left, bottom, right;
-    curralgo->findedges(&top, &left, &bottom, &right);
-
-    // no need to do anything if pattern is completely inside grid edges
-    if ( (gwd == 0 || (curralgo->gridleft < left && curralgo->gridright > right)) &&
-        (ght == 0 || (curralgo->gridtop < top && curralgo->gridbottom > bottom)) ) {
-        return true;
-    }
-
-    // if grid has infinite width or height then pattern might be too big to use setcell/getcell
-    if ( (gwd == 0 || ght == 0) && OutsideLimits(top, left, bottom, right) ) {
-        ErrorMessage("Pattern is too big!");
-        // return false so caller can exit step() loop
-        return false;
-    }
-
-    if (curralgo->sphere) {
-        // to get a sphere we join top edge with left edge, and right edge with bottom edge;
-        // note that grid must be square (gwd == ght)
-        int pl = left.toint();
-        int pt = top.toint();
-        int pr = right.toint();
-        int pb = bottom.toint();
-        JoinAdjacentEdges(curralgo, pt, pl, pb, pr);
-
-    } else if (curralgo->htwist || curralgo->vtwist) {
-        // Klein bottle or cross-surface
-        if ( (curralgo->htwist && curralgo->hshift != 0 && (gwd & 1) == 0) ||
-            (curralgo->vtwist && curralgo->vshift != 0 && (ght & 1) == 0) ) {
-            // Klein bottle with shift is only possible if the shift is on the
-            // twisted edge and that edge has an even number of cells
-            JoinTwistedAndShiftedEdges(curralgo);
-        } else {
-            JoinTwistedEdges(curralgo);
-        }
-
-    } else if (curralgo->hshift != 0 || curralgo->vshift != 0) {
-        // torus with horizontal or vertical shift
-        JoinShiftedEdges(curralgo, gwd, ght, curralgo->hshift, curralgo->vshift);
-
-    } else {
-        // unshifted torus or infinite tube
-        int pl = left.toint();
-        int pt = top.toint();
-        int pr = right.toint();
-        int pb = bottom.toint();
-        JoinEdges(curralgo, gwd, ght, pt, pl, pb, pr);
-    }
-
-    curralgo->endofpattern();
-    return true;
-}
-
-// -----------------------------------------------------------------------------
-
-void ClearRect(lifealgo* curralgo, int top, int left, int bottom, int right)
-{
-    int cx, cy, v;
-    for ( cy = top; cy <= bottom; cy++ ) {
-        for ( cx = left; cx <= right; cx++ ) {
-            int skip = curralgo->nextcell(cx, cy, v);
-            if (skip + cx > right)
-                skip = -1;           // pretend we found no more live cells
-            if (skip >= 0) {
-                // found next live cell so delete it
-                cx += skip;
-                curralgo->setcell(cx, cy, 0);
-            } else {
-                cx = right + 1;     // done this row
-            }
-        }
-    }
-}
-
-// -----------------------------------------------------------------------------
-
-bool DeleteBorderCells(lifealgo* curralgo)
-{
-    // no need to do anything if there is no pattern
-    if (curralgo->isEmpty()) return true;
-
-    int gwd = curralgo->gridwd;
-    int ght = curralgo->gridht;
-
-    // need to find pattern edges because pattern may have expanded beyond grid
-    // (typically by 2 cells, but could be more if rule allows births in empty space)
-    bigint top, left, bottom, right;
-    curralgo->findedges(&top, &left, &bottom, &right);
-
-    // no need to do anything if grid encloses entire pattern
-    if ( (gwd == 0 || (curralgo->gridleft <= left && curralgo->gridright >= right)) &&
-        (ght == 0 || (curralgo->gridtop <= top && curralgo->gridbottom >= bottom)) ) {
-        return true;
-    }
-
-    if ( OutsideLimits(top, left, bottom, right) ) {
-        ErrorMessage("Pattern is too big!");
-        // return false so caller can exit step() loop
-        return false;
-    }
-
-    // set pattern edges
-    int pl = left.toint();
-    int pt = top.toint();
-    int pr = right.toint();
-    int pb = bottom.toint();
-
-    // set grid edges
-    int gl = curralgo->gridleft.toint();
-    int gt = curralgo->gridtop.toint();
-    int gr = curralgo->gridright.toint();
-    int gb = curralgo->gridbottom.toint();
-
-    if (ght > 0 && pt < gt) {
-        // delete live cells above grid
-        ClearRect(curralgo, pt, pl, gt-1, pr);
-        pt = gt; // reduce size of rect below
-    }
-
-    if (ght > 0 && pb > gb) {
-        // delete live cells below grid
-        ClearRect(curralgo, gb+1, pl, pb, pr);
-        pb = gb; // reduce size of rect below
-    }
-
-    if (gwd > 0 && pl < gl) {
-        // delete live cells left of grid
-        ClearRect(curralgo, pt, pl, pb, gl-1);
-    }
-
-    if (gwd > 0 && pr > gr) {
-        // delete live cells right of grid
-        ClearRect(curralgo, pt, gr+1, pb, pr);
-    }
-
-    curralgo->endofpattern();
-    return true;
-}
-
-// -----------------------------------------------------------------------------
-
 void DisplayTimingInfo()
 {
     endtime = TimeInSeconds();
@@ -1056,7 +459,7 @@ bool StartGenerating()
 void StopGenerating()
 {
     if (!generating) Warning("Bug detected in StopGenerating!");
-
+    
     generating = false;
 
     PollerInterrupt();
@@ -1126,9 +529,9 @@ void NextGeneration(bool useinc)
                     inc = curralgo->getIncrement();
                     curralgo->setIncrement(1);
                 }
-                if (!CreateBorderCells(curralgo)) break;
+                if (!curralgo->CreateBorderCells()) break;
                 curralgo->step();
-                if (!DeleteBorderCells(curralgo)) break;
+                if (!curralgo->DeleteBorderCells()) break;
                 inc -= 1;
             }
             // safe way to restore correct increment in case user altered base/expo in above loop
@@ -1140,9 +543,9 @@ void NextGeneration(bool useinc)
         // step by 1 gen
         bigint saveinc = curralgo->getIncrement();
         curralgo->setIncrement(1);
-        if (boundedgrid) CreateBorderCells(curralgo);
+        if (boundedgrid) curralgo->CreateBorderCells();
         curralgo->step();
-        if (boundedgrid) DeleteBorderCells(curralgo);
+        if (boundedgrid) curralgo->DeleteBorderCells();
         curralgo->setIncrement(saveinc);
     }
 
@@ -1328,7 +731,6 @@ void ReduceCellStates(int newmaxstate)
 
 void ChangeRule(const std::string& rulestring)
 {
-    // load recently installed .rule/table/tree file
     std::string oldrule = currlayer->algo->getrule();
     int oldmaxstate = currlayer->algo->NumCellStates() - 1;
 
@@ -1350,48 +752,52 @@ void ChangeRule(const std::string& rulestring)
                     if (i != currlayer->algtype) {
                         RestoreRule(oldrule.c_str());
                         Warning("Algorithm could not be changed (pattern is too big to convert).");
-                        return;
                     } else {
                         UpdateEverything();
-                        return;
                     }
+                    return;
                 }
             }
         }
-        // should only get here if .rule/table/tree file contains some sort of error
+        // should only get here if .rule file contains some sort of error
         RestoreRule(oldrule.c_str());
         Warning("New rule is not valid in any algorithm!");
         return;
     }
 
     std::string newrule = currlayer->algo->getrule();
-    if (oldrule != newrule) {
+    int newmaxstate = currlayer->algo->NumCellStates() - 1;
+    if (oldrule != newrule || oldmaxstate != newmaxstate) {
         UpdateStatus();
+        
+		// if pattern exists and is at starting gen then ensure savestart is true
+		// so that SaveStartingPattern will save pattern to suitable file
+		// (and thus undo/reset will work correctly)
+		if (currlayer->algo->getGeneration() == currlayer->startgen && !currlayer->algo->isEmpty()) {
+			currlayer->savestart = true;
+		}
 
         // if grid is bounded then remove any live cells outside grid edges
         if (currlayer->algo->gridwd > 0 || currlayer->algo->gridht > 0) {
             ClearOutsideGrid();
         }
-    }
 
-    // new table/tree might have changed the number of cell states;
-    // if there are fewer states then pattern might change
-    int newmaxstate = currlayer->algo->NumCellStates() - 1;
-    if (newmaxstate < oldmaxstate && !currlayer->algo->isEmpty()) {
-        ReduceCellStates(newmaxstate);
+		// new table/tree might have changed the number of cell states;
+		// if there are fewer states then pattern might change
+		if (newmaxstate < oldmaxstate && !currlayer->algo->isEmpty()) {
+			ReduceCellStates(newmaxstate);
+		}
+
+        if (allowundo && !currlayer->stayclean) {
+            currlayer->undoredo->RememberRuleChange(oldrule.c_str());
+        }
     }
 
     // set colors and icons for new rule
     UpdateLayerColors();
 
-    // pattern might have changed or colors and/or icons might have changed
+    // pattern or colors or icons might have changed
     UpdateEverything();
-
-    if (oldrule != newrule) {
-        if (allowundo && !currlayer->stayclean) {
-            currlayer->undoredo->RememberRuleChange(oldrule.c_str());
-        }
-    }
 }
 
 // -----------------------------------------------------------------------------
@@ -1567,14 +973,14 @@ void ChangeAlgorithm(algo_type newalgotype, const char* newrule, bool inundoredo
     UpdateLayerColors();
 
     if (!inundoredo) {
-        if (rulechanged) {
-            // if pattern exists and is at starting gen then set savestart true
-            // so that SaveStartingPattern will save pattern to suitable file
-            // (and thus ResetPattern will work correctly)
-            if ( currlayer->algo->getGeneration() == currlayer->startgen && !currlayer->algo->isEmpty() ) {
-                currlayer->savestart = true;
-            }
+        // if pattern exists and is at starting gen then set savestart true
+        // so that SaveStartingPattern will save pattern to suitable file
+        // (and thus ResetPattern will work correctly)
+        if (currlayer->algo->getGeneration() == currlayer->startgen && !currlayer->algo->isEmpty()) {
+            currlayer->savestart = true;
+        }
 
+        if (rulechanged) {
             if (newrule[0] == 0) {
                 if (patternchanged) {
                     ErrorMessage("Rule has changed and pattern has changed.");
