@@ -49,6 +49,7 @@
 #include "lifealgo.h"
 #include "qlifealgo.h"
 #include "hlifealgo.h"
+#include "ruleloaderalgo.h" // for noTABLEorTREE
 #include "readpattern.h"    // for readpattern
 #include "writepattern.h"   // for writepattern, pattern_format
 
@@ -353,18 +354,44 @@ void LoadRule(const std::string& rulestring)
     // selection might change if grid becomes smaller,
     // so save current selection for RememberRuleChange/RememberAlgoChange
     SaveCurrentSelection();
-
-    const char* err = currlayer->algo->setrule(rulestring.c_str());
+    
+    // InitAlgorithms ensures the RuleLoader algo is the last algo
+    int rule_loader_algo = NumAlgos() - 1;
+    
+    const char* err;
+    if (currlayer->algtype == rule_loader_algo) {
+        // RuleLoader is current algo so no need to switch
+        err = currlayer->algo->setrule(rulestring.c_str());
+    } else {
+        // switch to RuleLoader algo
+        lifealgo* tempalgo = CreateNewUniverse(rule_loader_algo);
+        err = tempalgo->setrule(rulestring.c_str());
+        delete tempalgo;
+        if (!err) {
+            // change the current algorithm and switch to the new rule
+            ChangeAlgorithm(rule_loader_algo, rulestring.c_str());
+            if (rule_loader_algo != currlayer->algtype) {
+                RestoreRule(oldrule.c_str());
+                Warning("Algorithm could not be changed (pattern is too big to convert).");
+            }
+            return;
+        }
+    }
+    
     if (err) {
-        // try to find another algorithm that supports the given rule
-        for (int i = 0; i < NumAlgos(); i++) {
-            if (i != currlayer->algtype) {
+        // RuleLoader algo found some sort of error
+        if (strcmp(err, noTABLEorTREE) == 0) {
+            // .rule file has no TABLE or TREE section but it might be used
+            // to override a built-in rule, so try each algo
+            std::string temprule = rulestring;
+            std::replace(temprule.begin(), temprule.end(), '_', '/');   // eg. convert B3_S23 to B3/S23
+            for (int i = 0; i < NumAlgos(); i++) {
                 lifealgo* tempalgo = CreateNewUniverse(i);
-                err = tempalgo->setrule(rulestring.c_str());
+                err = tempalgo->setrule(temprule.c_str());
                 delete tempalgo;
                 if (!err) {
                     // change the current algorithm and switch to the new rule
-                    ChangeAlgorithm(i, rulestring.c_str());
+                    ChangeAlgorithm(i, temprule.c_str());
                     if (i != currlayer->algtype) {
                         RestoreRule(oldrule.c_str());
                         Warning("Algorithm could not be changed (pattern is too big to convert).");
@@ -373,36 +400,45 @@ void LoadRule(const std::string& rulestring)
                 }
             }
         }
-        // should only get here if rule/table/tree file contains some sort of error
+        
         RestoreRule(oldrule.c_str());
-        std::string msg = "Rule is not valid in any algorithm: " + rulestring;
+        std::string msg = "The rule file is not valid:\n";
+        msg += rulestring;
+        msg += "\n\nThe error message:\n";
+        msg += err;
         Warning(msg.c_str());
         return;
     }
 
     std::string newrule = currlayer->algo->getrule();
-    if (oldrule != newrule) {
+    int newmaxstate = currlayer->algo->NumCellStates() - 1;
+    if (oldrule != newrule || oldmaxstate != newmaxstate) {
+    	
+		// if pattern exists and is at starting gen then ensure savestart is true
+		// so that SaveStartingPattern will save pattern to suitable file
+		// (and thus undo/reset will work correctly)
+		if (currlayer->algo->getGeneration() == currlayer->startgen && !currlayer->algo->isEmpty()) {
+			currlayer->savestart = true;
+		}
+
         // if grid is bounded then remove any live cells outside grid edges
         if (currlayer->algo->gridwd > 0 || currlayer->algo->gridht > 0) {
             ClearOutsideGrid();
         }
-    }
 
-    // new rule might have changed the number of cell states;
-    // if there are fewer states then pattern might change
-    int newmaxstate = currlayer->algo->NumCellStates() - 1;
-    if (newmaxstate < oldmaxstate && !currlayer->algo->isEmpty()) {
-        ReduceCellStates(newmaxstate);
-    }
-
-    // set colors for new rule (loads any .rule file)
-    UpdateLayerColors();
-
-    if (oldrule != newrule) {
+		// new rule might have changed the number of cell states;
+		// if there are fewer states then pattern might change
+		if (newmaxstate < oldmaxstate && !currlayer->algo->isEmpty()) {
+			ReduceCellStates(newmaxstate);
+		}
+		
         if (allowundo && !currlayer->stayclean) {
             currlayer->undoredo->RememberRuleChange(oldrule.c_str());
         }
     }
+
+    // set colors for new rule
+    UpdateLayerColors();
 }
 
 // -----------------------------------------------------------------------------
