@@ -68,7 +68,6 @@ public:
 
     // each layer (cloned or not) has its own viewport for displaying patterns;
     // note that we use a pointer to the viewport to allow temporary switching
-    // in wxrender.cpp (eg. in DrawStackedLayers)
     viewport* view;
 
     // WARNING: this string is used to remember the current rule when
@@ -89,7 +88,6 @@ public:
     algo_type startalgo;          // starting algorithm
     bool savestart;               // need to save starting pattern?
     bool startdirty;              // starting state of dirty flag
-    std::string startfile;        // file for saving starting pattern
     std::string startname;        // starting currname
     std::string startrule;        // starting rule
     bigint startgen;              // starting generation (>= 0)
@@ -119,18 +117,18 @@ public:
     gBitmapPtr* icons15x15;       // icons for scale 1:16
     gBitmapPtr* icons31x31;       // icons for scale 1:32
 
-    // texture data for rendering icons
-    unsigned char** textures7x7;        // texture data for 7x7 icons
-    unsigned char** textures15x15;      // texture data for 15x15 icons
-    unsigned char** textures31x31;      // texture data for 31x31 icons
-
+    // texture atlases for rendering icons
+    unsigned char* atlas7x7;      // atlas for 7x7 icons
+    unsigned char* atlas15x15;    // atlas for 15x15 icons
+    unsigned char* atlas31x31;    // atlas for 31x31 icons
+    
+    int numicons;                 // number of icons (= number of live states)
     bool multicoloricons;         // are icons multi-colored? (grayscale if not)
 
-    // used if the layer has a timeline (see wxtimeline.cpp)
+    // used if the layer has a timeline
     int currframe;                // current frame in timeline
     int autoplay;                 // +ve = play forwards, -ve = play backwards, 0 = stop
     int tlspeed;                  // controls speed at which frames are played
-    long lastframe;               // time (in msecs) when last frame was displayed
 };
 
 const int MAX_LAYERS = 10;      // maximum number of layers
@@ -231,12 +229,20 @@ bool RestoreRule(const char* rule);
 // The latter can happen if the given rule's table/tree file was
 // deleted or was edited and some sort of error introduced.
 
-//??? move following color stuff into colors.*
+Layer* CreateTemporaryLayer();
+// Create a temporary layer with the same algo type as currlayer.
+
+
+// Color handling routines:
 
 void CreateColorGradient();
 // Create a color gradient for the current layer using
 // currlayer->fromrgb and currlayer->torgb.
 
+void UpdateIconColors();
+// Update the icon texture atlases for the current layer.
+// Must be called BEFORE calling UpdateCloneColors.
+
 void UpdateCloneColors();
 // If current layer has clones then update their colors.
 
@@ -247,7 +253,10 @@ void UpdateLayerColors();
 
 void InvertCellColors();
 // Invert the cell colors in all layers, including the dead cell color,
-// grid lines, and the colors in all icon bitmaps.
+// grid lines, and the colors in all icons.
+
+void InvertIconColors(unsigned char* atlasptr, int iconsize, int numicons);
+// Invert the icon colors in the given texture atlas.
 
 void SetLayerColors();
 // Open a dialog to change the current layer's colors.
