@@ -39,7 +39,153 @@
 // -----------------------------------------------------------------------------
 
 const char* lack_of_memory = "Due to lack of memory, some changes can't be undone!";
-const char* temp_prefix = "golly_undo_";
+
+// the following prefixes are used when creating temporary file names
+const char* genchange_prefix = "gg_";
+const char* setgen_prefix = "gs_";
+const char* dupe1_prefix = "g1_";
+const char* dupe2_prefix = "g2_";
+const char* dupe3_prefix = "g3_";
+const char* dupe4_prefix = "g4_";
+const char* dupe5_prefix = "g5_";
+const char* dupe6_prefix = "g6_";
+
+// -----------------------------------------------------------------------------
+
+// the next two classes are needed because Golly allows multiple starting points
+// (by setting the generation count back to 0), so we need to ensure that a Reset
+// goes back to the correct starting info
+
+class VariableInfo {
+public:
+    VariableInfo() {}
+    ~VariableInfo() {}
+    // note that we have to remember pointer to layer and not its index
+    // (the latter can change if user adds/deletes/moves a layer)
+    Layer* layerptr;
+    std::string savename;
+    bigint savex;
+    bigint savey;
+    int savemag;
+    int savebase;
+    int saveexpo;
+};
+
+class StartingInfo {
+public:
+    StartingInfo(StartingInfo* dupe, Layer* oldlayer, Layer* newlayer);
+    ~StartingInfo();
+    void Restore();
+    void RemoveClone(Layer* cloneptr);
+
+    // this info is the same in each clone
+    bool savedirty;
+    algo_type savealgo;
+    std::string saverule;
+    
+    // this info can be different in each clone
+    VariableInfo layer[MAX_LAYERS];
+    int count;
+};
+
+// -----------------------------------------------------------------------------
+
+StartingInfo::StartingInfo(StartingInfo* dupe, Layer* oldlayer, Layer* newlayer)
+{
+    if (dupe) {
+        // called from DuplicateHistory so duplicate given starting info
+        savedirty = dupe->savedirty;
+        savealgo = dupe->savealgo;
+        saverule = dupe->saverule;
+        
+        // new duplicate layer is not a clone
+        count = 0;
+        for (int n = 0; n < dupe->count; n++) {
+            if (dupe->layer[n].layerptr == oldlayer) {
+                layer[0].layerptr = newlayer;
+                layer[0].savename = dupe->layer[n].savename;
+                layer[0].savex = dupe->layer[n].savex;
+                layer[0].savey = dupe->layer[n].savey;
+                layer[0].savemag = dupe->layer[n].savemag;
+                layer[0].savebase = dupe->layer[n].savebase;
+                layer[0].saveexpo = dupe->layer[n].saveexpo;
+                count = 1;
+                break;
+            }
+        }
+
+    } else {
+        // save current starting info (set by most recent SaveStartingPattern)
+        savedirty = currlayer->startdirty;
+        savealgo = currlayer->startalgo;
+        saverule = currlayer->startrule;
+        
+        // save variable info for currlayer and its clones (if any)
+        count = 0;
+        for (int i = 0; i < numlayers; i++) {
+            Layer* lptr = GetLayer(i);
+            if (lptr == currlayer ||
+                (lptr->cloneid > 0 && lptr->cloneid == currlayer->cloneid)) {
+                layer[count].layerptr = lptr;
+                layer[count].savename = lptr->startname;
+                layer[count].savex = lptr->startx;
+                layer[count].savey = lptr->starty;
+                layer[count].savemag = lptr->startmag;
+                layer[count].savebase = lptr->startbase;
+                layer[count].saveexpo = lptr->startexpo;
+                count++;
+            }
+        }
+        if (count == 0) Warning("Bug detected in StartingInfo ctor!");
+    }
+}
+
+// -----------------------------------------------------------------------------
+
+StartingInfo::~StartingInfo()
+{
+    // no need to do anything
+}
+
+// -----------------------------------------------------------------------------
+
+void StartingInfo::Restore()
+{
+    // restore starting info (for use by next ResetPattern)
+    currlayer->startdirty = savedirty;
+    currlayer->startalgo = savealgo;
+    currlayer->startrule = saverule;
+    
+    // restore variable info for currlayer and its clones (if any);
+    // note that currlayer might have changed since the starting info
+    // was saved, and there might be more or fewer clones
+    for (int i = 0; i < numlayers; i++) {
+        Layer* lptr = GetLayer(i);
+        for (int n = 0; n < count; n++) {
+            if (lptr == layer[n].layerptr) {
+                lptr->startname = layer[n].savename;
+                lptr->startx = layer[n].savex;
+                lptr->starty = layer[n].savey;
+                lptr->startmag = layer[n].savemag;
+                lptr->startbase = layer[n].savebase;
+                lptr->startexpo = layer[n].saveexpo;
+                break;
+            }
+        }
+    }
+}
+
+// -----------------------------------------------------------------------------
+
+void StartingInfo::RemoveClone(Layer* cloneptr)
+{
+    for (int n = 0; n < count; n++) {
+        if (layer[n].layerptr == cloneptr) {
+            layer[n].layerptr = NULL;
+            return;
+        }
+    }
+}
 
 // -----------------------------------------------------------------------------
 
@@ -80,42 +226,35 @@ public:
     void ChangeCells(bool undo);
     // change cell states using cellinfo
 
-    change_type changeid;                  // specifies the type of change
-    bool olddirty;                         // layer's dirty state before change
-    bool newdirty;                         // layer's dirty state after change
+    change_type changeid;                   // specifies the type of change
+    bool olddirty;                          // layer's dirty state before change
+    bool newdirty;                          // layer's dirty state after change
 
     // cellstates info
-    cell_change* cellinfo;                 // dynamic array of cell changes
-    unsigned int cellcount;                // number of cell changes in array
+    cell_change* cellinfo;                  // dynamic array of cell changes
+    unsigned int cellcount;                 // number of cell changes in array
 
     // rotatecw/rotateacw/selchange info
-    Selection oldsel, newsel;              // old and new selections
+    Selection oldsel, newsel;               // old and new selections
 
     // genchange info
-    std::string oldfile, newfile;          // old and new pattern files
-    bigint oldgen, newgen;                 // old and new generation counts
-    bigint oldx, oldy, newx, newy;         // old and new positions
-    int oldmag, newmag;                    // old and new scales
-    int oldbase, newbase;                  // old and new base steps
-    int oldexpo, newexpo;                  // old and new step exponents
-    bool scriptgen;                        // gen change was done by script?
+    bool scriptgen;                         // gen change was done by script?
+    std::string oldfile, newfile;           // old and new pattern files
+    bigint oldgen, newgen;                  // old and new generation counts
+    bigint oldx, oldy, newx, newy;          // old and new positions
+    int oldmag, newmag;                     // old and new scales
+    int oldbase, newbase;                   // old and new base steps
+    int oldexpo, newexpo;                   // old and new step exponents
+    StartingInfo* startinfo;                // saves starting info for ResetPattern
     // also uses oldsel, newsel
 
     // setgen info
     bigint oldstartgen, newstartgen;            // old and new startgen values
     bool oldsave, newsave;                      // old and new savestart states
     std::string oldtempstart, newtempstart;     // old and new tempstart paths
-    std::string oldstartfile, newstartfile;     // old and new startfile paths
     std::string oldcurrfile, newcurrfile;       // old and new currfile paths
-    std::string oldclone[MAX_LAYERS];           // old starting names for cloned layers
-    std::string newclone[MAX_LAYERS];           // new starting names for cloned layers
     // also uses oldgen, newgen
-    // and oldrule, newrule
-    // and oldx, oldy, newx, newy, oldmag, newmag
-    // and oldbase, newbase
-    // and oldexpo, newexpo
-    // and oldsel, newsel
-    // and oldalgo, newalgo
+    // and startinfo
 
     // namechange info
     std::string oldname, newname;           // old and new layer names
@@ -138,38 +277,61 @@ public:
 ChangeNode::ChangeNode(change_type id)
 {
     changeid = id;
+    startinfo = NULL;
+    whichlayer = NULL;      // simplifies UndoRedo::DeletingClone
     cellinfo = NULL;
     cellcount = 0;
     oldfile.clear();
     newfile.clear();
     oldtempstart.clear();
     newtempstart.clear();
+    oldcurrfile.clear();
+    newcurrfile.clear();
 }
 
 // -----------------------------------------------------------------------------
 
+bool delete_all_temps = false;   // ok to delete all temporary files?
+
 ChangeNode::~ChangeNode()
 {
+    if (startinfo) delete startinfo;
     if (cellinfo) free(cellinfo);
+    
+    // it's always ok to delete oldfile and newfile if they exist
 
     if (!oldfile.empty() && FileExists(oldfile)) {
         RemoveFile(oldfile);
     }
+    
     if (!newfile.empty() && FileExists(newfile)) {
         RemoveFile(newfile);
     }
 
-    // only delete oldtempstart/newtempstart if they're not being used to
-    // store the current layer's starting pattern
-    if ( !oldtempstart.empty() && FileExists(oldtempstart) &&
-        oldtempstart != currlayer->startfile &&
-        oldtempstart != currlayer->currfile ) {
-        RemoveFile(oldtempstart);
-    }
-    if ( !newtempstart.empty() && FileExists(newtempstart) &&
-        newtempstart != currlayer->startfile &&
-        newtempstart != currlayer->currfile ) {
-        RemoveFile(newtempstart);
+    if (delete_all_temps) {
+        // we're in ClearUndoRedo so it's safe to delete oldtempstart/newtempstart/oldcurrfile/newcurrfile
+        // if they are in tempdir and not being used to store the current layer's starting pattern
+        // (the latter condition allows user to Reset after disabling undo/redo)
+        
+        if (!oldtempstart.empty() && FileExists(oldtempstart) &&
+            oldtempstart.compare(0,tempdir.length(),tempdir) == 0 && oldtempstart != currlayer->currfile) {
+            RemoveFile(oldtempstart);
+        }
+        
+        if (!newtempstart.empty() && FileExists(newtempstart) &&
+            newtempstart.compare(0,tempdir.length(),tempdir) == 0 && newtempstart != currlayer->currfile) {
+            RemoveFile(newtempstart);
+        }
+        
+        if (!oldcurrfile.empty() && FileExists(oldcurrfile) &&
+            oldcurrfile.compare(0,tempdir.length(),tempdir) == 0 && oldcurrfile != currlayer->currfile) {
+            RemoveFile(oldcurrfile);
+        }
+        
+        if (!newcurrfile.empty() && FileExists(newcurrfile) &&
+            newcurrfile.compare(0,tempdir.length(),tempdir) == 0 && newcurrfile != currlayer->currfile) {
+            RemoveFile(newcurrfile);
+        }
     }
 }
 
@@ -239,10 +401,15 @@ bool ChangeNode::DoChange(bool undo)
             break;
 
         case genchange:
+            currlayer->currfile = oldcurrfile;
             if (undo) {
                 currlayer->currsel = oldsel;
                 RestorePattern(oldgen, oldfile.c_str(), oldx, oldy, oldmag, oldbase, oldexpo);
             } else {
+                if (startinfo) {
+                    // restore starting info for use by ResetPattern
+                    startinfo->Restore();
+                }
                 currlayer->currsel = newsel;
                 RestorePattern(newgen, newfile.c_str(), newx, newy, newmag, newbase, newexpo);
             }
@@ -254,55 +421,17 @@ bool ChangeNode::DoChange(bool undo)
                 currlayer->startgen = oldstartgen;
                 currlayer->savestart = oldsave;
                 currlayer->tempstart = oldtempstart;
-                currlayer->startfile = oldstartfile;
                 currlayer->currfile = oldcurrfile;
-                if (oldtempstart != newtempstart) {
-                    currlayer->startdirty = olddirty;
-                    currlayer->startalgo = oldalgo;
-                    currlayer->startrule = oldrule;
-                    currlayer->startx = oldx;
-                    currlayer->starty = oldy;
-                    currlayer->startmag = oldmag;
-                    currlayer->startbase = oldbase;
-                    currlayer->startexpo = oldexpo;
-                    currlayer->startsel = oldsel;
-                    currlayer->startname = oldname;
-                    if (currlayer->cloneid > 0) {
-                        for ( int i = 0; i < numlayers; i++ ) {
-                            Layer* cloneptr = GetLayer(i);
-                            if (cloneptr != currlayer && cloneptr->cloneid == currlayer->cloneid) {
-                                cloneptr->startname = oldclone[i];
-                            }
-                        }
-                    }
+                if (startinfo) {
+                    // restore starting info for use by ResetPattern
+                    startinfo->Restore();
                 }
             } else {
                 ChangeGenCount(newgen.tostring(), true);
                 currlayer->startgen = newstartgen;
                 currlayer->savestart = newsave;
                 currlayer->tempstart = newtempstart;
-                currlayer->startfile = newstartfile;
                 currlayer->currfile = newcurrfile;
-                if (oldtempstart != newtempstart) {
-                    currlayer->startdirty = newdirty;
-                    currlayer->startalgo = newalgo;
-                    currlayer->startrule = newrule;
-                    currlayer->startx = newx;
-                    currlayer->starty = newy;
-                    currlayer->startmag = newmag;
-                    currlayer->startbase = newbase;
-                    currlayer->startexpo = newexpo;
-                    currlayer->startsel = newsel;
-                    currlayer->startname = newname;
-                    if (currlayer->cloneid > 0) {
-                        for ( int i = 0; i < numlayers; i++ ) {
-                            Layer* cloneptr = GetLayer(i);
-                            if (cloneptr != currlayer && cloneptr->cloneid == currlayer->cloneid) {
-                                cloneptr->startname = newclone[i];
-                            }
-                        }
-                    }
-                }
             }
             break;
 
@@ -325,9 +454,6 @@ bool ChangeNode::DoChange(bool undo)
                     if (olddirty == newdirty) SetPatternTitle(currlayer->currname.c_str());
                     // if olddirty != newdirty then UndoChange/RedoChange will call
                     // MarkLayerClean/MarkLayerDirty (they call SetPatternTitle)
-                } else {
-                    // whichlayer is non-active clone so only update Layer menu items
-                    //!!! for (int i = 0; i < numlayers; i++) UpdateLayerItem(i);
                 }
             }
             break;
@@ -384,7 +510,6 @@ UndoRedo::UndoRedo()
     doingscriptchanges = false;   // not undoing/redoing script changes
     prevfile.clear();             // play safe for ClearUndoRedo
     startcount = 0;               // unfinished RememberGenStart calls
-    fixsetgen = false;            // no setgen nodes need fixing
 
     // need to remember if script has created a new layer (not a clone)
     if (inscript) RememberScriptStart();
@@ -610,7 +735,6 @@ void UndoRedo::RememberSelection(const char* action)
 void UndoRedo::SaveCurrentPattern(const char* tempfile)
 {
     const char* err = NULL;
-    //!!! need lifealgo::CanWriteFormat(MC_format) method???
     if ( currlayer->algo->hyperCapable() ) {
         // save hlife pattern in a macrocell file
         err = WritePattern(tempfile, MC_format, no_compression, 0, 0, 0, 0);
@@ -660,46 +784,9 @@ void UndoRedo::RememberGenStart()
     if (prevgen == currlayer->startgen) {
         // we can just reset to starting pattern
         prevfile.clear();
-
-        if (fixsetgen) {
-            // SaveStartingPattern has just been called so search undolist for setgen
-            // node that changed tempstart and update the starting info in that node;
-            // yuk -- is there a simpler solution???
-            std::list<ChangeNode*>::iterator node = undolist.begin();
-            while (node != undolist.end()) {
-                ChangeNode* change = *node;
-                if (change->changeid == setgen &&
-                    change->oldtempstart != change->newtempstart) {
-                    change->newdirty = currlayer->startdirty;
-                    change->newalgo = currlayer->startalgo;
-                    change->newrule = currlayer->startrule;
-                    change->newx = currlayer->startx;
-                    change->newy = currlayer->starty;
-                    change->newmag = currlayer->startmag;
-                    change->newbase = currlayer->startbase;
-                    change->newexpo = currlayer->startexpo;
-                    change->newsel = currlayer->startsel;
-                    change->newname = currlayer->startname;
-                    if (currlayer->cloneid > 0) {
-                        for ( int i = 0; i < numlayers; i++ ) {
-                            Layer* cloneptr = GetLayer(i);
-                            if (cloneptr != currlayer && cloneptr->cloneid == currlayer->cloneid) {
-                                change->newclone[i] = cloneptr->startname;
-                            }
-                        }
-                    }
-                    // do NOT reset fixsetgen to false here; the gen change might
-                    // be removed when clearing the redo list and so we may need
-                    // to update this setgen node again after a new gen change
-                    break;
-                }
-                node++;
-            }
-        }
-
     } else {
         // save starting pattern in a unique temporary file
-        prevfile = CreateTempFileName(temp_prefix);
+        prevfile = CreateTempFileName(genchange_prefix);
 
         // if head of undo list is a genchange node then we can copy that
         // change node's newfile to prevfile; this makes consecutive generating
@@ -754,7 +841,7 @@ void UndoRedo::RememberGenFinish()
         fpath.clear();
     } else {
         // save finishing pattern in a unique temporary file
-        fpath = CreateTempFileName(temp_prefix);
+        fpath = CreateTempFileName(genchange_prefix);
         SaveCurrentPattern(fpath.c_str());
     }
 
@@ -782,6 +869,16 @@ void UndoRedo::RememberGenFinish()
     change->oldsel = prevsel;
     change->newsel = currlayer->currsel;
 
+    // also remember the file containing the starting pattern
+    // (in case it is changed by by RememberSetGen or RememberNameChange)
+    change->oldcurrfile = currlayer->currfile;
+
+    if (change->oldgen == currlayer->startgen) {
+        // save starting info set by recent SaveStartingPattern call
+        // (the info will be restored when redoing this genchange node)
+        change->startinfo = new StartingInfo(NULL, NULL, NULL);
+    }
+
     // prevfile has been saved in change->oldfile (~ChangeNode will delete it)
     prevfile.clear();
 
@@ -837,13 +934,17 @@ void UndoRedo::SyncUndoHistory()
         if (change->changeid == genchange && change->oldgen == currlayer->startgen) {
             if (change->scriptgen) {
                 // gen change was done by a script so keep winding back the undo list
-                // to just past the scriptstart node, or until the list is empty
+                // until the scriptstart node, or until the list is empty
                 while (!undolist.empty()) {
                     node = undolist.begin();
                     change = *node;
-                    undolist.erase(node);
-                    redolist.push_front(change);
-                    if (change->changeid == scriptstart) break;
+                    if (change->changeid == scriptstart) {
+                        undolist.erase(node);
+                        redolist.push_front(change);
+                        break;
+                    }
+                    // undo this change so Reset and Undo restore to the same pattern
+                    UndoChange();
                 }
             }
             return;
@@ -859,16 +960,14 @@ void UndoRedo::RememberSetGen(bigint& oldgen, bigint& newgen,
                               bigint& oldstartgen, bool oldsave)
 {
     std::string oldtempstart = currlayer->tempstart;
-    std::string oldstartfile = currlayer->startfile;
     std::string oldcurrfile = currlayer->currfile;
     if (oldgen > oldstartgen && newgen <= oldstartgen) {
         // if pattern is generated then tempstart will be clobbered by
         // SaveStartingPattern, so change tempstart to a new temporary file
-        currlayer->tempstart = CreateTempFileName("golly_setgen_");
+        currlayer->tempstart = CreateTempFileName(setgen_prefix);
 
-        // also need to update startfile and currfile (currlayer->savestart is true)
-        currlayer->startfile = currlayer->tempstart;
-        currlayer->currfile.clear();
+        // also need to update currfile (currlayer->savestart is true)
+        currlayer->currfile = currlayer->tempstart;
     }
 
     ClearRedoHistory();
@@ -885,54 +984,13 @@ void UndoRedo::RememberSetGen(bigint& oldgen, bigint& newgen,
     change->newsave = currlayer->savestart;
     change->oldtempstart = oldtempstart;
     change->newtempstart = currlayer->tempstart;
-    change->oldstartfile = oldstartfile;
-    change->newstartfile = currlayer->startfile;
     change->oldcurrfile = oldcurrfile;
     change->newcurrfile = currlayer->currfile;
 
-    if (change->oldtempstart != change->newtempstart) {
-        // save extra starting info set by previous SaveStartingPattern so that
-        // Undoing this setgen change will restore the correct info for a Reset
-        change->olddirty = currlayer->startdirty;
-        change->oldalgo = currlayer->startalgo;
-        change->oldrule = currlayer->startrule;
-        change->oldx = currlayer->startx;
-        change->oldy = currlayer->starty;
-        change->oldmag = currlayer->startmag;
-        change->oldbase = currlayer->startbase;
-        change->oldexpo = currlayer->startexpo;
-        change->oldsel = currlayer->startsel;
-        change->oldname = currlayer->startname;
-        if (currlayer->cloneid > 0) {
-            for ( int i = 0; i < numlayers; i++ ) {
-                Layer* cloneptr = GetLayer(i);
-                if (cloneptr != currlayer && cloneptr->cloneid == currlayer->cloneid) {
-                    change->oldclone[i] = cloneptr->startname;
-                }
-            }
-        }
-
-        // following settings will be updated by next RememberGenStart call so that
-        // Redoing this setgen change will restore the correct info for a Reset
-        fixsetgen = true;
-        change->newdirty = currlayer->startdirty;
-        change->newalgo = currlayer->startalgo;
-        change->newrule = currlayer->startrule;
-        change->newx = currlayer->startx;
-        change->newy = currlayer->starty;
-        change->newmag = currlayer->startmag;
-        change->newbase = currlayer->startbase;
-        change->newexpo = currlayer->startexpo;
-        change->newsel = currlayer->startsel;
-        change->newname = currlayer->startname;
-        if (currlayer->cloneid > 0) {
-            for ( int i = 0; i < numlayers; i++ ) {
-                Layer* cloneptr = GetLayer(i);
-                if (cloneptr != currlayer && cloneptr->cloneid == currlayer->cloneid) {
-                    change->newclone[i] = cloneptr->startname;
-                }
-            }
-        }
+    if (oldtempstart != currlayer->tempstart) {
+        // save starting info set by most recent SaveStartingPattern call
+        // (the info will be restored when undoing this setgen node)
+        change->startinfo = new StartingInfo(NULL, NULL, NULL);
     }
 
     undolist.push_front(change);
@@ -973,10 +1031,9 @@ void UndoRedo::RememberNameChange(const char* oldname, const char* oldcurrfile,
 void UndoRedo::DeletingClone(int index)
 {
     // the given cloned layer is about to be deleted, so we need to go thru the
-    // undo/redo lists and, for each namechange node, set a matching whichlayer
-    // ptr to NULL so DoChange can ignore later changes involving this layer;
-    // very ugly, but I don't see any better solution if we're going to allow
-    // cloned layers to have different names
+    // undo/redo lists and fix up any nodes that have pointers to that clone
+    // (very ugly, but I don't see any better solution if we're going to allow
+    // cloned layers to have different names)
 
     Layer* cloneptr = GetLayer(index);
     std::list<ChangeNode*>::iterator node;
@@ -984,16 +1041,24 @@ void UndoRedo::DeletingClone(int index)
     node = undolist.begin();
     while (node != undolist.end()) {
         ChangeNode* change = *node;
-        if (change->changeid == namechange && change->whichlayer == cloneptr)
+        if (change->whichlayer == cloneptr) {
             change->whichlayer = NULL;
+        }
+        if (change->startinfo) {
+            change->startinfo->RemoveClone(cloneptr);
+        }
         node++;
     }
 
     node = redolist.begin();
     while (node != redolist.end()) {
         ChangeNode* change = *node;
-        if (change->changeid == namechange && change->whichlayer == cloneptr)
+        if (change->whichlayer == cloneptr) {
             change->whichlayer = NULL;
+        }
+        if (change->startinfo) {
+            change->startinfo->RemoveClone(cloneptr);
+        }
         node++;
     }
 }
@@ -1277,12 +1342,15 @@ void UndoRedo::ClearUndoRedo()
         prevfile.clear();
         startcount = 0;
     }
+    
+    // set flag so ChangeNode::~ChangeNode() can delete all temporary files
+    delete_all_temps = true;
 
     // clear the undo/redo lists (and delete each node's data)
     ClearUndoHistory();
     ClearRedoHistory();
-
-    fixsetgen = false;
+    
+    delete_all_temps = false;
 
     if (inscript) {
         // script has called a command like new() so add a scriptstart node
@@ -1298,52 +1366,72 @@ void UndoRedo::ClearUndoRedo()
 
 bool CopyTempFiles(ChangeNode* srcnode, ChangeNode* destnode, const char* tempstart1)
 {
-    // if srcnode has any existing temporary files then create new
-    // temporary file names in the destnode and copy each file
+    // if srcnode has any existing temporary files then, if necessary, create new
+    // temporary file names in the destnode and copy such files
     bool allcopied = true;
 
     if ( !srcnode->oldfile.empty() && FileExists(srcnode->oldfile) ) {
-        destnode->oldfile = CreateTempFileName("golly_dupe1_");
+        destnode->oldfile = CreateTempFileName(dupe1_prefix);
         if ( !CopyFile(srcnode->oldfile, destnode->oldfile) )
             allcopied = false;
     }
-
+    
     if ( !srcnode->newfile.empty() && FileExists(srcnode->newfile) ) {
-        destnode->newfile = CreateTempFileName("golly_dupe2_");
+        destnode->newfile = CreateTempFileName(dupe2_prefix);
         if ( !CopyFile(srcnode->newfile, destnode->newfile) )
             allcopied = false;
     }
-
+    
+    if ( !srcnode->oldcurrfile.empty() && FileExists(srcnode->oldcurrfile) ) {
+        if (srcnode->oldcurrfile == currlayer->tempstart) {
+            // the file has already been copied to tempstart1 by Layer::Layer()
+            destnode->oldcurrfile = tempstart1;
+        } else if (srcnode->oldcurrfile.compare(0,tempdir.length(),tempdir) == 0) {
+            destnode->oldcurrfile = CreateTempFileName(dupe3_prefix);
+            if ( !CopyFile(srcnode->oldcurrfile, destnode->oldcurrfile) )
+                allcopied = false;
+        }
+    }
+    
+    if ( !srcnode->newcurrfile.empty() && FileExists(srcnode->newcurrfile) ) {
+        if (srcnode->newcurrfile == srcnode->oldcurrfile) {
+            // use destnode->oldcurrfile set above or earlier in DuplicateHistory
+            destnode->newcurrfile = destnode->oldcurrfile;
+        } else if (srcnode->newcurrfile == currlayer->tempstart) {
+            // the file has already been copied to tempstart1 by Layer::Layer()
+            destnode->newcurrfile = tempstart1;
+        } else if (srcnode->newcurrfile.compare(0,tempdir.length(),tempdir) == 0) {
+            destnode->newcurrfile = CreateTempFileName(dupe4_prefix);
+            if ( !CopyFile(srcnode->newcurrfile, destnode->newcurrfile) )
+                allcopied = false;
+        }
+    }
+    
     if ( !srcnode->oldtempstart.empty() && FileExists(srcnode->oldtempstart) ) {
         if (srcnode->oldtempstart == currlayer->tempstart) {
             // the file has already been copied to tempstart1 by Layer::Layer()
             destnode->oldtempstart = tempstart1;
-        } else {
-            destnode->oldtempstart = CreateTempFileName("golly_dupe3_");
+        } else if (srcnode->oldtempstart.compare(0,tempdir.length(),tempdir) == 0) {
+            destnode->oldtempstart = CreateTempFileName(dupe5_prefix);
             if ( !CopyFile(srcnode->oldtempstart, destnode->oldtempstart) )
                 allcopied = false;
         }
-        if (srcnode->oldstartfile == srcnode->oldtempstart)
-            destnode->oldstartfile = destnode->oldtempstart;
-        if (srcnode->oldcurrfile == srcnode->oldtempstart)
-            destnode->oldcurrfile = destnode->oldtempstart;
     }
-
+    
     if ( !srcnode->newtempstart.empty() && FileExists(srcnode->newtempstart) ) {
-        if (srcnode->newtempstart == currlayer->tempstart) {
+        if (srcnode->newtempstart == srcnode->oldtempstart) {
+            // use destnode->oldtempstart set above or earlier in DuplicateHistory
+            destnode->newtempstart = destnode->oldtempstart;
+        } else if (srcnode->newtempstart == currlayer->tempstart) {
             // the file has already been copied to tempstart1 by Layer::Layer()
             destnode->newtempstart = tempstart1;
-        } else {
-            destnode->newtempstart = CreateTempFileName("golly_dupe4_");
+        } else if (srcnode->newtempstart.compare(0,tempdir.length(),tempdir) == 0) {
+            destnode->newtempstart = CreateTempFileName(dupe6_prefix);
             if ( !CopyFile(srcnode->newtempstart, destnode->newtempstart) )
                 allcopied = false;
         }
-        if (srcnode->newstartfile == srcnode->newtempstart)
-            destnode->newstartfile = destnode->newtempstart;
-        if (srcnode->newcurrfile == srcnode->newtempstart)
-            destnode->newcurrfile = destnode->newtempstart;
     }
-
+    
     return allcopied;
 }
 
@@ -1375,11 +1463,10 @@ void UndoRedo::DuplicateHistory(Layer* oldlayer, Layer* newlayer)
     prevexpo = history->prevexpo;
     prevsel = history->prevsel;
     startcount = history->startcount;
-    fixsetgen = history->fixsetgen;
 
     // copy existing temporary file to new name
     if ( !prevfile.empty() && FileExists(prevfile) ) {
-        prevfile = CreateTempFileName(temp_prefix);
+        prevfile = CreateTempFileName(genchange_prefix);
         if ( !CopyFile(history->prevfile, prevfile) ) {
             Warning("Could not copy prevfile!");
             return;
@@ -1426,6 +1513,19 @@ void UndoRedo::DuplicateHistory(Layer* oldlayer, Layer* newlayer)
             }
             memcpy(newchange->cellinfo, change->cellinfo, bytes);
         }
+        
+        if (change->startinfo) {
+            newchange->startinfo = new StartingInfo(change->startinfo, oldlayer, newlayer);
+        }
+
+        // if node is a name change then update whichlayer
+        if (newchange->changeid == namechange) {
+            if (change->whichlayer == oldlayer) {
+                newchange->whichlayer = newlayer;
+            } else {
+                newchange->whichlayer = NULL;
+            }
+        }
 
         // copy any existing temporary files to new names
         if (!CopyTempFiles(change, newchange, newlayer->tempstart.c_str())) {
@@ -1434,11 +1534,6 @@ void UndoRedo::DuplicateHistory(Layer* oldlayer, Layer* newlayer)
             return;
         }
 
-        // if node is a name change then update whichlayer to point to new layer
-        if (newchange->changeid == namechange) {
-            newchange->whichlayer = newlayer;
-        }
-
         undolist.push_back(newchange);
         node++;
     }
@@ -1469,6 +1564,19 @@ void UndoRedo::DuplicateHistory(Layer* oldlayer, Layer* newlayer)
             }
             memcpy(newchange->cellinfo, change->cellinfo, bytes);
         }
+        
+        if (change->startinfo) {
+            newchange->startinfo = new StartingInfo(change->startinfo, oldlayer, newlayer);
+        }
+
+        // if node is a name change then update whichlayer to point to new layer
+        if (newchange->changeid == namechange) {
+            if (change->whichlayer == oldlayer) {
+                newchange->whichlayer = newlayer;
+            } else {
+                newchange->whichlayer = NULL;
+            }
+        }
 
         // copy any existing temporary files to new names
         if (!CopyTempFiles(change, newchange, newlayer->tempstart.c_str())) {
@@ -1477,13 +1585,7 @@ void UndoRedo::DuplicateHistory(Layer* oldlayer, Layer* newlayer)
             return;
         }
 
-        // if node is a name change then update whichlayer to point to new layer
-        if (newchange->changeid == namechange) {
-            newchange->whichlayer = newlayer;
-        }
-
         redolist.push_back(newchange);
         node++;
     }
 }
-
