@@ -41,3 +41,9 @@ UIViewController* CurrentViewController();
 
 // enable/disable tab bar
 void EnableTabBar(bool enable);
+
+// show/hide tab bar
+void ShowTabBar(bool show);
+
+// get height of tab bar
+CGFloat TabBarHeight();
