@@ -27,7 +27,7 @@
 #include <set>          // for std::set
 #include <algorithm>    // for std::count
 
-#include "prefs.h"      // for gollydir, recentpatterns, SavePrefs, etc
+#include "prefs.h"      // for userdir, recentpatterns, SavePrefs, etc
 #include "utils.h"      // for Warning, RemoveFile, FixURLPath, MoveFile, etc
 #include "file.h"       // for OpenFile
 
@@ -101,7 +101,7 @@ static void AppendHtmlData(std::string& htmldata, const std::string& dir,
     htmldata += "<br><br>";
     
     while (path = [dirEnum nextObject]) {
-        // path is relative; eg. "Life/Bounded-Grids/agar-p3.rle"
+        // path is relative to given dir (eg. "Life/Bounded-Grids/agar-p3.rle" if patternsdir)
         std::string pstr = [path cStringUsingEncoding:NSUTF8StringEncoding];
 
         // indent level is number of separators in path
@@ -188,7 +188,7 @@ static void AppendHtmlData(std::string& htmldata, const std::string& dir,
 - (void)showSavedPatterns
 {
     std::string htmldata;
-    AppendHtmlData(htmldata, datadir, "Documents/Saved/", "Saved patterns:", true);
+    AppendHtmlData(htmldata, savedir, "Documents/Saved/", "Saved patterns:", true);
     [htmlView loadHTMLString:[NSString stringWithCString:htmldata.c_str() encoding:NSUTF8StringEncoding]
                      baseURL:nil];
 }
@@ -215,7 +215,7 @@ static void AppendHtmlData(std::string& htmldata, const std::string& dir,
         std::list<std::string>::iterator next = recentpatterns.begin();
         while (next != recentpatterns.end()) {
             std::string path = *next;
-            if (path.find("Patterns/") == 0 || FileExists(gollydir + path)) {
+            if (path.find("Patterns/") == 0 || FileExists(userdir + path)) {
                 htmldata += "<a href=\"open:";
                 htmldata += path;
                 htmldata += "\">";
@@ -426,7 +426,7 @@ static void AppendHtmlData(std::string& htmldata, const std::string& dir,
             std::string question = "Do you really want to delete " + path + "?";
             if (YesNo(question.c_str())) {
                 // delete specified file
-                path = gollydir + path;
+                path = userdir + path;
                 RemoveFile(path);
                 // save current location
                 curroffset[curroption] = htmlView.scrollView.contentOffset;
@@ -446,10 +446,10 @@ static void AppendHtmlData(std::string& htmldata, const std::string& dir,
             std::string fullpath = path;
             if (path[0] != '/') {
                 if (fullpath.find("Patterns/") == 0 || fullpath.find("Rules/") == 0) {
-                    // Patterns and Rules directories are inside app bundle so prepend gollydir/Golly.app/
-                    fullpath = gollydir + "Golly.app/" + fullpath;
+                    // Patterns and Rules directories are inside supplieddir
+                    fullpath = supplieddir + fullpath;
                 } else {
-                    fullpath = gollydir + fullpath;
+                    fullpath = userdir + fullpath;
                 }
             }
             ShowTextFile(fullpath.c_str());
@@ -466,10 +466,10 @@ static void AppendHtmlData(std::string& htmldata, const std::string& dir,
 void MoveSharedFiles()
 {
     // check for files in the Documents folder (created by iTunes file sharing)
-    // and move any .rule/tree/table/colors/icons files into Documents/Rules/,
+    // and move any .rule/tree/table files into Documents/Rules/,
     // otherwise assume they are pattern files and move them into Documents/Saved/
     
-    std::string docdir = gollydir + "Documents/";
+    std::string docdir = userdir + "Documents/";
     NSFileManager *fm = [NSFileManager defaultManager];
     NSString *dirpath = [NSString stringWithCString:docdir.c_str() encoding:NSUTF8StringEncoding];
 	NSArray* contents = [fm contentsOfDirectoryAtPath:dirpath error:nil];
@@ -482,17 +482,17 @@ void MoveSharedFiles()
         } else {
             // path is to a file
             std::string filename = [item cStringUsingEncoding:NSUTF8StringEncoding];
-            if (filename[0] == '.') {
-                // ignore hidden file
+            if (filename[0] == '.' || EndsWith(filename,".colors") || EndsWith(filename,".icons")) {
+                // ignore hidden file or .colors/icons file
             } else if (IsRuleFile(filename)) {
-                // move .rule/tree/table/colors/icons file into Documents/Rules/
+                // move .rule/tree/table file into Documents/Rules/
                 std::string oldpath = docdir + filename;
                 std::string newpath = userrules + filename;
                 MoveFile(oldpath, newpath);
             } else {
                 // assume this is a pattern file amd move it into Documents/Saved/
                 std::string oldpath = docdir + filename;
-                std::string newpath = datadir + filename;
+                std::string newpath = savedir + filename;
                 MoveFile(oldpath, newpath);
             }
         }
