@@ -66,96 +66,6 @@ static GLuint viewFramebuffer = 0;
 
 // -----------------------------------------------------------------------------
 
-- (id)initWithCoder:(NSCoder *)c
-{
-    self = [super initWithCoder:c];
-    if (self) {
-        
-        // init the OpenGL ES stuff (based on Apple's GLPaint sample code):
-        
-		CAEAGLLayer *eaglLayer = (CAEAGLLayer *)self.layer;
-		eaglLayer.opaque = YES;
-        
-        // note that we're using OpenGL ES version 1
-		context = [[EAGLContext alloc] initWithAPI:kEAGLRenderingAPIOpenGLES1];
-		if (!context || ![EAGLContext setCurrentContext:context]) {
-			self = nil;
-			return self;
-		}
-		
-		// set the view's scale factor
-		self.contentScaleFactor = 1.0;
-        
-        // we only do 2D drawing
-        glDisable(GL_DEPTH_TEST);
-        glDisable(GL_DITHER);
-        glDisable(GL_MULTISAMPLE);
-        glDisable(GL_STENCIL_TEST);
-        glDisable(GL_FOG);
-        
-        glEnableClientState(GL_TEXTURE_COORD_ARRAY);
-        glEnableClientState(GL_VERTEX_ARRAY);
-        
-        glEnable(GL_BLEND);
-        // this blending function seems similar to the one used in desktop Golly
-        // (ie. selected patterns look much the same)
-        glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
-        
-        // add gesture recognizers to this view:
-        
-        [self setMultipleTouchEnabled:YES];
-        
-        // pinch gestures will zoom in/out
-        UIPinchGestureRecognizer *pinchGesture = [[UIPinchGestureRecognizer alloc]
-                                                  initWithTarget:self action:@selector(zoomView:)];
-        pinchGesture.delegate = self;
-        [self addGestureRecognizer:pinchGesture];
-        
-        // one-finger pan gestures will be used to draw/pick/select/etc, depending on the current touch mode
-        UIPanGestureRecognizer *pan1Gesture = [[UIPanGestureRecognizer alloc]
-                                               initWithTarget:self action:@selector(singleDrag:)];
-        pan1Gesture.minimumNumberOfTouches = 1;
-        pan1Gesture.maximumNumberOfTouches = 1;
-        pan1Gesture.delegate = self;
-        [self addGestureRecognizer:pan1Gesture];
-        
-        // two-finger pan gestures will move the view
-        UIPanGestureRecognizer *pan2Gesture = [[UIPanGestureRecognizer alloc]
-                                               initWithTarget:self action:@selector(moveView:)];
-        pan2Gesture.minimumNumberOfTouches = 2;
-        pan2Gesture.maximumNumberOfTouches = 2;
-        pan2Gesture.delegate = self;
-        [self addGestureRecognizer:pan2Gesture];
-        
-        // single-taps will do various actions depending on the current touch mode
-        UITapGestureRecognizer *tap1Gesture = [[UITapGestureRecognizer alloc]
-                                               initWithTarget:self action:@selector(singleTap:)];
-        tap1Gesture.numberOfTapsRequired = 1;
-        tap1Gesture.numberOfTouchesRequired = 1;
-        tap1Gesture.delegate = self;
-        [self addGestureRecognizer:tap1Gesture];
-        
-        /* too many problems if we have gesture recognizers for single-taps and double-taps:
-        
-        // double-taps will do various actions depending on the location
-        UITapGestureRecognizer *tap2Gesture = [[UITapGestureRecognizer alloc]
-                                               initWithTarget:self action:@selector(doubleTap:)];
-        tap2Gesture.numberOfTapsRequired = 2;
-        tap2Gesture.numberOfTouchesRequired = 1;
-        tap2Gesture.delegate = self;
-        [self addGestureRecognizer:tap2Gesture];
-        
-        // only do a single-tap if double-tap is not detected
-        // (this works but the delay is way too long)
-        // [tap1Gesture requireGestureRecognizerToFail:tap2Gesture];
-        
-        */
-    }
-    return self;
-}
-
-// -----------------------------------------------------------------------------
-
 - (void)createFramebuffer
 {
 	// generate IDs for a framebuffer object and a color renderbuffer
@@ -197,9 +107,7 @@ static GLuint viewFramebuffer = 0;
     // NSLog(@"layoutSubviews: wd=%f ht=%f", self.bounds.size.width, self.bounds.size.height);
 
     [EAGLContext setCurrentContext:context];
-    
-    // note that glLoadIdentity must be called after glMatrixMode but before glOrthof,
-    // otherwise the display is blank
+
     glMatrixMode(GL_PROJECTION);
     glLoadIdentity();
     CGRect frame = self.bounds;
@@ -339,8 +247,8 @@ static int startx, starty;
         if (currlayer->touchmode == movemode) {
             // if velocity is high then move further in current direction
             CGPoint velocity = [gestureRecognizer velocityInView:self];
-            if (abs(velocity.x) > 1000.0 || abs(velocity.y) > 1000.0) {
-                TouchMoved(prevpt.x + delta.x * abs(velocity.x/100.0), prevpt.y + delta.y * abs(velocity.y/100.0));
+            if (fabs(velocity.x) > 1000.0 || fabs(velocity.y) > 1000.0) {
+                TouchMoved(prevpt.x + delta.x * fabs(velocity.x/100.0), prevpt.y + delta.y * fabs(velocity.y/100.0));
             }
         }
         TouchEnded();
@@ -376,8 +284,8 @@ static int startx, starty;
     } else if (state == UIGestureRecognizerStateEnded) {
         // if velocity is high then move further in current direction
         CGPoint velocity = [gestureRecognizer velocityInView:self];
-        if (abs(velocity.x) > 1000.0 || abs(velocity.y) > 1000.0) {
-            TouchMoved(prevpt.x + delta.x * abs(velocity.x/100.0), prevpt.y + delta.y * abs(velocity.y/100.0));
+        if (fabs(velocity.x) > 1000.0 || fabs(velocity.y) > 1000.0) {
+            TouchMoved(prevpt.x + delta.x * fabs(velocity.x/100.0), prevpt.y + delta.y * fabs(velocity.y/100.0));
         }
         TouchEnded();
         currlayer->touchmode = oldmode;
@@ -521,7 +429,7 @@ static double prevtime = 0.0;   // used to detect a double tap
         ClearMessage();
         if (TimeInSeconds() - prevtime < 0.3) {
             // double tap
-            if (waitingforpaste && PointInPasteImage(pt.x, pt.y)) {
+            if (waitingforpaste && PointInPasteImage(pt.x, pt.y) && !drawingcells) {
                 // if generating then stop (consistent with doPaste in PatternViewController.m)
                 StopIfGenerating();
                 ClearMessage();
@@ -550,4 +458,92 @@ static double prevtime = 0.0;   // used to detect a double tap
 
 // -----------------------------------------------------------------------------
 
+- (id)initWithCoder:(NSCoder *)c
+{
+    self = [super initWithCoder:c];
+    if (self) {
+        
+        // init the OpenGL ES stuff (based on Apple's GLPaint sample code):
+        
+		CAEAGLLayer *eaglLayer = (CAEAGLLayer *)self.layer;
+		eaglLayer.opaque = YES;
+        
+        // note that we're using OpenGL ES version 1
+		context = [[EAGLContext alloc] initWithAPI:kEAGLRenderingAPIOpenGLES1];
+		if (!context || ![EAGLContext setCurrentContext:context]) {
+			self = nil;
+			return self;
+		}
+		
+		// set the view's scale factor
+		self.contentScaleFactor = 1.0;
+        
+        // we only do 2D drawing
+        glDisable(GL_DEPTH_TEST);
+        glDisable(GL_DITHER);
+        glDisable(GL_MULTISAMPLE);
+        glDisable(GL_STENCIL_TEST);
+        glDisable(GL_FOG);
+        
+        glEnableClientState(GL_TEXTURE_COORD_ARRAY);
+        glEnableClientState(GL_VERTEX_ARRAY);
+        
+        glEnable(GL_BLEND);
+        glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
+        
+        // add gesture recognizers to this view:
+        
+        [self setMultipleTouchEnabled:YES];
+        
+        // pinch gestures will zoom in/out
+        UIPinchGestureRecognizer *pinchGesture = [[UIPinchGestureRecognizer alloc]
+                                                  initWithTarget:self action:@selector(zoomView:)];
+        pinchGesture.delegate = self;
+        [self addGestureRecognizer:pinchGesture];
+        
+        // one-finger pan gestures will be used to draw/pick/select/etc, depending on the current touch mode
+        UIPanGestureRecognizer *pan1Gesture = [[UIPanGestureRecognizer alloc]
+                                               initWithTarget:self action:@selector(singleDrag:)];
+        pan1Gesture.minimumNumberOfTouches = 1;
+        pan1Gesture.maximumNumberOfTouches = 1;
+        pan1Gesture.delegate = self;
+        [self addGestureRecognizer:pan1Gesture];
+        
+        // two-finger pan gestures will move the view
+        UIPanGestureRecognizer *pan2Gesture = [[UIPanGestureRecognizer alloc]
+                                               initWithTarget:self action:@selector(moveView:)];
+        pan2Gesture.minimumNumberOfTouches = 2;
+        pan2Gesture.maximumNumberOfTouches = 2;
+        pan2Gesture.delegate = self;
+        [self addGestureRecognizer:pan2Gesture];
+        
+        // single-taps will do various actions depending on the current touch mode
+        UITapGestureRecognizer *tap1Gesture = [[UITapGestureRecognizer alloc]
+                                               initWithTarget:self action:@selector(singleTap:)];
+        tap1Gesture.numberOfTapsRequired = 1;
+        tap1Gesture.numberOfTouchesRequired = 1;
+        tap1Gesture.delegate = self;
+        [self addGestureRecognizer:tap1Gesture];
+        
+        /* too many problems if we have gesture recognizers for single-taps and double-taps:
+        
+        // double-taps will do various actions depending on the location
+        UITapGestureRecognizer *tap2Gesture = [[UITapGestureRecognizer alloc]
+                                               initWithTarget:self action:@selector(doubleTap:)];
+        tap2Gesture.numberOfTapsRequired = 2;
+        tap2Gesture.numberOfTouchesRequired = 1;
+        tap2Gesture.delegate = self;
+        [self addGestureRecognizer:tap2Gesture];
+        
+        // only do a single-tap if double-tap is not detected
+        // (this works but the delay is way too long)
+        // [tap1Gesture requireGestureRecognizerToFail:tap2Gesture];
+        
+        */
+    }
+    return self;
+}
+
+// -----------------------------------------------------------------------------
+
 @end
