@@ -31,7 +31,7 @@
 #include "layer.h"       // for AddLayer, currlayer
 #include "file.h"        // for NewPattern
 #include "control.h"     // for generating, StartGenerating, etc
-#include "view.h"        // for nopattupdate, SmallScroll
+#include "view.h"        // for nopattupdate, SmallScroll, drawingcells
 #include "undo.h"        // for currlayer->undoredo->...
 
 #import "GollyAppDelegate.h"        // for EnableTabBar
@@ -111,7 +111,7 @@ static void CreateDocSubdir(NSString *subdirname)
 
 static void InitPaths()
 {
-    // init userdir to directory containing Golly.app
+    // init userdir to directory containing user's data
     userdir = [NSHomeDirectory() cStringUsingEncoding:NSUTF8StringEncoding];
     userdir += "/";
 
@@ -128,7 +128,9 @@ static void InitPaths()
     CreateDocSubdir(@"Rules");
 
     // supplied patterns, rules, help are bundled inside Golly.app
-    supplieddir = userdir + "Golly.app/";
+    NSString *appdir = [[NSBundle mainBundle] resourcePath];
+    supplieddir = [appdir cStringUsingEncoding:NSUTF8StringEncoding];
+    supplieddir += "/";
     patternsdir = supplieddir + "Patterns/";
     rulesdir = supplieddir + "Rules/";
     helpdir = supplieddir + "Help/";
@@ -142,7 +144,7 @@ static void InitPaths()
     // init path to file that stores user preferences
     prefsfile = userdir + "Library/Preferences/GollyPrefs";
 
-    /*
+#if 0
     NSLog(@"userdir =     %s", userdir.c_str());
     NSLog(@"savedir =     %s", savedir.c_str());
     NSLog(@"downloaddir = %s", downloaddir.c_str());
@@ -154,7 +156,7 @@ static void InitPaths()
     NSLog(@"tempdir =     %s", tempdir.c_str());
     NSLog(@"clipfile =    %s", clipfile.c_str());
     NSLog(@"prefsfile =   %s", prefsfile.c_str());
-    */
+#endif
 }
 
 // -----------------------------------------------------------------------------
@@ -176,7 +178,7 @@ static void InitPaths()
     static bool firstload = true;
     if (firstload) {
         firstload = false;          // only do the following once
-        SetMessage("This is Golly 1.2 for iOS.  Copyright 2013 The Golly Gang.");
+        SetMessage("This is Golly 1.2 for iOS.  Copyright 2016 The Golly Gang.");
         MAX_MAG = 5;                // maximum cell size = 32x32
         InitAlgorithms();           // must initialize algoinfo first
         InitPaths();                // init userdir, etc (must be before GetPrefs)
@@ -341,6 +343,8 @@ static void InitPaths()
 
 - (IBAction)doNew:(id)sender
 {
+    if (drawingcells) return;
+    
     // undo/redo history is about to be cleared so no point calling RememberGenFinish
     // if we're generating a (possibly large) pattern
     bool saveundo = allowundo;
@@ -370,6 +374,8 @@ static void InitPaths()
 
 - (IBAction)doInfo:(id)sender
 {
+    if (drawingcells) return;
+    
     // if generating then just pause the timer so doGeneration won't be called;
     // best not to call PauseGenerating() because it calls StopGenerating()
     // which might result in a lengthy file save for undo/redo
@@ -381,7 +387,7 @@ static void InitPaths()
     InfoViewController *modalInfoController = [[InfoViewController alloc] initWithNibName:nil bundle:nil];
     
     [modalInfoController setModalPresentationStyle:UIModalPresentationFullScreen];
-    [self presentModalViewController:modalInfoController animated:YES];
+    [self presentViewController:modalInfoController animated:YES completion:nil];
     
     modalInfoController = nil;
     
@@ -392,6 +398,8 @@ static void InitPaths()
 
 - (IBAction)doSave:(id)sender
 {
+    if (drawingcells) return;
+    
     [self stopIfGenerating];
     
     if (event_checker > 0) {
@@ -406,7 +414,7 @@ static void InitPaths()
     SaveViewController *modalSaveController = [[SaveViewController alloc] initWithNibName:nil bundle:nil];
     
     [modalSaveController setModalPresentationStyle:UIModalPresentationFormSheet];
-    [self presentModalViewController:modalSaveController animated:YES];
+    [self presentViewController:modalSaveController animated:YES completion:nil];
     
     modalSaveController = nil;
 }
@@ -415,6 +423,8 @@ static void InitPaths()
 
 - (IBAction)doUndo:(id)sender
 {
+    if (drawingcells) return;
+    
     [self stopIfGenerating];
     
     if (event_checker > 0) {
@@ -432,6 +442,8 @@ static void InitPaths()
 
 - (IBAction)doRedo:(id)sender
 {
+    if (drawingcells) return;
+    
     [self stopIfGenerating];
     
     if (event_checker > 0) {
@@ -449,6 +461,8 @@ static void InitPaths()
 
 - (IBAction)doReset:(id)sender
 {
+    if (drawingcells) return;
+    
     [self stopIfGenerating];
     
     if (event_checker > 0) {
@@ -466,6 +480,10 @@ static void InitPaths()
 
 - (IBAction)doStartStop:(id)sender
 {
+    if (drawingcells) return;
+    // this can happen on iPad if user taps Start/Stop button while another finger
+    // is currently drawing cells
+    
     ClearMessage();
     if (generating) {
         [self stopGenTimer];
@@ -539,6 +557,8 @@ static void InitPaths()
 
 - (IBAction)doNext:(id)sender
 {
+    if (drawingcells) return;
+    
     [self stopIfGenerating];
     
     if (event_checker > 0) {
@@ -559,6 +579,8 @@ static void InitPaths()
 
 - (IBAction)doStep:(id)sender
 {
+    if (drawingcells) return;
+    
     [self stopIfGenerating];
     
     if (event_checker > 0) {
@@ -727,6 +749,8 @@ static void InitPaths()
 
 - (IBAction)doPaste:(id)sender
 {
+    if (drawingcells) return;
+    
     [self stopIfGenerating];
     
     if (event_checker > 0) {
@@ -748,6 +772,8 @@ static void InitPaths()
 
 - (IBAction)doRule:(id)sender
 {
+    if (drawingcells) return;
+    
     [self stopIfGenerating];
     
     if (event_checker > 0) {
@@ -762,7 +788,7 @@ static void InitPaths()
     RuleViewController *modalRuleController = [[RuleViewController alloc] initWithNibName:nil bundle:nil];
 
     [modalRuleController setModalPresentationStyle:UIModalPresentationFullScreen];
-    [self presentModalViewController:modalRuleController animated:YES];
+    [self presentViewController:modalRuleController animated:YES completion:nil];
     
     modalRuleController = nil;
 }
@@ -772,6 +798,7 @@ static void InitPaths()
 - (IBAction)toggleFullScreen:(id)sender
 {
     if (fullscreen) {
+        ShowTabBar(YES);
         topBar.hidden = NO;
         editBar.hidden = NO;
         bottomBar.hidden = NO;
@@ -782,10 +809,11 @@ static void InitPaths()
         CGFloat x = pattView.superview.frame.origin.x;
         CGFloat y = pattView.superview.frame.origin.y + topht;
         CGFloat wd = pattView.superview.frame.size.width;
-        CGFloat ht = pattView.superview.frame.size.height - (topht + barht);
+        CGFloat ht = pattView.superview.frame.size.height - (topht + barht + TabBarHeight());
         pattView.frame = CGRectMake(x, y, wd, ht);
         [statView setNeedsDisplay];
     } else {
+        ShowTabBar(NO);
         topBar.hidden = YES;
         editBar.hidden = YES;
         bottomBar.hidden = YES;
