@@ -465,7 +465,7 @@ static int globalButton;
 
 - (IBAction)cancelRuleChange:(id)sender
 {
-    [self dismissModalViewControllerAnimated:YES];
+    [self dismissViewControllerAnimated:YES completion:nil];
 }
 
 // -----------------------------------------------------------------------------
@@ -482,7 +482,7 @@ static int globalButton;
     }
     
     // dismiss modal view first in case ChangeAlgorithm calls BeginProgress
-    [self dismissModalViewControllerAnimated:YES];
+    [self dismissViewControllerAnimated:YES completion:nil];
     
     // get current rule in ruleText
     std::string newrule = [ruleText.text cStringUsingEncoding:NSUTF8StringEncoding];
@@ -499,6 +499,14 @@ static int globalButton;
         newrule = currlayer->algo->getrule();
         int newmaxstate = currlayer->algo->NumCellStates() - 1;
         if (oldrule != newrule || oldmaxstate != newmaxstate) {
+		
+			// if pattern exists and is at starting gen then ensure savestart is true
+			// so that SaveStartingPattern will save pattern to suitable file
+			// (and thus undo/reset will work correctly)
+			if (currlayer->algo->getGeneration() == currlayer->startgen && !currlayer->algo->isEmpty()) {
+				currlayer->savestart = true;
+			}
+
             // if grid is bounded then remove any live cells outside grid edges
             if (currlayer->algo->gridwd > 0 || currlayer->algo->gridht > 0) {
                 ClearOutsideGrid();
@@ -625,7 +633,7 @@ static int globalButton;
         // look for special prefixes used by Golly (and return NO if found)
         if ([link hasPrefix:@"open:"]) {
             // open specified file, but dismiss modal view first in case OpenFile calls BeginProgress
-            [self dismissModalViewControllerAnimated:YES];
+            [self dismissViewControllerAnimated:YES completion:nil];
             std::string path = [[link substringFromIndex:5] cStringUsingEncoding:NSUTF8StringEncoding];
             OpenFile(path.c_str());
             SavePrefs();
