@@ -293,13 +293,7 @@ static int InitGL()
     glDisable(GL_DEPTH_TEST);
     glDisable(GL_DITHER);
     glDisable(GL_STENCIL_TEST);
-    // following 2 cause WebGL warnings on Chrome and Firefox:
-    // glDisable(GL_FOG);
-    // glDisable(GL_MULTISAMPLE);
-
     glEnable(GL_BLEND);
-    // this blending function seems similar to the one used in desktop Golly
-    // (ie. selected patterns look much the same)
     glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
     glClearColor(1.0, 1.0, 1.0, 1.0);
 
@@ -1235,6 +1229,7 @@ static void FlipPasteOrSelection(bool direction)
     } else if (SelectionExists()) {
         FlipSelection(direction);
     }
+    UpdateEverything();
 }
 
 // -----------------------------------------------------------------------------
@@ -1246,6 +1241,7 @@ static void RotatePasteOrSelection(bool direction)
     } else if (SelectionExists()) {
         RotateSelection(direction);
     }
+    UpdateEverything();
 }
 
 // -----------------------------------------------------------------------------
@@ -1977,7 +1973,7 @@ static void DoFrame()
 
 int EMSCRIPTEN_KEEPALIVE main()
 {
-    SetMessage("This is Golly 0.8 for the web.  Copyright 2014 The Golly Gang.");
+    SetMessage("This is Golly 0.9 for the web.  Copyright 2016 The Golly Gang.");
     InitPaths();                // init tempdir, prefsfile, etc
     MAX_MAG = 5;                // maximum cell size = 32x32
     maxhashmem = 300;           // enough for caterpillar
