@@ -209,7 +209,14 @@ wxBitmapButton* ebbutt[NUM_BUTTONS];
 
 EditBar::EditBar(wxWindow* parent, wxCoord xorg, wxCoord yorg, int wd, int ht)
 : wxPanel(parent, wxID_ANY, wxPoint(xorg,yorg), wxSize(wd,ht),
-          wxNO_FULL_REPAINT_ON_RESIZE)
+#ifdef __WXMSW__
+            // need this to avoid buttons flashing on Windows
+            wxNO_FULL_REPAINT_ON_RESIZE
+#else
+            // better for Mac and Linux
+            wxFULL_REPAINT_ON_RESIZE
+#endif
+            )
 {
 #ifdef __WXGTK__
     // avoid erasing background on GTK+
@@ -299,9 +306,6 @@ EditBar::EditBar(wxWindow* parent, wxCoord xorg, wxCoord yorg, int wd, int ht)
     // we need to specify facename to get Monaco instead of Courier
     editfont = wxFont::New(10, wxMODERN, wxNORMAL, wxNORMAL, false, wxT("Monaco"));
     textascent = 10;
-#elif defined(__WXMAC__)
-    editfont = wxFont::New(10, wxMODERN, wxNORMAL, wxNORMAL);
-    textascent = 10;
 #else
     editfont = wxFont::New(10, wxMODERN, wxNORMAL, wxNORMAL);
     textascent = 10;
@@ -871,7 +875,7 @@ int EditBarHeight() {
 
 void ResizeEditBar(int wd)
 {
-    if (editbarptr) {
+    if (editbarptr && showedit) {
         editbarptr->SetSize(wd, editbarht);
     }
 }
@@ -921,26 +925,12 @@ void UpdateEditBar()
 void ToggleEditBar()
 {
     showedit = !showedit;
-    wxRect r = bigview->GetRect();
-    
-    if (showedit) {
-        // show edit bar at top of viewport window or underneath layer bar
-        r.y += editbarht;
-        r.height -= editbarht;
-        ResizeEditBar(r.width);
-    } else {
-        // hide edit bar
-        r.y -= editbarht;
-        r.height += editbarht;
-    }
-    bigview->SetSize(r);
+    mainptr->ResizeBigView();
     editbarptr->Show(showedit);    // needed on Windows
-    
     if (showlayer) {
         // line at bottom of layer bar may need to be added/removed
         RedrawLayerBar();
     }
-    
     mainptr->UpdateEverything();
 }
 
@@ -951,13 +941,7 @@ void ToggleAllStates()
     showallstates = !showallstates;
     editbarht = showallstates ? BIGHT : SMALLHT;
     if (showedit) {
-        int diff = BIGHT - SMALLHT;
-        if (!showallstates) diff *= -1;
-        wxRect r = bigview->GetRect();
-        ResizeEditBar(r.width);
-        r.y += diff;
-        r.height -= diff;
-        bigview->SetSize(r);
+        mainptr->ResizeBigView();
         mainptr->UpdateEverything();
     } else if (showallstates) {
         // show the edit bar using new height
