@@ -49,7 +49,6 @@
 #include "wxinfo.h"        // for GetInfoFrame
 #include "wxstatus.h"      // for statusptr->...
 #include "wxview.h"        // for viewptr->...
-#include "wxrender.h"      // for SetSelectionColor
 #include "wxscript.h"      // for RunScript, inscript
 #include "wxmain.h"        // for MainFrame, etc
 #include "wxundo.h"        // for currlayer->undoredo->...
@@ -58,11 +57,6 @@
 #include "wxhelp.h"        // for ShowHelp, LoadRule
 #include "wxtimeline.h"    // for InitTimelineFrame, ToggleTimelineBar, etc
 
-#if defined(__WXMAC__) && !defined(__WXOSX_COCOA__)
-    #include <Carbon/Carbon.h>                      // for OpaqueWindowPtr, etc
-    #include "wx/mac/corefoundation/cfstring.h"     // for wxMacCFStringHolder
-#endif
-
 #ifdef __WXMAC__
     // convert path to decomposed UTF8 so fopen will work
     #define FILEPATH path.fn_str()
@@ -88,20 +82,6 @@ wxString MainFrame::GetBaseName(const wxString& path)
 
 // -----------------------------------------------------------------------------
 
-void MainFrame::MySetTitle(const wxString& title)
-{
-    // fix if SetTitle still causes window refresh in Cocoa version!!!
-#if defined(__WXMAC__) && !defined(__WXOSX_COCOA__)
-    // avoid wxMac's SetTitle call -- it causes an undesirable window refresh
-    SetWindowTitleWithCFString((OpaqueWindowPtr*)this->MacGetWindowRef(),
-                               wxMacCFStringHolder(title, wxFONTENCODING_DEFAULT));
-#else
-    SetTitle(title);
-#endif
-}
-
-// -----------------------------------------------------------------------------
-
 void MainFrame::SetWindowTitle(const wxString& filename)
 {
     if ( !filename.IsEmpty() ) {
@@ -140,25 +120,7 @@ void MainFrame::SetWindowTitle(const wxString& filename)
 #endif
     
     // nicer to truncate a really long title???
-    MySetTitle(wtitle);
-}
-
-// -----------------------------------------------------------------------------
-
-void MainFrame::SetGenIncrement()
-{
-    if (currlayer->currexpo > 0) {
-        bigint inc = 1;
-        // set increment to currbase^currexpo
-        int i = currlayer->currexpo;
-        while (i > 0) {
-            inc.mul_smallint(currlayer->currbase);
-            i--;
-        }
-        currlayer->algo->setIncrement(inc);
-    } else {
-        currlayer->algo->setIncrement(1);
-    }
+    SetTitle(wtitle);
 }
 
 // -----------------------------------------------------------------------------
@@ -185,10 +147,9 @@ void MainFrame::CreateUniverse()
 void MainFrame::NewPattern(const wxString& title)
 {
     if (generating) {
-        // terminate generating loop and set command_pending flag
-        Stop();
         command_pending = true;
         cmdevent.SetId(wxID_NEW);
+        Stop();
         return;
     }
     
@@ -353,7 +314,7 @@ void MainFrame::LoadPattern(const wxString& path, const wxString& newtitle,
     if (!newtitle.IsEmpty()) {
         // show new file name in window title but no rule (which readpattern can change);
         // nicer if user can see file name while loading a very large pattern
-        MySetTitle(_("Loading ") + newtitle);
+        SetTitle(_("Loading ") + newtitle);
     }
     
     if (IsImageFile(path)) {
@@ -818,8 +779,6 @@ void MainFrame::OpenFile(const wxString& path, bool remember)
     }
     
     if (generating) {
-        // terminate generating loop and set command_pending flag
-        Stop();
         command_pending = true;
         // assume remember is true (should only be false if called from a script)
         if ( IsScriptFile(path) ) {
@@ -829,6 +788,7 @@ void MainFrame::OpenFile(const wxString& path, bool remember)
             AddRecentPattern(path);
             cmdevent.SetId(ID_OPEN_RECENT + 1);
         }
+        Stop();
         return;
     }
     
@@ -996,10 +956,9 @@ void MainFrame::AddRecentScript(const wxString& inpath)
 void MainFrame::OpenPattern()
 {
     if (generating) {
-        // terminate generating loop and set command_pending flag
-        Stop();
         command_pending = true;
         cmdevent.SetId(wxID_OPEN);
+        Stop();
         return;
     }
     
@@ -1020,11 +979,6 @@ void MainFrame::OpenPattern()
                          opensavedir, wxEmptyString, filetypes,
                          wxFD_OPEN | wxFD_FILE_MUST_EXIST);
     
-#ifdef __WXGTK__
-    // opensavedir is ignored above (bug in wxGTK 2.8.0???)
-    opendlg.SetDirectory(opensavedir);
-#endif
-    
     if ( opendlg.ShowModal() == wxID_OK ) {
         wxFileName fullpath( opendlg.GetPath() );
         opensavedir = fullpath.GetPath();
@@ -1037,26 +991,23 @@ void MainFrame::OpenPattern()
 void MainFrame::OpenScript()
 {
     if (generating) {
-        // terminate generating loop and set command_pending flag
-        Stop();
         command_pending = true;
         cmdevent.SetId(ID_RUN_SCRIPT);
+        Stop();
         return;
     }
     
-    wxString filetypes = _("Perl or Python (*.pl;*.py)|*.pl;*.py");
-    filetypes +=         _("|Perl (*.pl)|*.pl");
+    wxString filetypes = _("Lua or Python (*.lua;*.py)|*.lua;*.py");
+    filetypes +=         _("|Lua (*.lua)|*.lua");
     filetypes +=         _("|Python (*.py)|*.py");
-    
+#ifdef ENABLE_PERL
+    filetypes +=         _("|Perl (*.pl)|*.pl");
+#endif
+
     wxFileDialog opendlg(this, _("Choose a script"),
                          rundir, wxEmptyString, filetypes,
                          wxFD_OPEN | wxFD_FILE_MUST_EXIST);
     
-#ifdef __WXGTK__
-    // rundir is ignored above (bug in wxGTK 2.8.0???)
-    opendlg.SetDirectory(rundir);
-#endif
-    
     if ( opendlg.ShowModal() == wxID_OK ) {
         wxFileName fullpath( opendlg.GetPath() );
         rundir = fullpath.GetPath();
@@ -1211,10 +1162,9 @@ bool MainFrame::ClipboardContainsRule()
 void MainFrame::OpenClipboard()
 {
     if (generating) {
-        // terminate generating loop and set command_pending flag
-        Stop();
         command_pending = true;
         cmdevent.SetId(ID_OPEN_CLIP);
+        Stop();
         return;
     }
     
@@ -1244,9 +1194,10 @@ void MainFrame::OpenClipboard()
 
 wxString MainFrame::GetScriptFileName(const wxString& text)
 {
-    // examine given text to see if it contains Perl or Python code;
-    // if "use" or "my" occurs at start of line then we assume Perl,
-    // if "import" or "from" occurs at start of line then we assume Python,
+    // examine given text to see if it contains Lua, Perl or Python code:
+    // if "--" or "local" or "require" is at start of line then we assume Lua,
+    // if "use" or "my" is at start of line then we assume Perl,
+    // if "import" or "from" is at start of line then we assume Python,
     // otherwise we compare counts for dollars + semicolons vs colons
     int dollars = 0;
     int semicolons = 0;
@@ -1289,10 +1240,18 @@ wxString MainFrame::GetScriptFileName(const wxString& text)
                 linelen = 0;
                 p++;
                 break;
+            case '-':
+                if (linelen == 1 && p[-1] == '-') return luafile;   // Lua comment at start of line
+                linelen++; p++;
+                break;
             case ' ':
                 // look for language-specific keyword at start of line
                 if (linelen == 2 && strncmp(p-2,"my",2) == 0) return perlfile;
                 if (linelen == 3 && strncmp(p-3,"use",3) == 0) return perlfile;
+                
+                if (linelen == 5 && strncmp(p-5,"local",5) == 0) return luafile;
+                if (linelen == 7 && strncmp(p-7,"require",7) == 0) return luafile;
+                
                 if (linelen == 4 && strncmp(p-4,"from",4) == 0) return pythonfile;
                 if (linelen == 6 && strncmp(p-6,"import",6) == 0) return pythonfile;
                 // don't break
@@ -1323,10 +1282,9 @@ wxString MainFrame::GetScriptFileName(const wxString& text)
 void MainFrame::RunClipboard()
 {
     if (generating) {
-        // terminate generating loop and set command_pending flag
-        Stop();
         command_pending = true;
         cmdevent.SetId(ID_RUN_CLIP);
+        Stop();
         return;
     }
     
@@ -1339,18 +1297,7 @@ void MainFrame::RunClipboard()
         // copy clipboard data to scriptfile
         wxFile outfile(scriptfile, wxFile::write);
         if (outfile.IsOpened()) {
-#ifdef __WXMAC__
-            if (scriptfile == perlfile) {
-                // Perl script, so replace CRs with LFs
-                wxString str = data.GetText();
-                str.Replace(wxT("\015"), wxT("\012"));
-                outfile.Write( str );
-            } else {
-                outfile.Write( data.GetText() );
-            }
-#else
             outfile.Write( data.GetText() );
-#endif
             outfile.Close();
             RunScript(scriptfile);
         } else {
@@ -1370,10 +1317,9 @@ void MainFrame::RunClipboard()
 void MainFrame::OpenRecentPattern(int id)
 {
     if (generating) {
-        // terminate generating loop and set command_pending flag
-        Stop();
         command_pending = true;
         cmdevent.SetId(id);
+        Stop();
         return;
     }
     
@@ -1401,10 +1347,9 @@ void MainFrame::OpenRecentPattern(int id)
 void MainFrame::OpenRecentScript(int id)
 {
     if (generating) {
-        // terminate generating loop and set command_pending flag
-        Stop();
         command_pending = true;
         cmdevent.SetId(id);
+        Stop();
         return;
     }
     
@@ -1559,26 +1504,7 @@ const char* MainFrame::WritePattern(const wxString& path,
     if (format == RLE_format && (currlayer->algo->gridwd > 0 || currlayer->algo->gridht > 0))
         format = XRLE_format;
     const char* err = writepattern(FILEPATH, *currlayer->algo, format,
-                                   compression, top, left, bottom, right);
-    
-#ifdef __WXMAC__
-    if (!err) {
-        // set the file's creator and type
-        wxFileName filename(path);
-        wxUint32 creator = 'GoLy';
-        wxUint32 type = 'GoLR';         // RLE or XRLE
-        if (format == MC_format) {
-            type = 'GoLM';
-        }
-    #if defined(__WXOSX_COCOA__)
-        // there is no Cocoa call to set file type and creator
-        creator = 'GoLy';               // avoid compiler warning
-    #else
-        filename.MacSetTypeAndCreator(type, creator);
-    #endif
-    }
-#endif
-    
+                                   compression, top, left, bottom, right);    
     return err;
 }
 
@@ -1587,14 +1513,14 @@ const char* MainFrame::WritePattern(const wxString& path,
 bool MainFrame::SavePattern()
 {
     if (generating) {
-        // terminate generating loop and set command_pending flag
-        Stop();
         command_pending = true;
         cmdevent.SetId(wxID_SAVE);
+        Stop();
         return false;
     }
     
-    if (warn_on_save && currlayer->dirty && currlayer->algo->getGeneration() > currlayer->startgen) {
+    if (warn_on_save && currlayer->dirty && currlayer->algo->getGeneration() > currlayer->startgen &&
+        !TimelineExists()) {
         wxString msg = _("Saving this generation will not save the changes you made earlier, ");
         msg +=         _("so you might want to select Reset or Undo and save those changes.");
         msg +=         _("\n\n(This warning can be disabled in Preferences > Layer.)");
@@ -1660,11 +1586,6 @@ bool MainFrame::SavePattern()
                          opensavedir, currlayer->currname, filetypes,
                          wxFD_SAVE | wxFD_OVERWRITE_PROMPT );
     
-#ifdef __WXGTK__
-    // opensavedir is ignored above (bug in wxGTK 2.8.0???)
-    savedlg.SetDirectory(opensavedir);
-#endif
-    
     if ( savedlg.ShowModal() == wxID_OK ) {
         wxFileName fullpath( savedlg.GetPath() );
         opensavedir = fullpath.GetPath();
@@ -1784,7 +1705,7 @@ void MainFrame::SaveSucceeded(const wxString& path)
 
 // -----------------------------------------------------------------------------
 
-void MainFrame::ToggleShowPatterns()
+void MainFrame::ToggleShowFiles()
 {
     if (splitwin->IsSplit()) dirwinwd = splitwin->GetSashPosition();
 #ifndef __WXMAC__
@@ -1793,21 +1714,15 @@ void MainFrame::ToggleShowPatterns()
     bigview->SetScrollbar(wxVERTICAL, 0, 0, 0, true);
 #endif
     
-    showpatterns = !showpatterns;
-    if (showpatterns && showscripts) {
-        showscripts = false;
-        splitwin->Unsplit(scriptctrl);
-        splitwin->SplitVertically(patternctrl, RightPane(), dirwinwd);
+    showfiles = !showfiles;
+    if (splitwin->IsSplit()) {
+        // hide left pane
+        splitwin->Unsplit(filectrl);
     } else {
-        if (splitwin->IsSplit()) {
-            // hide left pane
-            splitwin->Unsplit(patternctrl);
-        } else {
-            splitwin->SplitVertically(patternctrl, RightPane(), dirwinwd);
-        }
-        viewptr->SetFocus();
+        splitwin->SplitVertically(filectrl, RightPane(), dirwinwd);
     }
-    
+    viewptr->SetFocus();
+
 #ifndef __WXMAC__
     // restore scroll bars
     bigview->UpdateScrollBars();
@@ -1816,125 +1731,36 @@ void MainFrame::ToggleShowPatterns()
 
 // -----------------------------------------------------------------------------
 
-void MainFrame::ToggleShowScripts()
+void MainFrame::ChangeFileDir()
 {
-    if (splitwin->IsSplit()) dirwinwd = splitwin->GetSashPosition();
-#ifndef __WXMAC__
-    // hide scroll bars
-    bigview->SetScrollbar(wxHORIZONTAL, 0, 0, 0, true);
-    bigview->SetScrollbar(wxVERTICAL, 0, 0, 0, true);
-#endif
-    
-    showscripts = !showscripts;
-    if (showscripts && showpatterns) {
-        showpatterns = false;
-        splitwin->Unsplit(patternctrl);
-        splitwin->SplitVertically(scriptctrl, RightPane(), dirwinwd);
-    } else {
-        if (splitwin->IsSplit()) {
-            // hide left pane
-            splitwin->Unsplit(scriptctrl);
-        } else {
-            splitwin->SplitVertically(scriptctrl, RightPane(), dirwinwd);
-        }
-        viewptr->SetFocus();
-    }
-    
-#ifndef __WXMAC__
-    // restore scroll bars
-    bigview->UpdateScrollBars();
-#endif
-}
-
-// -----------------------------------------------------------------------------
-
-void MainFrame::ChangePatternDir()
-{
-    wxDirDialog dirdlg(this, _("Choose a new pattern folder"), patterndir, wxDD_NEW_DIR_BUTTON);
+    wxDirDialog dirdlg(this, _("Choose a new file folder"), filedir, wxDD_NEW_DIR_BUTTON);
     if (dirdlg.ShowModal() == wxID_OK)
-        SetPatternDir(dirdlg.GetPath());
+        SetFileDir(dirdlg.GetPath());
 }
 
 // -----------------------------------------------------------------------------
 
-void MainFrame::ChangeScriptDir()
+void MainFrame::SetFileDir(const wxString& newdir)
 {
-    wxDirDialog dirdlg(this, _("Choose a new script folder"), scriptdir, wxDD_NEW_DIR_BUTTON);
-    if (dirdlg.ShowModal() == wxID_OK)
-        SetScriptDir(dirdlg.GetPath());
-}
-
-// -----------------------------------------------------------------------------
-
-void MainFrame::SetPatternDir(const wxString& newdir)
-{
-    if (patterndir != newdir) {
-        patterndir = newdir;
-        if (showpatterns) {
-            // show new pattern directory
-            SimplifyTree(patterndir, patternctrl->GetTreeCtrl(), patternctrl->GetRootId());
+    if (filedir != newdir) {
+        filedir = newdir;
+        if (showfiles) {
+            // show new file directory
+            SimplifyTree(filedir, filectrl->GetTreeCtrl(), filectrl->GetRootId());
         }
     }
 }
 
 // -----------------------------------------------------------------------------
 
-void MainFrame::SetScriptDir(const wxString& newdir)
-{
-    if (scriptdir != newdir) {
-        scriptdir = newdir;
-        if (showscripts) {
-            // show new script directory
-            SimplifyTree(scriptdir, scriptctrl->GetTreeCtrl(), scriptctrl->GetRootId());
-        }
-    }
-}
-
-// -----------------------------------------------------------------------------
-
-void MainFrame::SetStepExponent(int newexpo)
-{
-    currlayer->currexpo = newexpo;
-    if (currlayer->currexpo < minexpo) currlayer->currexpo = minexpo;
-    SetGenIncrement();
-}
-
-// -----------------------------------------------------------------------------
-
-void MainFrame::SetMinimumStepExponent()
-{
-    // set minexpo depending on mindelay and maxdelay
-    minexpo = 0;
-    if (mindelay > 0) {
-        int d = mindelay;
-        minexpo--;
-        while (d < maxdelay) {
-            d *= 2;
-            minexpo--;
-        }
-    }
-}
-
-// -----------------------------------------------------------------------------
-
-void MainFrame::UpdateStepExponent()
-{
-    SetMinimumStepExponent();
-    if (currlayer->currexpo < minexpo) currlayer->currexpo = minexpo;
-    SetGenIncrement();
-}
-
-// -----------------------------------------------------------------------------
-
 void MainFrame::ShowPrefsDialog(const wxString& page)
 {
     if (viewptr->waitingforclick) return;
     
     if (generating) {
-        // terminate generating loop and set command_pending flag
-        Stop();
         command_pending = true;
         cmdevent.SetId(wxID_PREFERENCES);
+        Stop();
         return;
     }
     
@@ -1950,9 +1776,6 @@ void MainFrame::ShowPrefsDialog(const wxString& page)
     if (ChangePrefs(page)) {
         // user hit OK button
         
-        // selection color may have changed
-        SetSelectionColor();
-        
         // if maxpatterns was reduced then we may need to remove some paths
         while (numpatterns > maxpatterns) {
             numpatterns--;
