@@ -33,7 +33,8 @@
 #include "wx/wfstream.h"        // for wxFileOutputStream, wxFileInputStream
 #include "wx/zipstrm.h"         // for wxZipInputStream
 
-#include "lifealgo.h"      // for lifealgo class
+#include "lifealgo.h"           // for lifealgo class
+#include "ruleloaderalgo.h"     // for noTABLEorTREE
 
 #include "wxgolly.h"       // for wxGetApp, mainptr, viewptr
 #include "wxmain.h"        // for mainptr->...
@@ -324,7 +325,7 @@ void ShowHelp(const wxString& filepath)
         htmlwin->canreload = false;
         
         helpptr->Show(true);
-        UpdateHelpButtons();    // must be after Show to avoid hbar appearing on Mac
+        UpdateHelpButtons();    // must be after Show to avoid scroll bar appearing on Mac
         
         // allow HtmlView::OnSize to call CheckAndLoad if window is resized
         htmlwin->canreload = true;
@@ -412,11 +413,11 @@ void HelpFrame::OnClose(wxCloseEvent& WXUNUSED(event))
 
 // -----------------------------------------------------------------------------
 
-void LoadRule(const wxString& rulestring)
+void LoadRule(const wxString& rulestring, bool fromfile)
 {
-    // load recently installed .rule/table/tree/colors/icons file
     wxString oldrule = wxString(currlayer->algo->getrule(),wxConvLocal);
     int oldmaxstate = currlayer->algo->NumCellStates() - 1;
+    const char* err;
     
     // selection might change if grid becomes smaller,
     // so save current selection for RememberRuleChange/RememberAlgoChange
@@ -426,58 +427,130 @@ void LoadRule(const wxString& rulestring)
     
     if (mainptr->generating) {
         Warning(_("Cannot change rule while generating a pattern."));
-        // or stop and create pending event??? see ID_LOAD_LEXICON
         return;
     } else if (inscript) {
         Warning(_("Cannot change rule while a script is running."));
         return;
     }
     
-    const char* err = currlayer->algo->setrule( rulestring.mb_str(wxConvLocal) );
-    if (err) {
-        // try to find another algorithm that supports the given rule
-        for (int i = 0; i < NumAlgos(); i++) {
-            if (i != currlayer->algtype) {
-                lifealgo* tempalgo = CreateNewUniverse(i);
-                err = tempalgo->setrule( rulestring.mb_str(wxConvLocal) );
-                delete tempalgo;
-                if (!err) {
-                    // change the current algorithm and switch to the new rule
-                    mainptr->ChangeAlgorithm(i, rulestring);
-                    if (i != currlayer->algtype) {
-                        RestoreRule(oldrule);
-                        Warning(_("Algorithm could not be changed (pattern is too big to convert)."));
-                        return;
-                    } else {
-                        mainptr->SetWindowTitle(wxEmptyString);
-                        mainptr->UpdateEverything();
+    if (fromfile) {
+        // load given rule from a .rule file
+        
+        // InitAlgorithms ensures the RuleLoader algo is the last algo
+        int rule_loader_algo = NumAlgos() - 1;
+        
+        if (currlayer->algtype == rule_loader_algo) {
+            // RuleLoader is current algo so no need to switch
+            err = currlayer->algo->setrule( rulestring.mb_str(wxConvLocal) );
+        } else {
+            // switch to RuleLoader algo
+            lifealgo* tempalgo = CreateNewUniverse(rule_loader_algo);
+            err = tempalgo->setrule( rulestring.mb_str(wxConvLocal) );
+            delete tempalgo;
+            if (!err) {
+                // change the current algorithm and switch to the new rule
+                mainptr->ChangeAlgorithm(rule_loader_algo, rulestring);
+                if (rule_loader_algo != currlayer->algtype) {
+                    RestoreRule(oldrule);
+                    Warning(_("Algorithm could not be changed (pattern is too big to convert)."));
+                } else {
+                    mainptr->SetWindowTitle(wxEmptyString);
+                    mainptr->UpdateEverything();
+                }
+                return;
+            }
+        }
+        
+        if (err) {
+            // RuleLoader algo found some sort of error
+            if (strcmp(err, noTABLEorTREE) == 0) {
+                // .rule file has no TABLE or TREE section but it might be used
+                // to override a built-in rule, so try each algo
+                wxString temprule = rulestring;
+                temprule.Replace(wxT("_"), wxT("/"));   // eg. convert B3_S23 to B3/S23
+                for (int i = 0; i < NumAlgos(); i++) {
+                    lifealgo* tempalgo = CreateNewUniverse(i);
+                    err = tempalgo->setrule( temprule.mb_str(wxConvLocal) );
+                    delete tempalgo;
+                    if (!err) {
+                        // change the current algorithm and switch to the new rule
+                        mainptr->ChangeAlgorithm(i, temprule);
+                        if (i != currlayer->algtype) {
+                            RestoreRule(oldrule);
+                            Warning(_("Algorithm could not be changed (pattern is too big to convert)."));
+                        } else {
+                            mainptr->SetWindowTitle(wxEmptyString);
+                            mainptr->UpdateEverything();
+                        }
                         return;
                     }
                 }
             }
+            RestoreRule(oldrule);
+            wxString msg = _("The rule file is not valid:\n") + rulestring;
+            msg += _("\n\nThe error message:\n") + wxString(err,wxConvLocal);
+            Warning(msg);
+            return;
+        }
+    
+    } else {
+        // fromfile is false, so switch to rule given in a "rule:" link
+        
+        err = currlayer->algo->setrule( rulestring.mb_str(wxConvLocal) );
+        if (err) {
+            // try to find another algorithm that supports the given rule
+            for (int i = 0; i < NumAlgos(); i++) {
+                if (i != currlayer->algtype) {
+                    lifealgo* tempalgo = CreateNewUniverse(i);
+                    err = tempalgo->setrule( rulestring.mb_str(wxConvLocal) );
+                    delete tempalgo;
+                    if (!err) {
+                        // change the current algorithm and switch to the new rule
+                        mainptr->ChangeAlgorithm(i, rulestring);
+                        if (i != currlayer->algtype) {
+                            RestoreRule(oldrule);
+                            Warning(_("Algorithm could not be changed (pattern is too big to convert)."));
+                        } else {
+                            mainptr->SetWindowTitle(wxEmptyString);
+                            mainptr->UpdateEverything();
+                        }
+                        return;
+                    }
+                }
+            }
+            RestoreRule(oldrule);
+            Warning(_("Given rule is not valid in any algorithm:\n") + rulestring);
+            return;
         }
-        // should only get here if .rule/table/tree file contains some sort of error
-        RestoreRule(oldrule);
-        Warning(_("Rule is not valid in any algorithm: ") + rulestring);
-        return;
     }
     
     wxString newrule = wxString(currlayer->algo->getrule(),wxConvLocal);
-    if (oldrule != newrule) {
+    int newmaxstate = currlayer->algo->NumCellStates() - 1;
+    if (oldrule != newrule || oldmaxstate != newmaxstate) {
         // show new rule in main window's title but don't change name
         mainptr->SetWindowTitle(wxEmptyString);
         
+		// if pattern exists and is at starting gen then ensure savestart is true
+		// so that SaveStartingPattern will save pattern to suitable file
+		// (and thus undo/reset will work correctly)
+		if (currlayer->algo->getGeneration() == currlayer->startgen && !currlayer->algo->isEmpty()) {
+			currlayer->savestart = true;
+		}
+        
         // if grid is bounded then remove any live cells outside grid edges
         if (currlayer->algo->gridwd > 0 || currlayer->algo->gridht > 0) {
             mainptr->ClearOutsideGrid();
         }
-    }
-    
-    // new rule might have changed the number of cell states;
-    // if there are fewer states then pattern might change
-    int newmaxstate = currlayer->algo->NumCellStates() - 1;
-    if (newmaxstate < oldmaxstate && !currlayer->algo->isEmpty()) {
-        mainptr->ReduceCellStates(newmaxstate);
+	
+		// new rule might have changed the number of cell states;
+		// if there are fewer states then pattern might change
+		if (newmaxstate < oldmaxstate && !currlayer->algo->isEmpty()) {
+			mainptr->ReduceCellStates(newmaxstate);
+		}
+
+        if (allowundo && !currlayer->stayclean) {
+            currlayer->undoredo->RememberRuleChange(oldrule);
+        }
     }
     
     // update colors and/or icons for the new rule
@@ -485,12 +558,6 @@ void LoadRule(const wxString& rulestring)
     
     // pattern might have changed or colors/icons might have changed
     mainptr->UpdateEverything();
-    
-    if (oldrule != newrule) {
-        if (allowundo && !currlayer->stayclean) {
-            currlayer->undoredo->RememberRuleChange(oldrule);
-        }
-    }
 }
 
 // -----------------------------------------------------------------------------
@@ -650,7 +717,7 @@ void GetURL(const wxString& url)
         htmlwin->LoadPage(filepath);
         
     } else if (IsRuleFile(filename)) {
-        // load corresponding rule
+        // load corresponding .rule file
         LoadRule(filename.BeforeLast('.'));
         
     } else if (IsTextFile(filename)) {
@@ -696,7 +763,7 @@ void UnzipFile(const wxString& zippath, const wxString& entry)
                     mainptr->EditFile(rulefile);
                 }
             } else {         
-                // load corresponding rule
+                // load corresponding .rule file
                 LoadRule(filename.BeforeLast('.'));
             }
         } else {
@@ -781,10 +848,9 @@ void ClickLexiconPattern(const wxHtmlCell* htmlcell)
                     }
                     
                     if (mainptr->generating) {
-                        // terminate generating loop and set command_pending flag
-                        mainptr->Stop();
                         mainptr->command_pending = true;
                         mainptr->cmdevent.SetId(ID_LOAD_LEXICON);
+                        mainptr->Stop();
                         return;
                     }
                     
@@ -818,7 +884,7 @@ void LoadLexiconPattern()
         // switching to B3/S23 or Life; if that fails then switch to QuickLife
         const char* err = currlayer->algo->setrule("B3/S23");
         if (err) {
-            // try "Life" in case current algo is RuleLoader and Life.table/tree exists
+            // try "Life" in case current algo is RuleLoader and Life.rule exists
             // (also had to make a similar change to the loadpattern code in readpattern.cpp)
             err = currlayer->algo->setrule("Life");
         }
@@ -858,7 +924,6 @@ void HtmlView::OnLinkClicked(const wxHtmlLinkInfo& link)
     } else if ( url.StartsWith(wxT("get:")) ) {
         if (mainptr->generating) {
             Warning(_("Cannot download file while generating a pattern."));
-            // or stop and create pending event??? see ID_LOAD_LEXICON
         } else if (inscript) {
             Warning(_("Cannot download file while a script is running."));
         } else if (editlink && IsZipFile(url)) {
@@ -925,7 +990,8 @@ void HtmlView::OnLinkClicked(const wxHtmlLinkInfo& link)
         }
         
     } else if ( url.StartsWith(wxT("rule:")) ) {
-        LoadRule( url.AfterFirst(':') );
+        // switch to given rule (false = not necessarily in a .rule file)
+        LoadRule( url.AfterFirst(':'), false );
         
     } else {
         // assume it's a link to a local target or another help file
@@ -995,7 +1061,7 @@ void HtmlView::ClearStatus()
 void HtmlView::OnMouseDown(wxMouseEvent& event)
 {
     // set flag so ctrl/right-clicked file can be opened in editor
-    // (this is consistent with how we handle clicks in pattern/script pane)
+    // (this is consistent with how we handle clicks in the file pane)
     editlink = event.ControlDown() || event.RightDown();
     event.Skip();
 }
