@@ -81,7 +81,6 @@ public:
     algo_type startalgo;          // starting algorithm
     bool savestart;               // need to save starting pattern?
     bool startdirty;              // starting state of dirty flag
-    wxString startfile;           // file for saving starting pattern
     wxString startname;           // starting currname
     wxString startrule;           // starting rule
     bigint startgen;              // starting generation (>= 0)
@@ -106,22 +105,23 @@ public:
     unsigned char cellg[256];     // green components for states 0..255
     unsigned char cellb[256];     // blue components for states 0..255
     
-    wxBrush* deadbrush;           // brush for drawing dead cells
-    wxPen* gridpen;               // pen for drawing plain grid
-    wxPen* boldpen;               // pen for drawing bold grid
-    
     // icons for this layer
     wxBitmap** icons7x7;          // icons for scale 1:8
     wxBitmap** icons15x15;        // icons for scale 1:16
     wxBitmap** icons31x31;        // icons for scale 1:32
+
+    // texture atlases for rendering icons
+    unsigned char* atlas7x7;      // atlas for 7x7 icons
+    unsigned char* atlas15x15;    // atlas for 15x15 icons
+    unsigned char* atlas31x31;    // atlas for 31x31 icons
     
+    int numicons;                 // number of icons (= number of live states)
     bool multicoloricons;         // are icons multi-colored? (grayscale if not)
     
     // used if the layer has a timeline (see wxtimeline.cpp)
     int currframe;                // current frame in timeline
     int autoplay;                 // +ve = play forwards, -ve = play backwards, 0 = stop
     int tlspeed;                  // controls speed at which frames are played
-    long lastframe;               // time (in msecs) when last frame was displayed
 };
 
 const int MAX_LAYERS = 10;  // maximum number of layers
@@ -130,6 +130,9 @@ extern int numclones;       // number of cloned layers
 extern int currindex;       // index of current layer (0..numlayers-1)
 extern Layer* currlayer;    // pointer to current layer
 
+Layer* GetLayer(int index);
+// Return a pointer to the layer specified by the given index.
+
 void AddLayer();
 // Add a new layer (with an empty universe) and make it the current layer.
 // The first call creates the initial layer.  Later calls insert the
@@ -212,9 +215,6 @@ void SwitchToClickedTile(int index);
 void ResizeLayers(int wd, int ht);
 // Resize the viewport in all layers.
 
-Layer* GetLayer(int index);
-// Return a pointer to the layer specified by the given index.
-
 bool RestoreRule(const wxString& rule);
 // Try to set the current layer's rule to a previously known rule.
 // If this succeeds return true, but if it fails then warn the user,
@@ -222,6 +222,9 @@ bool RestoreRule(const wxString& rule);
 // The latter can happen if the given rule's table/tree file was
 // deleted or was edited and some sort of error introduced.
 
+Layer* CreateTemporaryLayer();
+// Create a temporary layer with the same algo type as currlayer.
+
 
 // Layer bar routines:
 
@@ -248,12 +251,16 @@ void ToggleLayerBar();
 // Show/hide the layer bar.
 
 
-// move this color stuff into wxcolors.*???
+// Color handling routines:
 
 void CreateColorGradient();
 // Create a color gradient for the current layer using
 // currlayer->fromrgb and currlayer->torgb.
 
+void UpdateIconColors();
+// Update the icon texture atlases for the current layer.
+// Must be called BEFORE calling UpdateCloneColors.
+
 void UpdateCloneColors();
 // If current layer has clones then update their colors.
 
@@ -264,7 +271,10 @@ void UpdateLayerColors();
 
 void InvertCellColors();
 // Invert the cell colors in all layers, including the dead cell color,
-// grid lines, and the colors in all icon bitmaps.
+// grid lines, and the colors in all icons.
+
+void InvertIconColors(unsigned char* atlasptr, int iconsize, int numicons);
+// Invert the icon colors in the given texture atlas.
 
 void SetLayerColors();
 // Open a dialog to change the current layer's colors.
