@@ -50,7 +50,7 @@ public:
     void UpdateToolBar();
     void EnableAllMenus(bool enable);
     void UpdateMenuItems();
-    void UpdatePatternAndStatus();
+    void UpdatePatternAndStatus(bool update_now = false);
     void UpdateStatus();
     void UpdateMenuAccelerators();
     
@@ -71,12 +71,9 @@ public:
     void SetWindowTitle(const wxString& filename);
     void OpenPattern();
     void OpenScript();
-    void ToggleShowPatterns();
-    void ToggleShowScripts();
-    void ChangePatternDir();
-    void ChangeScriptDir();
-    void SetPatternDir(const wxString& newdir);
-    void SetScriptDir(const wxString& newdir);
+    void ToggleShowFiles();
+    void ChangeFileDir();
+    void SetFileDir(const wxString& newdir);
     bool SavePattern();
     bool SaveCurrentLayer();
     const char* SaveFile(const wxString& path, const wxString& format, bool remember);
@@ -104,10 +101,14 @@ public:
     void ShowPrefsDialog(const wxString& page = wxEmptyString);
     
     // control functions
-    void GeneratePattern();
+    void StartGenTimer();
+    void StartGenerating();
+    void StopGenerating();
+    void StartOrStop();
+    void Stop();
+    void FinishUp();
     void GoFaster();
     void GoSlower();
-    void Stop();
     void DisplayTimingInfo();
     void NextGeneration(bool useinc);
     void ToggleAutoFit();
@@ -129,9 +130,6 @@ public:
     void ChangeAlgorithm(algo_type newalgotype,
                          const wxString& newrule = wxEmptyString,
                          bool inundoredo = false);
-    bool CreateBorderCells(lifealgo* curralgo);
-    bool DeleteBorderCells(lifealgo* curralgo);
-    void ClearRect(lifealgo* curralgo, int top, int left, int bottom, int right);
     
     // view functions
     void ToggleStatusBar();
@@ -141,6 +139,7 @@ public:
     void ShowPatternInfo();
     void ResizeSplitWindow(int wd, int ht);
     void ResizeStatusBar(int wd, int ht);
+    void ResizeBigView();
     wxWindow* RightPane();
     
     // layer functions
@@ -153,24 +152,32 @@ public:
     void EditFile(const wxString& filepath);
     void QuitApp();
     
-    bool generating;           // currently generating pattern?
-    bool fullscreen;           // in full screen mode?
-    bool showbanner;           // showing banner message?
-    bool keepmessage;          // don't clear message created by script?
-    bool command_pending;      // user selected a command while generating?
-    bool draw_pending;         // user wants to draw while generating?
-    wxCommandEvent cmdevent;   // the pending command
-    wxMouseEvent mouseevent;   // the pending draw
+    wxTimer* gentimer;          // timer for generating patterns
+    bool generating;            // currently generating a pattern?
+    bool fullscreen;            // in full screen mode?
+    bool showbanner;            // showing banner message?
+    bool keepmessage;           // don't clear message created by script?
+    bool command_pending;       // user selected a command while generating?
+    bool draw_pending;          // user wants to draw while generating?
+    wxCommandEvent cmdevent;    // the pending command
+    wxMouseEvent mouseevent;    // the pending draw
     
     // temporary files
-    wxString clipfile;         // name of temporary file for storing clipboard data
-    wxString perlfile;         // name of temporary Perl script
-    wxString pythonfile;       // name of temporary Python script
+    wxString clipfile;          // name of temporary file for storing clipboard data
+    wxString luafile;           // name of temporary Lua script
+    wxString perlfile;          // name of temporary Perl script
+    wxString pythonfile;        // name of temporary Python script
     
     // store files passed via command line (processed in first OnIdle)
     wxArrayString pendingfiles;
 
-    bool infront;              // main window is active?
+    bool infront;               // main window is active?
+
+#ifdef __WXMAC__
+    // these scroll bars are needed to avoid bug in wxGLCanvas on Mac
+    wxScrollBar* hbar;
+    wxScrollBar* vbar;
+#endif
     
 private:
     // any class wishing to process wxWidgets events must use this macro
@@ -184,12 +191,14 @@ private:
     void OnIdle(wxIdleEvent& event);
     void OnDirTreeSelection(wxTreeEvent& event);
     void OnSashDblClick(wxSplitterEvent& event);
-    void OnOneTimer(wxTimerEvent& event);
+    void OnGenTimer(wxTimerEvent& event);
     void OnClose(wxCloseEvent& event);
+#ifdef __WXMAC__
+    void OnScroll(wxScrollEvent& event);
+#endif
     
     // file functions
     bool LoadImage(const wxString& path);
-    void MySetTitle(const wxString& title);
     wxString GetBaseName(const wxString& path);
     void SaveSucceeded(const wxString& path);
     void AddRecentPattern(const wxString& path);
@@ -210,23 +219,22 @@ private:
     // miscellaneous functions
     void CreateMenus();
     void CreateToolbar();
-    void CreateDirControls();
+    void CreateDirControl();
     void SimplifyTree(wxString& dir, wxTreeCtrl* treectrl, wxTreeItemId root);
     void DoPendingAction(bool restart);
     
-    // splittable window contains pattern/script directory in left pane
-    // and layer bar plus viewport window in right pane
+    // splittable window contains file directory in left pane
+    // and layer/edit/timeline bars plus viewport window in right pane
     wxSplitterWindow* splitwin;
-    wxGenericDirCtrl* patternctrl;
-    wxGenericDirCtrl* scriptctrl;
-    
-    int minexpo;                  // currexpo at maximum delay (must be <= 0)
-    long whentosee;               // when to do next gen (if currexpo < 0)
-    long begintime, endtime;      // for timing info
-    double begingen, endgen;      // ditto
+    wxGenericDirCtrl* filectrl;
+
+    int hypdown;                    // for hyperspeed
+    int minexpo;                    // currexpo at maximum delay (must be <= 0)
+    long begintime, endtime;        // for timing info
+    double begingen, endgen;        // ditto
 };
 
-// ids for menu commands
+// ids for menu commands, etc
 enum {
     // File menu
     // wxID_NEW,
@@ -236,8 +244,6 @@ enum {
     // last 2 items in Open Recent submenu
     ID_CLEAR_MISSING_PATTERNS = ID_OPEN_RECENT + MAX_RECENT + 1,
     ID_CLEAR_ALL_PATTERNS,
-    ID_SHOW_PATTERNS,
-    ID_PATTERN_DIR,
     // wxID_SAVE,
     ID_SAVE_XRLE,
     ID_RUN_SCRIPT,
@@ -246,8 +252,8 @@ enum {
     // last 2 items in Run Recent submenu
     ID_CLEAR_MISSING_SCRIPTS = ID_RUN_RECENT + MAX_RECENT + 1,
     ID_CLEAR_ALL_SCRIPTS,
-    ID_SHOW_SCRIPTS,
-    ID_SCRIPT_DIR,
+    ID_SHOW_FILES,
+    ID_FILE_DIR,
     // wxID_PREFERENCES,
     // wxID_EXIT,
     
@@ -338,7 +344,7 @@ enum {
     ID_GRID,
     ID_ICONS,
     ID_INVERT,
-    ID_BUFF,
+    ID_SMARTSCALE,
     ID_TIMELINE,
     ID_INFO,
     
@@ -373,7 +379,7 @@ enum {
     ID_HELP_ALGOS,
     ID_HELP_KEYBOARD,
     ID_HELP_MOUSE,
-    ID_HELP_PERL,
+    ID_HELP_LUA,
     ID_HELP_PYTHON,
     ID_HELP_LEXICON,
     ID_HELP_ARCHIVES,
@@ -391,8 +397,9 @@ enum {
     ID_HELP_CREDITS,
     
     // these ids aren't associated with any menu item
-    ID_LOAD_LEXICON,     // load lexicon pattern
-    ID_HELP_BUTT         // help button in tool bar
+    ID_LOAD_LEXICON,    // for loading a lexicon pattern
+    ID_HELP_BUTT,       // for help button in tool bar
+    ID_GENTIMER         // for gentimer
 };
 
 #endif
