@@ -94,8 +94,8 @@ extern bool showexact;           // show exact numbers in status bar?
 extern bool showtimeline;        // show timeline bar?
 extern bool showgridlines;       // display grid lines?
 extern bool showicons;           // display icons for cell states?
+extern bool smartscale;          // smarter scaling when zoomed out?
 extern bool swapcolors;          // swap colors used for cell states?
-extern bool buffered;            // use wxWdgets buffering to avoid flicker?
 extern bool scrollpencil;        // scroll if pencil cursor is dragged outside view?
 extern bool scrollcross;         // scroll if cross cursor is dragged outside view?
 extern bool scrollhand;          // scroll if hand cursor is dragged outside view?
@@ -131,16 +131,13 @@ extern int mindelay;             // minimum millisec delay
 extern int maxdelay;             // maximum millisec delay
 extern wxString opensavedir;     // directory for Open and Save dialogs
 extern wxString rundir;          // directory for Run Script dialog
-extern wxString icondir;         // directory used by Load Icon File button
 extern wxString choosedir;       // directory used by Choose File button
-extern wxString patterndir;      // directory used by Show Patterns
-extern wxString scriptdir;       // directory used by Show Scripts
+extern wxString filedir;         // directory used by Show Files
 extern wxString texteditor;      // path of user's preferred text editor
 extern wxString perllib;         // name of Perl library (loaded at runtime)
 extern wxString pythonlib;       // name of Python library (loaded at runtime)
-extern int dirwinwd;             // width of pattern/script directory window
-extern bool showpatterns;        // show pattern directory?
-extern bool showscripts;         // show script directory?
+extern int dirwinwd;             // width of file directory window
+extern bool showfiles;           // show file directory?
 extern wxMenu* patternSubMenu;   // submenu of recent pattern files
 extern wxMenu* scriptSubMenu;    // submenu of recent script files
 extern int numpatterns;          // current number of recent pattern files
@@ -157,14 +154,13 @@ extern wxArrayString namedrules;
 // define the actions that can be invoked by various key combinations
 typedef enum {
     DO_NOTHING = 0,               // null action must be zero
-    DO_OPENFILE,                  // open a chosen pattern/script/html file
+    DO_OPENFILE,                  // open a chosen pattern/script/rule/html file
     // rest are in alphabetical order (well, almost)
     DO_ABOUT,                     // about Golly
     DO_ADD,                       // add layer
     DO_ADVANCEOUT,                // advance outside
     DO_ADVANCE,                   // advance selection
     DO_AUTOFIT,                   // auto fit
-    DO_BUFFERED,                  // buffered
     DO_CHANGE00,                  // change origin
     DO_CLEAROUT,                  // clear outside
     DO_CLEAR,                     // clear selection
@@ -230,9 +226,9 @@ typedef enum {
     DO_SW,                        // scroll SW
     DO_SELALL,                    // select all
     DO_SETBASE,                   // set base step...
+    DO_FILEDIR,                   // set file folder...
     DO_SETGEN,                    // set generation...
     DO_SETCOLORS,                 // set layer colors...
-    DO_PATTDIR,                   // set pattern folder...
     DO_SETRULE,                   // set rule...
     DO_SCALE1,                    // set scale 1:1
     DO_SCALE2,                    // set scale 1:2
@@ -240,17 +236,15 @@ typedef enum {
     DO_SCALE8,                    // set scale 1:8
     DO_SCALE16,                   // set scale 1:16
     DO_SCALE32,                   // set scale 1:32
-    DO_SCRIPTDIR,                 // set script folder...
     DO_SHOWSTATES,                // show all states
     DO_SHOWICONS,                 // show cell icons
     DO_SHOWEDIT,                  // show edit bar
     DO_SHOWEXACT,                 // show exact numbers
+    DO_SHOWFILES,                 // show files
     DO_SHOWGRID,                  // show grid lines
     DO_HASHINFO,                  // show hash info
     DO_HELP,                      // show help
     DO_SHOWLAYER,                 // show layer bar
-    DO_PATTERNS,                  // show patterns
-    DO_SCRIPTS,                   // show scripts
     DO_SHOWSTATUS,                // show status bar
     DO_SHOWTIME,                  // show timeline
     DO_TIMING,                    // show timing
@@ -258,6 +252,7 @@ typedef enum {
     DO_SHRINKFIT,                 // shrink and fit
     DO_SHRINK,                    // shrink selection
     DO_SLOWER,                    // slower
+    DO_SMARTSCALE,                // smarter scaling
     DO_STACK,                     // stack layers
     DO_STARTSTOP,                 // start/stop generating
     DO_RECORD,                    // start/stop recording
@@ -297,10 +292,8 @@ wxString GetShortcutTable();
 extern wxColor* borderrgb;       // color for border around bounded grid
 extern wxColor* selectrgb;       // color for selected cells
 extern wxColor* pastergb;        // color for pasted pattern
-extern wxBrush* borderbrush;     // brush for filling grid border
-extern wxPen* pastepen;          // pen for drawing paste rect
 
-void SetBrushesAndPens();        // update colors in above brushes and pens
+void UpdateStatusBrushes();      // update colors in status brushes
 void FreeDefaultColors();        // deallocate memory allocated by CreateDefaultColors()
 
 // Various constants:
@@ -322,10 +315,30 @@ const int MAX_SPACING = 1000;    // maximum value of boldspacing
 const int MIN_MEM_MB = 0;        // minimum value of maximum memory
 const int MAX_MEM_MB =           // maximum value of maximum memory
           sizeof(char*) <= 4 ? 4000 : 100000000;
-const int MAX_BASESTEP = 2000000000;  // maximum base step
+const int MAX_BASESTEP = 2000000000;    // maximum base step
 const int MAX_DELAY = 5000;      // maximum mindelay or maxdelay
 const int MAX_THUMBRANGE = 500;  // maximum thumbrange
-const int MIN_DIRWD = 50;        // minimum dirwinwd
+const int MIN_DIRWD = 10;        // minimum dirwinwd
+
+// Golly uses wxTimers to control the speed of generating patterns, drawing cells,
+// playing timelines, etc.  The value defined below will cause the timers to fire
+// at approximately 60 times per sec (to match the refresh rate of most screens).
+#ifdef __WXMSW__
+    // It seems that wxTimers on Windows only have a resolution of about 15.6ms
+    // so we need to set SIXTY_HERTZ to a value below 16, based on Tim Hutton's
+    // test results (using a 64-bit build of Golly on Windows 10):
+    // interval=5  : 63.9gps = 15.6ms actual interval
+    // interval=10 : 63.9gps = 15.6ms actual interval
+    // interval=15 : 63.9gps = 15.6ms actual interval
+    // interval=16 : 40.0gps = 25.0ms actual interval
+    // interval=17 : 31.9gps = 31.3ms actual interval (= 2 times 15.6ms)
+    // interval=20 : 31.9gps = 31.3ms actual interval
+    // interval=25 : 31.9gps = 31.3ms actual interval
+    const int SIXTY_HERTZ = 15;
+#else
+    // Mac and Linux
+    const int SIXTY_HERTZ = 16;     // 1000/60
+#endif
 
 // Following are used by GetPrefs() before the view window is created:
 
@@ -337,8 +350,8 @@ typedef enum {
     And, Copy, Or, Xor
 } paste_mode;
 
-extern paste_location plocation; // location of cursor in paste rectangle
-extern paste_mode pmode;         // logical paste mode
+extern paste_location plocation;    // location of cursor in paste rectangle
+extern paste_mode pmode;            // logical paste mode
 
 // get/set plocation
 const char* GetPasteLocation();
@@ -350,12 +363,12 @@ void SetPasteMode(const char* s);
 
 // Cursor modes:
 
-extern wxCursor* curs_pencil;    // for drawing cells
-extern wxCursor* curs_pick;      // for picking cell states
-extern wxCursor* curs_cross;     // for selecting cells
-extern wxCursor* curs_hand;      // for moving view by dragging
-extern wxCursor* curs_zoomin;    // for zooming in to a clicked cell
-extern wxCursor* curs_zoomout;   // for zooming out from a clicked cell
+extern wxCursor* curs_pencil;       // for drawing cells
+extern wxCursor* curs_pick;         // for picking cell states
+extern wxCursor* curs_cross;        // for selecting cells
+extern wxCursor* curs_hand;         // for moving view by dragging
+extern wxCursor* curs_zoomin;       // for zooming in to a clicked cell
+extern wxCursor* curs_zoomout;      // for zooming out from a clicked cell
 
 void FreeCursors();
 // deallocate memory allocated by CreateCursors()
