@@ -35,7 +35,6 @@
 #include "wxselect.h"      // for Selection
 #include "wxedit.h"        // for ToggleEditBar, ToggleAllStates, UpdateEditBar
 #include "wxview.h"        // for viewptr->...
-#include "wxrender.h"      // for SetSelectionColor
 #include "wxstatus.h"      // for statusptr->...
 #include "wxutils.h"       // for Warning, Beep, IsScriptFile
 #include "wxprefs.h"       // for gollydir, allowundo, etc
@@ -43,6 +42,7 @@
 #include "wxalgos.h"       // for *_ALGO, algoinfo
 #include "wxlayer.h"       // for currlayer, SyncClones
 #include "wxtimeline.h"    // for TimelineExists
+#include "wxlua.h"         // for RunLuaScript, AbortLuaScript
 #include "wxperl.h"        // for RunPerlScript, AbortPerlScript
 #include "wxpython.h"      // for RunPythonScript, AbortPythonScript
 #include "wxscript.h"
@@ -61,6 +61,7 @@ wxString scripterr;        // Perl/Python error message
 wxString mousepos;         // current mouse position
 
 // local globals:
+static bool luascript = false;      // a Lua script is running?
 static bool plscript = false;       // a Perl script is running?
 static bool pyscript = false;       // a Python script is running?
 static bool showtitle;              // need to update window title?
@@ -76,7 +77,7 @@ void DoAutoUpdate()
 {
     if (autoupdate) {
         inscript = false;
-        mainptr->UpdatePatternAndStatus();
+        mainptr->UpdatePatternAndStatus(true);  // call Update()
         if (showtitle) {
             mainptr->SetWindowTitle(wxEmptyString);
             showtitle = false;
@@ -117,17 +118,16 @@ void ChangeWindowTitle(const wxString& name)
 // They are called by corresponding pl_* and py_* functions in wxperl.cpp
 // and wxpython.cpp respectively.
 
-const char* GSF_open(char* filename, int remember)
+const char* GSF_open(const wxString& filename, int remember)
 {
     // convert non-absolute filename to absolute path relative to scriptloc
-    wxString fname = wxString(filename,wxConvLocal);
-    wxFileName fullname(fname);
-    if (!fullname.IsAbsolute()) fullname = scriptloc + fname;
+    wxFileName fullname(filename);
+    if (!fullname.IsAbsolute()) fullname = scriptloc + filename;
     
     // return error message here if file doesn't exist
     wxString fullpath = fullname.GetFullPath();
     if (!wxFileName::FileExists(fullpath)) {
-        return "Given file does not exist.";
+        return "open error: given file does not exist.";
     }
     
     // only add file to Open Recent submenu if remember flag is non-zero
@@ -139,23 +139,21 @@ const char* GSF_open(char* filename, int remember)
 
 // -----------------------------------------------------------------------------
 
-const char* GSF_save(char* filename, char* format, int remember)
+const char* GSF_save(const wxString& filename, const char* format, int remember)
 {
     // convert non-absolute filename to absolute path relative to scriptloc
-    wxString fname = wxString(filename,wxConvLocal);
-    wxFileName fullname(fname);
-    if (!fullname.IsAbsolute()) fullname = scriptloc + fname;
+    wxFileName fullname(filename);
+    if (!fullname.IsAbsolute()) fullname = scriptloc + filename;
     
     // only add file to Open Recent submenu if remember flag is non-zero
-    return mainptr->SaveFile(fullname.GetFullPath(),
-                             wxString(format,wxConvLocal), remember != 0);
+    return mainptr->SaveFile(fullname.GetFullPath(), wxString(format,wxConvLocal), remember != 0);
 }
 
 // -----------------------------------------------------------------------------
 
-const char* GSF_setdir(char* dirname, char* newdir)
+const char* GSF_setdir(const char* dirname, const wxString& newdir)
 {
-    wxString dirpath = wxString(newdir,wxConvLocal);
+    wxString dirpath = newdir;
     if (dirpath.Last() != wxFILE_SEP_PATH) dirpath += wxFILE_SEP_PATH;
     if (!wxFileName::DirExists(dirpath)) {
         return "New directory does not exist.";
@@ -173,13 +171,10 @@ const char* GSF_setdir(char* dirname, char* newdir)
     } else if (strcmp(dirname, "rules") == 0) {
         userrules = dirpath;
         
-    } else if (strcmp(dirname, "patterns") == 0) {
-        // change patterndir and update panel if currently shown
-        mainptr->SetPatternDir(dirpath);
-        
-    } else if (strcmp(dirname, "scripts") == 0) {
-        // change scriptdir and update panel if currently shown
-        mainptr->SetScriptDir(dirpath);
+    } else if (strcmp(dirname, "files") == 0 ||
+               strcmp(dirname, "patterns") == 0) {  // deprecated
+        // change filedir and update panel if currently shown
+        mainptr->SetFileDir(dirpath);
         
     } else if (strcmp(dirname, "download") == 0) {
         downloaddir = dirpath;
@@ -193,7 +188,7 @@ const char* GSF_setdir(char* dirname, char* newdir)
 
 // -----------------------------------------------------------------------------
 
-const char* GSF_getdir(char* dirname)
+const char* GSF_getdir(const char* dirname)
 {
     wxString dirpath;
     
@@ -201,8 +196,9 @@ const char* GSF_getdir(char* dirname)
     else if (strcmp(dirname, "data") == 0)       dirpath = datadir;
     else if (strcmp(dirname, "temp") == 0)       dirpath = tempdir;
     else if (strcmp(dirname, "rules") == 0)      dirpath = userrules;
-    else if (strcmp(dirname, "patterns") == 0)   dirpath = patterndir;
-    else if (strcmp(dirname, "scripts") == 0)    dirpath = scriptdir;
+    else if (strcmp(dirname, "files") == 0)      dirpath = filedir;
+    else if (strcmp(dirname, "patterns") == 0)   dirpath = filedir;         // deprecated
+    else if (strcmp(dirname, "scripts") == 0)    dirpath = filedir;         // ditto
     else if (strcmp(dirname, "download") == 0)   dirpath = downloaddir;
     else {
         return NULL;   // unknown directory name
@@ -224,10 +220,10 @@ const char* GSF_getdir(char* dirname)
 
 // -----------------------------------------------------------------------------
 
-const char* GSF_setalgo(char* algostring)
+const char* GSF_setalgo(const char* algostring)
 {
     // find index for given algo name
-    char* algoname = ReplaceDeprecatedAlgo(algostring);
+    char* algoname = ReplaceDeprecatedAlgo((char*) algostring);
     algo_type algoindex = -1;
     for (int i = 0; i < NumAlgos(); i++) {
         if (strcmp(algoname, GetAlgoName(i)) == 0) {
@@ -254,7 +250,7 @@ const char* GSF_setalgo(char* algostring)
 
 // -----------------------------------------------------------------------------
 
-const char* GSF_setrule(char* rulestring)
+const char* GSF_setrule(const char* rulestring)
 {
     wxString oldrule = wxString(currlayer->algo->getrule(),wxConvLocal);
     int oldmaxstate = currlayer->algo->NumCellStates() - 1;
@@ -310,6 +306,13 @@ const char* GSF_setrule(char* rulestring)
         // show new rule in main window's title but don't change name
         ChangeWindowTitle(wxEmptyString);
         
+		// if pattern exists and is at starting gen then ensure savestart is true
+		// so that SaveStartingPattern will save pattern to suitable file
+		// (and thus undo/reset will work correctly)
+		if (currlayer->algo->getGeneration() == currlayer->startgen && !currlayer->algo->isEmpty()) {
+			currlayer->savestart = true;
+		}
+        
         // if grid is bounded then remove any live cells outside grid edges
         if (currlayer->algo->gridwd > 0 || currlayer->algo->gridht > 0) {
             mainptr->ClearOutsideGrid();
@@ -338,7 +341,7 @@ const char* GSF_setrule(char* rulestring)
 
 // -----------------------------------------------------------------------------
 
-const char* GSF_setgen(char* genstring)
+const char* GSF_setgen(const char* genstring)
 {
     const char* err = mainptr->ChangeGenCount(genstring);
     if (!err) DoAutoUpdate();
@@ -348,7 +351,7 @@ const char* GSF_setgen(char* genstring)
 
 // -----------------------------------------------------------------------------
 
-const char* GSF_setpos(char* x, char* y)
+const char* GSF_setpos(const char* x, const char* y)
 {
     // disallow alphabetic chars in x,y
     int i;
@@ -367,11 +370,11 @@ const char* GSF_setpos(char* x, char* y)
     
     // check if x,y is outside bounded grid
     if ( (currlayer->algo->gridwd > 0 &&
-          (bigx < currlayer->algo->gridleft || bigx > currlayer->algo->gridright)) ||
-        (currlayer->algo->gridht > 0 &&
-         (bigy < currlayer->algo->gridtop || bigy > currlayer->algo->gridbottom)) ) {
-            return "Given position is outside grid boundary.";
-        }
+            (bigx < currlayer->algo->gridleft || bigx > currlayer->algo->gridright)) ||
+         (currlayer->algo->gridht > 0 &&
+            (bigy < currlayer->algo->gridtop || bigy > currlayer->algo->gridbottom)) ) {
+        return "Given position is outside grid boundary.";
+    }
     
     viewptr->SetPosMag(bigx, bigy, viewptr->GetMag());
     DoAutoUpdate();
@@ -381,9 +384,9 @@ const char* GSF_setpos(char* x, char* y)
 
 // -----------------------------------------------------------------------------
 
-void GSF_setname(char* name, int index)
+void GSF_setname(const wxString& name, int index)
 {
-    if (name == NULL || name[0] == 0) return;
+    if (name.length() == 0) return;
     
     // inscript should be true but play safe
     if (allowundo && !currlayer->stayclean && inscript)
@@ -395,7 +398,7 @@ void GSF_setname(char* name, int index)
         
         // show new name in main window's title;
         // also sets currlayer->currname and updates menu item
-        ChangeWindowTitle(wxString(name,wxConvLocal));
+        ChangeWindowTitle(name);
         
         if (allowundo && !currlayer->stayclean) {
             // note that currfile and savestart/dirty flags don't change
@@ -408,7 +411,7 @@ void GSF_setname(char* name, int index)
         currlayer = GetLayer(index);
         wxString oldname = currlayer->currname;
         
-        currlayer->currname = wxString(name,wxConvLocal);
+        currlayer->currname = name;
         
         if (allowundo && !currlayer->stayclean) {
             // note that currfile and savestart/dirty flags don't change
@@ -430,13 +433,13 @@ const char* GSF_setcell(int x, int y, int newstate)
 {
     // check if x,y is outside bounded grid
     if ( (currlayer->algo->gridwd > 0 &&
-          (x < currlayer->algo->gridleft.toint() ||
-           x > currlayer->algo->gridright.toint())) ||
-        (currlayer->algo->gridht > 0 &&
-         (y < currlayer->algo->gridtop.toint() ||
-          y > currlayer->algo->gridbottom.toint())) ) {
-             return "Given cell is outside grid boundary.";
-         }
+            (x < currlayer->algo->gridleft.toint() ||
+             x > currlayer->algo->gridright.toint())) ||
+         (currlayer->algo->gridht > 0 &&
+            (y < currlayer->algo->gridtop.toint() ||
+             y > currlayer->algo->gridbottom.toint())) ) {
+        return "Given cell is outside grid boundary.";
+    }
     
     int oldstate = currlayer->algo->getcell(x, y);
     if (newstate != oldstate) {
@@ -455,17 +458,17 @@ const char* GSF_setcell(int x, int y, int newstate)
 
 // -----------------------------------------------------------------------------
 
-const char* GSF_paste(int x, int y, char* mode)
+const char* GSF_paste(int x, int y, const char* mode)
 {
     // check if x,y is outside bounded grid
     if ( (currlayer->algo->gridwd > 0 &&
-          (x < currlayer->algo->gridleft.toint() ||
-           x > currlayer->algo->gridright.toint())) ||
-        (currlayer->algo->gridht > 0 &&
-         (y < currlayer->algo->gridtop.toint() ||
-          y > currlayer->algo->gridbottom.toint())) ) {
-             return "Given cell is outside grid boundary.";
-         }
+            (x < currlayer->algo->gridleft.toint() ||
+             x > currlayer->algo->gridright.toint())) ||
+         (currlayer->algo->gridht > 0 &&
+            (y < currlayer->algo->gridtop.toint() ||
+             y > currlayer->algo->gridbottom.toint())) ) {
+        return "Given cell is outside grid boundary.";
+    }
     
     if (!mainptr->ClipboardHasText())
         return "No pattern in clipboard.";
@@ -500,11 +503,11 @@ const char* GSF_checkpos(lifealgo* algo, int x, int y)
 {
     // check that x,y is within bounded grid
     if ( (algo->gridwd > 0 &&
-          (x < algo->gridleft.toint() || x > algo->gridright.toint())) ||
-        (algo->gridht > 0 &&
-         (y < algo->gridtop.toint() || y > algo->gridbottom.toint())) ) {
-            return "Cell is outside grid boundary.";
-        }
+            (x < algo->gridleft.toint() || x > algo->gridright.toint())) ||
+         (algo->gridht > 0 &&
+            (y < algo->gridtop.toint() || y > algo->gridbottom.toint())) ) {
+        return "Cell is outside grid boundary.";
+    }
     return NULL;
 }
 
@@ -517,17 +520,17 @@ const char* GSF_checkrect(int x, int y, int wd, int ht)
     
     // check that rect is completely within bounded grid
     if ( (currlayer->algo->gridwd > 0 &&
-          (x < currlayer->algo->gridleft.toint() ||
-           x > currlayer->algo->gridright.toint() ||
-           x+wd-1 < currlayer->algo->gridleft.toint() ||
-           x+wd-1 > currlayer->algo->gridright.toint())) ||
-        (currlayer->algo->gridht > 0 &&
-         (y < currlayer->algo->gridtop.toint() ||
-          y > currlayer->algo->gridbottom.toint() ||
-          y+ht-1 < currlayer->algo->gridtop.toint() ||
-          y+ht-1 > currlayer->algo->gridbottom.toint())) ) {
-             return "Rectangle is outside grid boundary.";
-         }
+            (x < currlayer->algo->gridleft.toint() ||
+             x > currlayer->algo->gridright.toint() ||
+             x+wd-1 < currlayer->algo->gridleft.toint() ||
+             x+wd-1 > currlayer->algo->gridright.toint())) ||
+         (currlayer->algo->gridht > 0 &&
+            (y < currlayer->algo->gridtop.toint() ||
+             y > currlayer->algo->gridbottom.toint() ||
+             y+ht-1 < currlayer->algo->gridtop.toint() ||
+             y+ht-1 > currlayer->algo->gridbottom.toint())) ) {
+        return "Rectangle is outside grid boundary.";
+    }
     return NULL;
 }
 
@@ -585,7 +588,7 @@ void GSF_select(int x, int y, int wd, int ht)
 
 // -----------------------------------------------------------------------------
 
-bool GSF_setoption(char* optname, int newval, int* oldval)
+bool GSF_setoption(const char* optname, int newval, int* oldval)
 {
     if (strcmp(optname, "autofit") == 0) {
         *oldval = currlayer->autofit ? 1 : 0;
@@ -624,8 +627,7 @@ bool GSF_setoption(char* optname, int newval, int* oldval)
         *oldval = mainptr->fullscreen ? 1 : 0;
         if (*oldval != newval) {
             mainptr->ToggleFullScreen();
-            // above always does an update (due to resizing viewport window)
-            // DoAutoUpdate();
+            DoAutoUpdate();
         }
         
     } else if (strcmp(optname, "hyperspeed") == 0) {
@@ -680,8 +682,7 @@ bool GSF_setoption(char* optname, int newval, int* oldval)
         *oldval = showallstates ? 1 : 0;
         if (*oldval != newval) {
             ToggleAllStates();
-            // above always does an update (due to resizing viewport window)
-            // DoAutoUpdate();
+            DoAutoUpdate();
         }
         
     } else if (strcmp(optname, "showboldlines") == 0) {
@@ -695,16 +696,14 @@ bool GSF_setoption(char* optname, int newval, int* oldval)
         *oldval = showedit ? 1 : 0;
         if (*oldval != newval) {
             ToggleEditBar();
-            // above always does an update (due to resizing viewport window)
-            // DoAutoUpdate();
+            DoAutoUpdate();
         }
         
     } else if (strcmp(optname, "showexact") == 0) {
         *oldval = showexact ? 1 : 0;
         if (*oldval != newval) {
             mainptr->ToggleExactNumbers();
-            // above always does an update (due to resizing viewport window)
-            // DoAutoUpdate();
+            DoAutoUpdate();
         }
         
     } else if (strcmp(optname, "showgrid") == 0) {
@@ -730,40 +729,43 @@ bool GSF_setoption(char* optname, int newval, int* oldval)
         *oldval = showlayer ? 1 : 0;
         if (*oldval != newval) {
             ToggleLayerBar();
-            // above always does an update (due to resizing viewport window)
-            // DoAutoUpdate();
+            DoAutoUpdate();
         }
         
-    } else if (strcmp(optname, "showpatterns") == 0) {
-        *oldval = showpatterns ? 1 : 0;
+    } else if (strcmp(optname, "showfiles") == 0 ||
+               strcmp(optname, "showpatterns") == 0) {      // deprecated
+        *oldval = showfiles ? 1 : 0;
         if (*oldval != newval) {
-            mainptr->ToggleShowPatterns();
-            // above always does an update (due to resizing viewport window)
-            // DoAutoUpdate();
+            mainptr->ToggleShowFiles();
+            DoAutoUpdate();
         }
         
     } else if (strcmp(optname, "showscripts") == 0) {
-        *oldval = showscripts ? 1 : 0;
+        *oldval = 0;
         if (*oldval != newval) {
-            mainptr->ToggleShowScripts();
-            // above always does an update (due to resizing viewport window)
-            // DoAutoUpdate();
-        }
-        
-    } else if (strcmp(optname, "showtoolbar") == 0) {
-        *oldval = showtool ? 1 : 0;
-        if (*oldval != newval) {
-            mainptr->ToggleToolBar();
-            // above always does an update (due to resizing viewport window)
-            // DoAutoUpdate();
+            // deprecated so do nothing
+            DoAutoUpdate();
         }
         
     } else if (strcmp(optname, "showstatusbar") == 0) {
         *oldval = showstatus ? 1 : 0;
         if (*oldval != newval) {
             mainptr->ToggleStatusBar();
-            // above always does an update (due to resizing viewport window)
-            // DoAutoUpdate();
+            DoAutoUpdate();
+        }
+        
+    } else if (strcmp(optname, "showtoolbar") == 0) {
+        *oldval = showtool ? 1 : 0;
+        if (*oldval != newval) {
+            mainptr->ToggleToolBar();
+            DoAutoUpdate();
+        }
+        
+    } else if (strcmp(optname, "smartscale") == 0) {
+        *oldval = smartscale ? 1 : 0;
+        if (*oldval != newval) {
+            viewptr->ToggleSmarterScaling();
+            DoAutoUpdate();
         }
         
     } else if (strcmp(optname, "swapcolors") == 0) {
@@ -798,16 +800,14 @@ bool GSF_setoption(char* optname, int newval, int* oldval)
         *oldval = stacklayers ? 1 : 0;
         if (*oldval != newval) {
             ToggleStackLayers();
-            // above always does an update
-            // DoAutoUpdate();
+            DoAutoUpdate();
         }
         
     } else if (strcmp(optname, "tilelayers") == 0) {
         *oldval = tilelayers ? 1 : 0;
         if (*oldval != newval) {
             ToggleTileLayers();
-            // above always does an update
-            // DoAutoUpdate();
+            DoAutoUpdate();
         }
         
         // this option is deprecated (use setalgo command)
@@ -832,7 +832,7 @@ bool GSF_setoption(char* optname, int newval, int* oldval)
 
 // -----------------------------------------------------------------------------
 
-bool GSF_getoption(char* optname, int* optval)
+bool GSF_getoption(const char* optname, int* optval)
 {
     if      (strcmp(optname, "autofit") == 0)       *optval = currlayer->autofit ? 1 : 0;
     else if (strcmp(optname, "boldspacing") == 0)   *optval = boldspacing;
@@ -852,10 +852,12 @@ bool GSF_getoption(char* optname, int* optval)
     else if (strcmp(optname, "showhashinfo") == 0)  *optval = currlayer->showhashinfo ? 1 : 0;
     else if (strcmp(optname, "showicons") == 0)     *optval = showicons ? 1 : 0;
     else if (strcmp(optname, "showlayerbar") == 0)  *optval = showlayer ? 1 : 0;
-    else if (strcmp(optname, "showpatterns") == 0)  *optval = showpatterns ? 1 : 0;
-    else if (strcmp(optname, "showscripts") == 0)   *optval = showscripts ? 1 : 0;
+    else if (strcmp(optname, "showfiles") == 0)     *optval = showfiles ? 1 : 0;
+    else if (strcmp(optname, "showpatterns") == 0)  *optval = showfiles ? 1 : 0;    // deprecated
+    else if (strcmp(optname, "showscripts") == 0)   *optval = 0;                    // ditto
     else if (strcmp(optname, "showstatusbar") == 0) *optval = showstatus ? 1 : 0;
     else if (strcmp(optname, "showtoolbar") == 0)   *optval = showtool ? 1 : 0;
+    else if (strcmp(optname, "smartscale") == 0)    *optval = smartscale ? 1 : 0;
     else if (strcmp(optname, "stacklayers") == 0)   *optval = stacklayers ? 1 : 0;
     else if (strcmp(optname, "swapcolors") == 0)    *optval = swapcolors ? 1 : 0;
     else if (strcmp(optname, "switchlayers") == 0)  *optval = canswitch ? 1 : 0;
@@ -874,7 +876,7 @@ bool GSF_getoption(char* optname, int* optval)
 
 // -----------------------------------------------------------------------------
 
-bool GSF_setcolor(char* colname, wxColor& newcol, wxColor& oldcol)
+bool GSF_setcolor(const char* colname, wxColor& newcol, wxColor& oldcol)
 {
     if (strncmp(colname, "livecells", 9) == 0) {
         // livecells0..livecells9 are deprecated; get and set color of state 1
@@ -883,6 +885,7 @@ bool GSF_setcolor(char* colname, wxColor& newcol, wxColor& oldcol)
             currlayer->cellr[1] = newcol.Red();
             currlayer->cellg[1] = newcol.Green();
             currlayer->cellb[1] = newcol.Blue();
+            UpdateIconColors();
             UpdateCloneColors();
             DoAutoUpdate();
         }
@@ -894,6 +897,7 @@ bool GSF_setcolor(char* colname, wxColor& newcol, wxColor& oldcol)
             currlayer->cellr[0] = newcol.Red();
             currlayer->cellg[0] = newcol.Green();
             currlayer->cellb[0] = newcol.Blue();
+            UpdateIconColors();
             UpdateCloneColors();
             DoAutoUpdate();
         }
@@ -902,7 +906,6 @@ bool GSF_setcolor(char* colname, wxColor& newcol, wxColor& oldcol)
         oldcol = *borderrgb;
         if (oldcol != newcol) {
             *borderrgb = newcol;
-            SetBrushesAndPens();
             DoAutoUpdate();
         }
         
@@ -910,7 +913,6 @@ bool GSF_setcolor(char* colname, wxColor& newcol, wxColor& oldcol)
         oldcol = *pastergb;
         if (oldcol != newcol) {
             *pastergb = newcol;
-            SetBrushesAndPens();
             DoAutoUpdate();
         }
         
@@ -918,8 +920,6 @@ bool GSF_setcolor(char* colname, wxColor& newcol, wxColor& oldcol)
         oldcol = *selectrgb;
         if (oldcol != newcol) {
             *selectrgb = newcol;
-            SetBrushesAndPens();
-            SetSelectionColor();    // see wxrender.cpp
             DoAutoUpdate();
         }
         
@@ -927,7 +927,7 @@ bool GSF_setcolor(char* colname, wxColor& newcol, wxColor& oldcol)
         oldcol = algoinfo[HLIFE_ALGO]->statusrgb;
         if (oldcol != newcol) {
             algoinfo[HLIFE_ALGO]->statusrgb = newcol;
-            SetBrushesAndPens();
+            UpdateStatusBrushes();
             DoAutoUpdate();
         }
         
@@ -935,19 +935,19 @@ bool GSF_setcolor(char* colname, wxColor& newcol, wxColor& oldcol)
         oldcol = algoinfo[QLIFE_ALGO]->statusrgb;
         if (oldcol != newcol) {
             algoinfo[QLIFE_ALGO]->statusrgb = newcol;
-            SetBrushesAndPens();
+            UpdateStatusBrushes();
             DoAutoUpdate();
         }
         
     } else {
         // look for algo name
-        char* algoname = ReplaceDeprecatedAlgo(colname);
+        char* algoname = ReplaceDeprecatedAlgo((char*) colname);
         for (int i = 0; i < NumAlgos(); i++) {
             if (strcmp(algoname, GetAlgoName(i)) == 0) {
                 oldcol = algoinfo[i]->statusrgb;
                 if (oldcol != newcol) {
                     algoinfo[i]->statusrgb = newcol;
-                    SetBrushesAndPens();
+                    UpdateStatusBrushes();
                     DoAutoUpdate();
                 }
                 return true;
@@ -961,7 +961,7 @@ bool GSF_setcolor(char* colname, wxColor& newcol, wxColor& oldcol)
 
 // -----------------------------------------------------------------------------
 
-bool GSF_getcolor(char* colname, wxColor& color)
+bool GSF_getcolor(const char* colname, wxColor& color)
 {
     if (strncmp(colname, "livecells", 9) == 0) {
         // livecells0..livecells9 are deprecated; return color of state 1
@@ -979,7 +979,7 @@ bool GSF_getcolor(char* colname, wxColor& color)
     else if (strcmp(colname, "nothashing") == 0) color = algoinfo[QLIFE_ALGO]->statusrgb;
     else {
         // look for algo name
-        char* algoname = ReplaceDeprecatedAlgo(colname);
+        char* algoname = ReplaceDeprecatedAlgo((char*) colname);
         for (int i = 0; i < NumAlgos(); i++) {
             if (strcmp(algoname, GetAlgoName(i)) == 0) {
                 color = algoinfo[i]->statusrgb;
@@ -1157,18 +1157,18 @@ const char* GSF_doevent(const wxString& event)
 
 // the following is deprecated (use GSF_getevent)
 
-void GSF_getkey(char* s)
+char GSF_getkey()
 {
     passkeys = true;   // future keyboard events will call PassKeyToScript
     
     if (scriptchars.length() == 0) {
         // return empty string
-        s[0] = '\0';
+        return '\0';
     } else {
         // return first char in scriptchars and then remove it
-        s[0] = scriptchars.GetChar(0);
-        s[1] = '\0';
-        scriptchars = scriptchars.AfterFirst(s[0]);
+        char ch = scriptchars.GetChar(0);
+        scriptchars = scriptchars.AfterFirst(ch);
+        return ch;
     }
 }
 
@@ -1176,7 +1176,7 @@ void GSF_getkey(char* s)
 
 // the following is deprecated (use GSF_doevent)
 
-void GSF_dokey(char* ascii)
+void GSF_dokey(const char* ascii)
 {
     if (*ascii) {
         // convert ascii char to corresponding wx key code;
@@ -1215,7 +1215,8 @@ void GSF_update()
     // update viewport, status bar, and possibly other bars
     inscript = false;
     
-    mainptr->UpdatePatternAndStatus();
+    // pass in true so that Update() is called
+    mainptr->UpdatePatternAndStatus(true);
     
     if (showtitle) {
         mainptr->SetWindowTitle(wxEmptyString);
@@ -1232,12 +1233,12 @@ void GSF_update()
 
 // -----------------------------------------------------------------------------
 
-void GSF_exit(char* errmsg)
+void GSF_exit(const wxString& errmsg)
 {
-    if (errmsg && errmsg[0] != 0) {
+    if (!errmsg.IsEmpty()) {
         // display given error message
         inscript = false;
-        statusptr->ErrorMessage(wxString(errmsg,wxConvLocal));
+        statusptr->ErrorMessage(errmsg);
         inscript = true;
         // make sure status bar is visible
         if (!showstatus) mainptr->ToggleStatusBar();
@@ -1253,11 +1254,13 @@ void CheckScriptError(const wxString& ext)
     if (scripterr.IsEmpty()) return;    // no error
     
     if (scripterr.Find(wxString(abortmsg,wxConvLocal)) >= 0) {
-        // error was caused by AbortPerlScript/AbortPythonScript
+        // error was caused by AbortLuaScript/AbortPerlScript/AbortPythonScript
         // so don't display scripterr
     } else {
         wxString errtype;
-        if (ext.IsSameAs(wxT("pl"), false)) {
+        if (ext.IsSameAs(wxT("lua"), false)) {
+            errtype = _("Lua error:");
+        } else if (ext.IsSameAs(wxT("pl"), false)) {
             errtype = _("Perl error:");
             scripterr.Replace(wxT(". at "), wxT("\nat "));
         } else {
@@ -1265,9 +1268,9 @@ void CheckScriptError(const wxString& ext)
             scripterr.Replace(wxT("  File \"<string>\", line 1, in ?\n"), wxT(""));
         }
         Beep();
-#ifdef __WXMAC__
-        wxSetCursor(*wxSTANDARD_CURSOR);
-#endif
+        #ifdef __WXMAC__
+            wxSetCursor(*wxSTANDARD_CURSOR);
+        #endif
         wxMessageBox(scripterr, errtype, wxOK | wxICON_EXCLAMATION, wxGetActiveWindow());
     }
     
@@ -1327,6 +1330,7 @@ void RunScript(const wxString& filename)
     
     // use these flags to allow re-entrancy
     bool already_inscript = inscript;
+    bool in_luascript = luascript;
     bool in_plscript = plscript;
     bool in_pyscript = pyscript;
     wxString savecwd;
@@ -1365,10 +1369,10 @@ void RunScript(const wxString& filename)
     wxSetWorkingDirectory(scriptloc);
     
     wxString fpath = fullname.GetFullPath();
-#ifdef __WXMAC__
-    // use decomposed UTF8 so interpreter can open names with non-ASCII chars
-    fpath = wxString(fpath.fn_str(),wxConvLocal);
-#endif
+    #ifdef __WXMAC__
+        // use decomposed UTF8 so interpreter can open names with non-ASCII chars
+        fpath = wxString(fpath.fn_str(),wxConvLocal);
+    #endif
     
     if (!already_inscript) {
         if (allowundo) {
@@ -1398,7 +1402,10 @@ void RunScript(const wxString& filename)
     }
     
     wxString ext = filename.AfterLast('.');
-    if (ext.IsSameAs(wxT("pl"), false)) {
+    if (ext.IsSameAs(wxT("lua"), false)) {
+        luascript = true;
+        RunLuaScript(fpath);
+    } else if (ext.IsSameAs(wxT("pl"), false)) {
         plscript = true;
         RunPerlScript(fpath);
     } else if (ext.IsSameAs(wxT("py"), false)) {
@@ -1406,6 +1413,7 @@ void RunScript(const wxString& filename)
         RunPythonScript(fpath);
     } else {
         // should never happen
+        luascript = false;
         plscript = false;
         pyscript = false;
         Warning(_("Unexpected extension in script file:\n") + filename);
@@ -1416,10 +1424,13 @@ void RunScript(const wxString& filename)
         scriptloc = savecwd;
         wxSetWorkingDirectory(scriptloc);
         
-        // display any Perl/Python error message
+        // display any Lua/Perl/Python error message
         CheckScriptError(ext);
         if (!scripterr.IsEmpty()) {
-            if (in_pyscript) {
+            if (in_luascript) {
+                // abort the calling Lua script
+                AbortLuaScript();
+            } else if (in_pyscript) {
                 // abort the calling Python script
                 AbortPythonScript();
             } else if (in_plscript) {
@@ -1428,6 +1439,7 @@ void RunScript(const wxString& filename)
             }
         }
         
+        luascript = in_luascript;
         plscript = in_plscript;
         pyscript = in_pyscript;
         
@@ -1475,13 +1487,14 @@ void RunScript(const wxString& filename)
         // restore current directory to location of Golly app
         wxSetWorkingDirectory(gollydir);
         
+        luascript = false;
         plscript = false;
         pyscript = false;
         
         // update Undo/Redo items based on current layer's history
         if (allowundo) currlayer->undoredo->UpdateUndoRedoItems();
         
-        // display any Perl/Python error message
+        // display any error message
         CheckScriptError(ext);
         
         // update title, menu bar, cursor, viewport, status bar, tool bar, etc
@@ -1519,6 +1532,7 @@ void PassKeyToScript(int key, int modifiers)
             // interrupt a run() or step() command
             wxGetApp().PollerInterrupt();
         }
+        if (luascript) AbortLuaScript();
         if (plscript) AbortPerlScript();
         if (pyscript) AbortPythonScript();
     } else {
@@ -1608,12 +1622,14 @@ void FinishScripting()
             // interrupt a run() or step() command
             wxGetApp().PollerInterrupt();
         }
+        if (luascript) AbortLuaScript();
         if (plscript) AbortPerlScript();
         if (pyscript) AbortPythonScript();
         wxSetWorkingDirectory(gollydir);
         inscript = false;
     }
     
+    FinishLuaScripting();
     FinishPerlScripting();
     FinishPythonScripting();
 }
