@@ -295,9 +295,9 @@ void Selection::Advance()
         bigint savegen = currlayer->algo->getGeneration();
         bigint saveinc = currlayer->algo->getIncrement();
         currlayer->algo->setIncrement(1);
-        if (boundedgrid) mainptr->CreateBorderCells(currlayer->algo);
+        if (boundedgrid) currlayer->algo->CreateBorderCells();
         currlayer->algo->step();
-        if (boundedgrid) mainptr->DeleteBorderCells(currlayer->algo);
+        if (boundedgrid) currlayer->algo->DeleteBorderCells();
         currlayer->algo->setIncrement(saveinc);
         currlayer->algo->setGeneration(savegen);
         
@@ -344,9 +344,9 @@ void Selection::Advance()
     mainptr->generating = true;
     wxGetApp().PollerReset();
     tempalgo->setIncrement(1);
-    if (boundedgrid) mainptr->CreateBorderCells(tempalgo);
+    if (boundedgrid) tempalgo->CreateBorderCells();
     tempalgo->step();
-    if (boundedgrid) mainptr->DeleteBorderCells(tempalgo);
+    if (boundedgrid) tempalgo->DeleteBorderCells();
     mainptr->generating = false;
     
     if ( !tempalgo->isEmpty() ) {
@@ -433,9 +433,9 @@ void Selection::AdvanceOutside()
         bigint savegen = currlayer->algo->getGeneration();
         bigint saveinc = currlayer->algo->getIncrement();
         currlayer->algo->setIncrement(1);
-        if (boundedgrid) mainptr->CreateBorderCells(currlayer->algo);
+        if (boundedgrid) currlayer->algo->CreateBorderCells();
         currlayer->algo->step();
-        if (boundedgrid) mainptr->DeleteBorderCells(currlayer->algo);
+        if (boundedgrid) currlayer->algo->DeleteBorderCells();
         currlayer->algo->setIncrement(saveinc);
         currlayer->algo->setGeneration(savegen);
         
@@ -500,9 +500,9 @@ void Selection::AdvanceOutside()
     mainptr->generating = true;
     wxGetApp().PollerReset();
     currlayer->algo->setIncrement(1);
-    if (boundedgrid) mainptr->CreateBorderCells(currlayer->algo);
+    if (boundedgrid) currlayer->algo->CreateBorderCells();
     currlayer->algo->step();
-    if (boundedgrid) mainptr->DeleteBorderCells(currlayer->algo);
+    if (boundedgrid) currlayer->algo->DeleteBorderCells();
     mainptr->generating = false;
     
     if ( !currlayer->algo->isEmpty() ) {
@@ -799,22 +799,25 @@ void Selection::Fit()
 
 // -----------------------------------------------------------------------------
 
-void Selection::Shrink(bool fit)
+void Selection::Shrink(bool fit, bool remove_if_empty)
 {
     if (!exists) return;
     
     if (mainptr->generating) {
-        // terminate generating loop and set command_pending flag
-        mainptr->Stop();
         mainptr->command_pending = true;
         mainptr->cmdevent.SetId(fit ? ID_SHRINKFIT : ID_SHRINK);
+        mainptr->Stop();
         return;
     }
     
     // check if there is no pattern
     if (currlayer->algo->isEmpty()) {
-        statusptr->ErrorMessage(empty_selection);
-        if (fit) viewptr->FitSelection();
+        if (remove_if_empty) {
+            viewptr->RemoveSelection();
+        } else {
+            statusptr->ErrorMessage(empty_selection);
+            if (fit) viewptr->FitSelection();
+        }
         return;
     }
     
@@ -831,7 +834,7 @@ void Selection::Shrink(bool fit)
         viewptr->RememberNewSelection(_("Shrink Selection"));
         viewptr->DisplaySelectionSize();
         if (fit)
-            viewptr->FitSelection();   // calls UpdateEverything
+            viewptr->FitSelection();
         else
             mainptr->UpdatePatternAndStatus();
         return;
@@ -839,8 +842,12 @@ void Selection::Shrink(bool fit)
     
     // check if selection is completely outside pattern edges
     if (Outside(top, left, bottom, right)) {
-        statusptr->ErrorMessage(empty_selection);
-        if (fit) viewptr->FitSelection();
+        if (remove_if_empty) {
+            viewptr->RemoveSelection();
+        } else {
+            statusptr->ErrorMessage(empty_selection);
+            if (fit) viewptr->FitSelection();
+        }
         return;
     }
     
@@ -884,8 +891,14 @@ void Selection::Shrink(bool fit)
     // copy live cells in selection to temporary universe
     if ( viewptr->CopyRect(top.toint(), left.toint(), bottom.toint(), right.toint(),
                            currlayer->algo, tempalgo, false, _("Copying selection")) ) {
-        if ( tempalgo->isEmpty() ) {
-            statusptr->ErrorMessage(empty_selection);
+        if (tempalgo->isEmpty()) {
+            if (remove_if_empty) {
+                viewptr->RemoveSelection();
+                delete tempalgo;
+                return;
+            } else {
+                statusptr->ErrorMessage(empty_selection);
+            }
         } else {
             viewptr->SaveCurrentSelection();
             tempalgo->findedges(&seltop, &selleft, &selbottom, &selright);
@@ -981,10 +994,9 @@ void Selection::Clear()
     if (currlayer->algo->isEmpty()) return;
     
     if (mainptr->generating) {
-        // terminate generating loop and set command_pending flag
-        mainptr->Stop();
         mainptr->command_pending = true;
         mainptr->cmdevent.SetId(ID_CLEAR);
+        mainptr->Stop();
         return;
     }
     
@@ -1154,10 +1166,9 @@ void Selection::ClearOutside()
     }
     
     if (mainptr->generating) {
-        // terminate generating loop and set command_pending flag
-        mainptr->Stop();
         mainptr->command_pending = true;
         mainptr->cmdevent.SetId(ID_OUTSIDE);
+        mainptr->Stop();
         return;
     }
     
@@ -1484,10 +1495,9 @@ void Selection::RandomFill()
     }
     
     if (mainptr->generating) {
-        // terminate generating loop and set command_pending flag
-        mainptr->Stop();
         mainptr->command_pending = true;
         mainptr->cmdevent.SetId(ID_RANDOM);
+        mainptr->Stop();
         return;
     }
     
@@ -1642,10 +1652,9 @@ bool Selection::Flip(bool topbottom, bool inundoredo)
     if (!exists) return false;
     
     if (mainptr->generating) {
-        // terminate generating loop and set command_pending flag
-        mainptr->Stop();
         mainptr->command_pending = true;
         mainptr->cmdevent.SetId(topbottom ? ID_FLIPTB : ID_FLIPLR);
+        mainptr->Stop();
         return true;
     }
     
@@ -1942,10 +1951,9 @@ bool Selection::Rotate(bool clockwise, bool inundoredo)
     if (!exists) return false;
     
     if (mainptr->generating) {
-        // terminate generating loop and set command_pending flag
-        mainptr->Stop();
         mainptr->command_pending = true;
         mainptr->cmdevent.SetId(clockwise ? ID_ROTATEC : ID_ROTATEA);
+        mainptr->Stop();
         return true;
     }
     
