@@ -108,8 +108,8 @@ void StatusBar::DisplayMessage(const wxString& s)
             wxRect r = wxRect( wxPoint(0,statusht-BOTGAP+DESCHT-LINEHT),
                               wxPoint(wd-1,ht-1) );
             Refresh(false, &r);
-            // show message immediately
-            Update();
+            // no need to show message immediately???
+            // Update();
         }
     }
 }
@@ -146,7 +146,6 @@ void StatusBar::UpdateXYLocation()
         else
             r = wxRect( wxPoint(h_xy, 0), wxPoint(wd-1, BASELINE1+DESCHT) );
         Refresh(false, &r);
-        // no need to Update() immediately
     }
 }
 
@@ -543,7 +542,6 @@ void StatusBar::OnMouseDown(wxMouseEvent& event)
             mainptr->SetStepExponent(0);
             // update status bar
             Refresh(false);
-            Update();
         }
     }
 }
@@ -563,8 +561,10 @@ StatusBar::StatusBar(wxWindow* parent, wxCoord xorg, wxCoord yorg, int wd, int h
 : wxWindow(parent, wxID_ANY, wxPoint(xorg,yorg), wxSize(wd,ht),
            wxNO_BORDER | wxFULL_REPAINT_ON_RESIZE)
 {
+#ifdef __WXGTK__
     // avoid erasing background on GTK+
     SetBackgroundStyle(wxBG_STYLE_CUSTOM);
+#endif
     
     // create font for text in status bar and set textascent for use in DisplayText
 #ifdef __WXMSW__
@@ -588,7 +588,6 @@ StatusBar::StatusBar(wxWindow* parent, wxCoord xorg, wxCoord yorg, int wd, int h
     statusfont = wxFont::New(10, wxMODERN, wxNORMAL, wxNORMAL, false, wxT("Monaco"));
     textascent = 10;
 #else
-    // __WXMAC__
     statusfont = wxFont::New(10, wxMODERN, wxNORMAL, wxNORMAL);
     textascent = 10;
 #endif
