@@ -48,7 +48,154 @@
 
 const wxString lack_of_memory = _("Due to lack of memory, some changes can't be undone!");
 const wxString to_gen = _("to Gen ");
-const wxString temp_prefix = wxT("golly_undo_");
+
+// the following prefixes are used when creating temporary file names
+// (only use 3 characters because longer strings are truncated on Windows)
+const wxString genchange_prefix = wxT("gg_");
+const wxString setgen_prefix = wxT("gs_");
+const wxString dupe1_prefix = wxT("g1_");
+const wxString dupe2_prefix = wxT("g2_");
+const wxString dupe3_prefix = wxT("g3_");
+const wxString dupe4_prefix = wxT("g4_");
+const wxString dupe5_prefix = wxT("g5_");
+const wxString dupe6_prefix = wxT("g6_");
+
+// -----------------------------------------------------------------------------
+
+// the next two classes are needed because Golly allows multiple starting points
+// (by setting the generation count back to 0), so we need to ensure that a Reset
+// goes back to the correct starting info
+
+class VariableInfo {
+public:
+    VariableInfo() {}
+    ~VariableInfo() {}
+    // note that we have to remember pointer to layer and not its index
+    // (the latter can change if user adds/deletes/moves a layer)
+    Layer* layerptr;
+    wxString savename;
+    bigint savex;
+    bigint savey;
+    int savemag;
+    int savebase;
+    int saveexpo;
+};
+
+class StartingInfo {
+public:
+    StartingInfo(StartingInfo* dupe, Layer* oldlayer, Layer* newlayer);
+    ~StartingInfo();
+    void Restore();
+    void RemoveClone(Layer* cloneptr);
+
+    // this info is the same in each clone
+    bool savedirty;
+    algo_type savealgo;
+    wxString saverule;
+    
+    // this info can be different in each clone
+    VariableInfo layer[MAX_LAYERS];
+    int count;
+};
+
+// -----------------------------------------------------------------------------
+
+StartingInfo::StartingInfo(StartingInfo* dupe, Layer* oldlayer, Layer* newlayer)
+{
+    if (dupe) {
+        // called from DuplicateHistory so duplicate given starting info
+        savedirty = dupe->savedirty;
+        savealgo = dupe->savealgo;
+        saverule = dupe->saverule;
+        
+        // new duplicate layer is not a clone
+        count = 0;
+        for (int n = 0; n < dupe->count; n++) {
+            if (dupe->layer[n].layerptr == oldlayer) {
+                layer[0].layerptr = newlayer;
+                layer[0].savename = dupe->layer[n].savename;
+                layer[0].savex = dupe->layer[n].savex;
+                layer[0].savey = dupe->layer[n].savey;
+                layer[0].savemag = dupe->layer[n].savemag;
+                layer[0].savebase = dupe->layer[n].savebase;
+                layer[0].saveexpo = dupe->layer[n].saveexpo;
+                count = 1;
+                break;
+            }
+        }
+
+    } else {
+        // save current starting info (set by most recent SaveStartingPattern)
+        savedirty = currlayer->startdirty;
+        savealgo = currlayer->startalgo;
+        saverule = currlayer->startrule;
+        
+        // save variable info for currlayer and its clones (if any)
+        count = 0;
+        for (int i = 0; i < numlayers; i++) {
+            Layer* lptr = GetLayer(i);
+            if (lptr == currlayer ||
+                (lptr->cloneid > 0 && lptr->cloneid == currlayer->cloneid)) {
+                layer[count].layerptr = lptr;
+                layer[count].savename = lptr->startname;
+                layer[count].savex = lptr->startx;
+                layer[count].savey = lptr->starty;
+                layer[count].savemag = lptr->startmag;
+                layer[count].savebase = lptr->startbase;
+                layer[count].saveexpo = lptr->startexpo;
+                count++;
+            }
+        }
+        if (count == 0) Warning(_("Bug detected in StartingInfo ctor!"));
+    }
+}
+
+// -----------------------------------------------------------------------------
+
+StartingInfo::~StartingInfo()
+{
+    // no need to do anything
+}
+
+// -----------------------------------------------------------------------------
+
+void StartingInfo::Restore()
+{
+    // restore starting info (for use by next ResetPattern)
+    currlayer->startdirty = savedirty;
+    currlayer->startalgo = savealgo;
+    currlayer->startrule = saverule;
+    
+    // restore variable info for currlayer and its clones (if any);
+    // note that currlayer might have changed since the starting info
+    // was saved, and there might be more or fewer clones
+    for (int i = 0; i < numlayers; i++) {
+        Layer* lptr = GetLayer(i);
+        for (int n = 0; n < count; n++) {
+            if (lptr == layer[n].layerptr) {
+                lptr->startname = layer[n].savename;
+                lptr->startx = layer[n].savex;
+                lptr->starty = layer[n].savey;
+                lptr->startmag = layer[n].savemag;
+                lptr->startbase = layer[n].savebase;
+                lptr->startexpo = layer[n].saveexpo;
+                break;
+            }
+        }
+    }
+}
+
+// -----------------------------------------------------------------------------
+
+void StartingInfo::RemoveClone(Layer* cloneptr)
+{
+    for (int n = 0; n < count; n++) {
+        if (layer[n].layerptr == cloneptr) {
+            layer[n].layerptr = NULL;
+            return;
+        }
+    }
+}
 
 // -----------------------------------------------------------------------------
 
@@ -89,56 +236,49 @@ public:
     void ChangeCells(bool undo);
     // change cell states using cellinfo
     
-    change_type changeid;                  // specifies the type of change
-    wxString suffix;                       // action string for Undo/Redo item
-    bool olddirty;                         // layer's dirty state before change
-    bool newdirty;                         // layer's dirty state after change
+    change_type changeid;                   // specifies the type of change
+    wxString suffix;                        // action string for Undo/Redo item
+    bool olddirty;                          // layer's dirty state before change
+    bool newdirty;                          // layer's dirty state after change
     
     // cellstates info
-    cell_change* cellinfo;                 // dynamic array of cell changes
-    unsigned int cellcount;                // number of cell changes in array
+    cell_change* cellinfo;                  // dynamic array of cell changes
+    unsigned int cellcount;                 // number of cell changes in array
     
     // rotatecw/rotateacw/selchange info
-    Selection oldsel, newsel;              // old and new selections
+    Selection oldsel, newsel;               // old and new selections
     
     // genchange info
-    wxString oldfile, newfile;             // old and new pattern files
-    bigint oldgen, newgen;                 // old and new generation counts
-    bigint oldx, oldy, newx, newy;         // old and new positions
-    int oldmag, newmag;                    // old and new scales
-    int oldbase, newbase;                  // old and new base steps
-    int oldexpo, newexpo;                  // old and new step exponents
-    bool scriptgen;                        // gen change was done by script?
+    bool scriptgen;                         // gen change was done by script?
+    wxString oldfile, newfile;              // old and new pattern files
+    bigint oldgen, newgen;                  // old and new generation counts
+    bigint oldx, oldy, newx, newy;          // old and new positions
+    int oldmag, newmag;                     // old and new scales
+    int oldbase, newbase;                   // old and new base steps
+    int oldexpo, newexpo;                   // old and new step exponents
+    StartingInfo* startinfo;                // saves starting info for ResetPattern
     // also uses oldsel, newsel
     
     // setgen info
-    bigint oldstartgen, newstartgen;       // old and new startgen values
-    bool oldsave, newsave;                 // old and new savestart states
-    wxString oldtempstart, newtempstart;   // old and new tempstart paths
-    wxString oldstartfile, newstartfile;   // old and new startfile paths
-    wxString oldcurrfile, newcurrfile;     // old and new currfile paths
-    wxString oldclone[MAX_LAYERS];         // old starting names for cloned layers
-    wxString newclone[MAX_LAYERS];         // new starting names for cloned layers
+    bigint oldstartgen, newstartgen;        // old and new startgen values
+    bool oldsave, newsave;                  // old and new savestart states
+    wxString oldtempstart, newtempstart;    // old and new tempstart paths
+    wxString oldcurrfile, newcurrfile;      // old and new currfile paths
     // also uses oldgen, newgen
-    // and oldrule, newrule
-    // and oldx, oldy, newx, newy, oldmag, newmag
-    // and oldbase, newbase
-    // and oldexpo, newexpo
-    // and oldsel, newsel
-    // and oldalgo, newalgo
+    // and startinfo
     
     // namechange info
-    wxString oldname, newname;             // old and new layer names
-    Layer* whichlayer;                     // which layer was changed
+    wxString oldname, newname;              // old and new layer names
+    Layer* whichlayer;                      // which layer was changed
     // also uses oldsave, newsave
     // and oldcurrfile, newcurrfile
     
     // rulechange info
-    wxString oldrule, newrule;             // old and new rules
+    wxString oldrule, newrule;              // old and new rules
     // also uses oldsel, newsel
     
     // algochange info
-    algo_type oldalgo, newalgo;            // old and new algorithm types
+    algo_type oldalgo, newalgo;             // old and new algorithm types
     // also uses oldrule, newrule
     // and oldsel, newsel
 };
@@ -148,40 +288,65 @@ public:
 ChangeNode::ChangeNode(change_type id)
 {
     changeid = id;
+    startinfo = NULL;
+    whichlayer = NULL;      // simplifies UndoRedo::DeletingClone
     cellinfo = NULL;
     cellcount = 0;
     oldfile = wxEmptyString;
     newfile = wxEmptyString;
     oldtempstart = wxEmptyString;
     newtempstart = wxEmptyString;
+    oldcurrfile = wxEmptyString;
+    newcurrfile = wxEmptyString;
 }
 
 // -----------------------------------------------------------------------------
 
+bool delete_all_temps = false;   // ok to delete all temporary files?
+
 ChangeNode::~ChangeNode()
 {
+    if (startinfo) delete startinfo;
     if (cellinfo) free(cellinfo);
     
+    // it's always ok to delete oldfile and newfile if they exist
+    
     if (!oldfile.IsEmpty() && wxFileExists(oldfile)) {
         wxRemoveFile(oldfile);
-        //printf("removed oldfile: %s\n", (const char*)oldfile.mb_str(wxConvLocal));
-    }
-    if (!newfile.IsEmpty() && wxFileExists(newfile)) {
-        wxRemoveFile(newfile);
-        //printf("removed newfile: %s\n", (const char*)newfile.mb_str(wxConvLocal));
     }
     
-    // only delete oldtempstart/newtempstart if they're not being used to
-    // store the current layer's starting pattern
-    if ( !oldtempstart.IsEmpty() && wxFileExists(oldtempstart) &&
-        oldtempstart != currlayer->startfile &&
-        oldtempstart != currlayer->currfile ) {
-        wxRemoveFile(oldtempstart);
+    if (!newfile.IsEmpty() && wxFileExists(newfile)) {
+        wxRemoveFile(newfile);
     }
-    if ( !newtempstart.IsEmpty() && wxFileExists(newtempstart) &&
-        newtempstart != currlayer->startfile &&
-        newtempstart != currlayer->currfile ) {
-        wxRemoveFile(newtempstart);
+
+    if (delete_all_temps) {
+        // we're in ClearUndoRedo so it's safe to delete oldtempstart/newtempstart/oldcurrfile/newcurrfile
+        // if they are in tempdir and not being used to store the current layer's starting pattern
+        // (the latter condition allows user to Reset after disabling undo/redo)
+        
+        if (!oldtempstart.IsEmpty() && wxFileExists(oldtempstart) &&
+            oldtempstart.StartsWith(tempdir) && oldtempstart != currlayer->currfile) {
+            wxRemoveFile(oldtempstart);
+            //printf("removed oldtempstart: %s\n", (const char*)oldtempstart.mb_str(wxConvLocal)); fflush(stdout);
+        }
+        
+        if (!newtempstart.IsEmpty() && wxFileExists(newtempstart) &&
+            newtempstart.StartsWith(tempdir) && newtempstart != currlayer->currfile) {
+            wxRemoveFile(newtempstart);
+            //printf("removed newtempstart: %s\n", (const char*)newtempstart.mb_str(wxConvLocal)); fflush(stdout);
+        }
+        
+        if (!oldcurrfile.IsEmpty() && wxFileExists(oldcurrfile) &&
+            oldcurrfile.StartsWith(tempdir) && oldcurrfile != currlayer->currfile) {
+            wxRemoveFile(oldcurrfile);
+            //printf("removed oldcurrfile: %s\n", (const char*)oldcurrfile.mb_str(wxConvLocal)); fflush(stdout);
+        }
+        
+        if (!newcurrfile.IsEmpty() && wxFileExists(newcurrfile) &&
+            newcurrfile.StartsWith(tempdir) && newcurrfile != currlayer->currfile) {
+            wxRemoveFile(newcurrfile);
+            //printf("removed newcurrfile: %s\n", (const char*)newcurrfile.mb_str(wxConvLocal)); fflush(stdout);
+        }
     }
 }
 
@@ -260,10 +425,15 @@ bool ChangeNode::DoChange(bool undo)
             break;
             
         case genchange:
+            currlayer->currfile = oldcurrfile;
             if (undo) {
                 currlayer->currsel = oldsel;
                 mainptr->RestorePattern(oldgen, oldfile, oldx, oldy, oldmag, oldbase, oldexpo);
             } else {
+                if (startinfo) {
+                    // restore starting info for use by ResetPattern
+                    startinfo->Restore();
+                }
                 currlayer->currsel = newsel;
                 mainptr->RestorePattern(newgen, newfile, newx, newy, newmag, newbase, newexpo);
             }
@@ -275,55 +445,17 @@ bool ChangeNode::DoChange(bool undo)
                 currlayer->startgen = oldstartgen;
                 currlayer->savestart = oldsave;
                 currlayer->tempstart = oldtempstart;
-                currlayer->startfile = oldstartfile;
                 currlayer->currfile = oldcurrfile;
-                if (oldtempstart != newtempstart) {
-                    currlayer->startdirty = olddirty;
-                    currlayer->startalgo = oldalgo;
-                    currlayer->startrule = oldrule;
-                    currlayer->startx = oldx;
-                    currlayer->starty = oldy;
-                    currlayer->startmag = oldmag;
-                    currlayer->startbase = oldbase;
-                    currlayer->startexpo = oldexpo;
-                    currlayer->startsel = oldsel;
-                    currlayer->startname = oldname;
-                    if (currlayer->cloneid > 0) {
-                        for ( int i = 0; i < numlayers; i++ ) {
-                            Layer* cloneptr = GetLayer(i);
-                            if (cloneptr != currlayer && cloneptr->cloneid == currlayer->cloneid) {
-                                cloneptr->startname = oldclone[i];
-                            }
-                        }
-                    }
+                if (startinfo) {
+                    // restore starting info for use by ResetPattern
+                    startinfo->Restore();
                 }
             } else {
                 mainptr->ChangeGenCount(newgen.tostring(), true);
                 currlayer->startgen = newstartgen;
                 currlayer->savestart = newsave;
                 currlayer->tempstart = newtempstart;
-                currlayer->startfile = newstartfile;
                 currlayer->currfile = newcurrfile;
-                if (oldtempstart != newtempstart) {
-                    currlayer->startdirty = newdirty;
-                    currlayer->startalgo = newalgo;
-                    currlayer->startrule = newrule;
-                    currlayer->startx = newx;
-                    currlayer->starty = newy;
-                    currlayer->startmag = newmag;
-                    currlayer->startbase = newbase;
-                    currlayer->startexpo = newexpo;
-                    currlayer->startsel = newsel;
-                    currlayer->startname = newname;
-                    if (currlayer->cloneid > 0) {
-                        for ( int i = 0; i < numlayers; i++ ) {
-                            Layer* cloneptr = GetLayer(i);
-                            if (cloneptr != currlayer && cloneptr->cloneid == currlayer->cloneid) {
-                                cloneptr->startname = newclone[i];
-                            }
-                        }
-                    }
-                }
             }
             // Reset item may become enabled/disabled
             mainptr->UpdateMenuItems();
@@ -347,7 +479,7 @@ bool ChangeNode::DoChange(bool undo)
                 if (whichlayer == currlayer) {
                     if (olddirty == newdirty) mainptr->SetWindowTitle(currlayer->currname);
                     // if olddirty != newdirty then UndoChange/RedoChange will call
-                    // MarkLayerClean/MarkLayerDirty (they call SetWindowTitle)
+                    // MarkLayerClean/MarkLayerDirty (and they call SetWindowTitle)
                 } else {
                     // whichlayer is non-active clone so only update Layer menu items
                     for (int i = 0; i < numlayers; i++)
@@ -414,7 +546,6 @@ UndoRedo::UndoRedo()
     doingscriptchanges = false;   // not undoing/redoing script changes
     prevfile = wxEmptyString;     // play safe for ClearUndoRedo
     startcount = 0;               // unfinished RememberGenStart calls
-    fixsetgen = false;            // no setgen nodes need fixing
     
     // need to remember if script has created a new layer (not a clone)
     if (inscript) RememberScriptStart();
@@ -651,11 +782,9 @@ void UndoRedo::RememberSelection(const wxString& action)
 void UndoRedo::SaveCurrentPattern(const wxString& tempfile)
 {
     const char* err = NULL;
-    //!!! need lifealgo::CanWriteFormat(MC_format) method???
     if ( currlayer->algo->hyperCapable() ) {
         // save hlife pattern in a macrocell file
-        err = mainptr->WritePattern(tempfile, MC_format, no_compression,
-                                    0, 0, 0, 0);
+        err = mainptr->WritePattern(tempfile, MC_format, no_compression, 0, 0, 0, 0);
     } else {
         // can only save RLE file if edges are within getcell/setcell limits
         bigint top, left, bottom, right;
@@ -708,46 +837,9 @@ void UndoRedo::RememberGenStart()
     if (prevgen == currlayer->startgen) {
         // we can just reset to starting pattern
         prevfile = wxEmptyString;
-        
-        if (fixsetgen) {
-            // SaveStartingPattern has just been called so search undolist for setgen
-            // node that changed tempstart and update the starting info in that node;
-            // yuk -- is there a simpler solution???
-            wxList::compatibility_iterator node = undolist.GetFirst();
-            while (node) {
-                ChangeNode* change = (ChangeNode*) node->GetData();
-                if (change->changeid == setgen &&
-                    change->oldtempstart != change->newtempstart) {
-                    change->newdirty = currlayer->startdirty;
-                    change->newalgo = currlayer->startalgo;
-                    change->newrule = currlayer->startrule;
-                    change->newx = currlayer->startx;
-                    change->newy = currlayer->starty;
-                    change->newmag = currlayer->startmag;
-                    change->newbase = currlayer->startbase;
-                    change->newexpo = currlayer->startexpo;
-                    change->newsel = currlayer->startsel;
-                    change->newname = currlayer->startname;
-                    if (currlayer->cloneid > 0) {
-                        for ( int i = 0; i < numlayers; i++ ) {
-                            Layer* cloneptr = GetLayer(i);
-                            if (cloneptr != currlayer && cloneptr->cloneid == currlayer->cloneid) {
-                                change->newclone[i] = cloneptr->startname;
-                            }
-                        }
-                    }
-                    // do NOT reset fixsetgen to false here; the gen change might
-                    // be removed when clearing the redo list and so we may need
-                    // to update this setgen node again after a new gen change
-                    break;
-                }
-                node = node->GetNext();
-            }
-        }
-        
     } else {
         // save starting pattern in a unique temporary file
-        prevfile = wxFileName::CreateTempFileName(tempdir + temp_prefix);
+        prevfile = wxFileName::CreateTempFileName(tempdir + genchange_prefix);
         
         // if head of undo list is a genchange node then we can copy that
         // change node's newfile to prevfile; this makes consecutive generating
@@ -802,7 +894,7 @@ void UndoRedo::RememberGenFinish()
         fpath = wxEmptyString;
     } else {
         // save finishing pattern in a unique temporary file
-        fpath = wxFileName::CreateTempFileName(tempdir + temp_prefix);
+        fpath = wxFileName::CreateTempFileName(tempdir + genchange_prefix);
         SaveCurrentPattern(fpath);
     }
     
@@ -831,6 +923,16 @@ void UndoRedo::RememberGenFinish()
     change->newexpo = currlayer->currexpo;
     change->oldsel = prevsel;
     change->newsel = currlayer->currsel;
+
+    // also remember the file containing the starting pattern
+    // (in case it is changed by by RememberSetGen or RememberNameChange)
+    change->oldcurrfile = currlayer->currfile;
+
+    if (change->oldgen == currlayer->startgen) {
+        // save starting info set by recent SaveStartingPattern call
+        // (the info will be restored when redoing this genchange node)
+        change->startinfo = new StartingInfo(NULL, NULL, NULL);
+    }
     
     // prevfile has been saved in change->oldfile (~ChangeNode will delete it)
     prevfile = wxEmptyString;
@@ -890,13 +992,17 @@ void UndoRedo::SyncUndoHistory()
         if (change->changeid == genchange && change->oldgen == currlayer->startgen) {
             if (change->scriptgen) {
                 // gen change was done by a script so keep winding back the undo list
-                // to just past the scriptstart node, or until the list is empty
+                // until the scriptstart node, or until the list is empty
                 while (!undolist.IsEmpty()) {
                     node = undolist.GetFirst();
                     change = (ChangeNode*) node->GetData();
-                    undolist.Erase(node);
-                    redolist.Insert(change);
-                    if (change->changeid == scriptstart) break;
+                    if (change->changeid == scriptstart) {
+                        undolist.Erase(node);
+                        redolist.Insert(change);
+                        break;
+                    }
+                    // undo this change so Reset and Undo restore to the same pattern
+                    UndoChange();
                 }
             }
             // update Undo/Redo items so they show the correct suffix
@@ -914,16 +1020,15 @@ void UndoRedo::RememberSetGen(bigint& oldgen, bigint& newgen,
                               bigint& oldstartgen, bool oldsave)
 {
     wxString oldtempstart = currlayer->tempstart;
-    wxString oldstartfile = currlayer->startfile;
     wxString oldcurrfile = currlayer->currfile;
     if (oldgen > oldstartgen && newgen <= oldstartgen) {
         // if pattern is generated then tempstart will be clobbered by
         // SaveStartingPattern, so change tempstart to a new temporary file
-        currlayer->tempstart = wxFileName::CreateTempFileName(tempdir + wxT("golly_setgen_"));
+        currlayer->tempstart = wxFileName::CreateTempFileName(tempdir + setgen_prefix);
         
-        // also need to update startfile and currfile (currlayer->savestart is true)
-        currlayer->startfile = currlayer->tempstart;
-        currlayer->currfile = wxEmptyString;
+        // also need to update currfile (currlayer->savestart is true)
+        if (!currlayer->savestart) Warning(_("Bug in RememberSetGen: savestart is false!"));
+        currlayer->currfile = currlayer->tempstart;
     }
     
     // clear the redo history
@@ -943,56 +1048,15 @@ void UndoRedo::RememberSetGen(bigint& oldgen, bigint& newgen,
     change->newsave = currlayer->savestart;
     change->oldtempstart = oldtempstart;
     change->newtempstart = currlayer->tempstart;
-    change->oldstartfile = oldstartfile;
-    change->newstartfile = currlayer->startfile;
     change->oldcurrfile = oldcurrfile;
     change->newcurrfile = currlayer->currfile;
-    
-    if (change->oldtempstart != change->newtempstart) {
-        // save extra starting info set by previous SaveStartingPattern so that
-        // Undoing this setgen change will restore the correct info for a Reset
-        change->olddirty = currlayer->startdirty;
-        change->oldalgo = currlayer->startalgo;
-        change->oldrule = currlayer->startrule;
-        change->oldx = currlayer->startx;
-        change->oldy = currlayer->starty;
-        change->oldmag = currlayer->startmag;
-        change->oldbase = currlayer->startbase;
-        change->oldexpo = currlayer->startexpo;
-        change->oldsel = currlayer->startsel;
-        change->oldname = currlayer->startname;
-        if (currlayer->cloneid > 0) {
-            for ( int i = 0; i < numlayers; i++ ) {
-                Layer* cloneptr = GetLayer(i);
-                if (cloneptr != currlayer && cloneptr->cloneid == currlayer->cloneid) {
-                    change->oldclone[i] = cloneptr->startname;
-                }
-            }
-        }
-        
-        // following settings will be updated by next RememberGenStart call so that
-        // Redoing this setgen change will restore the correct info for a Reset
-        fixsetgen = true;
-        change->newdirty = currlayer->startdirty;
-        change->newalgo = currlayer->startalgo;
-        change->newrule = currlayer->startrule;
-        change->newx = currlayer->startx;
-        change->newy = currlayer->starty;
-        change->newmag = currlayer->startmag;
-        change->newbase = currlayer->startbase;
-        change->newexpo = currlayer->startexpo;
-        change->newsel = currlayer->startsel;
-        change->newname = currlayer->startname;
-        if (currlayer->cloneid > 0) {
-            for ( int i = 0; i < numlayers; i++ ) {
-                Layer* cloneptr = GetLayer(i);
-                if (cloneptr != currlayer && cloneptr->cloneid == currlayer->cloneid) {
-                    change->newclone[i] = cloneptr->startname;
-                }
-            }
-        }
+
+    if (oldtempstart != currlayer->tempstart) {
+        // save starting info set by most recent SaveStartingPattern call
+        // (the info will be restored when undoing this setgen node)
+        change->startinfo = new StartingInfo(NULL, NULL, NULL);
     }
-    
+        
     undolist.Insert(change);
     
     // update Undo item in Edit menu
@@ -1040,10 +1104,9 @@ void UndoRedo::RememberNameChange(const wxString& oldname, const wxString& oldcu
 void UndoRedo::DeletingClone(int index)
 {
     // the given cloned layer is about to be deleted, so we need to go thru the
-    // undo/redo lists and, for each namechange node, set a matching whichlayer
-    // ptr to NULL so DoChange can ignore later changes involving this layer;
-    // very ugly, but I don't see any better solution if we're going to allow
-    // cloned layers to have different names
+    // undo/redo lists and fix up any nodes that have pointers to that clone
+    // (very ugly, but I don't see any better solution if we're going to allow
+    // cloned layers to have different names)
     
     Layer* cloneptr = GetLayer(index);
     wxList::compatibility_iterator node;
@@ -1051,16 +1114,24 @@ void UndoRedo::DeletingClone(int index)
     node = undolist.GetFirst();
     while (node) {
         ChangeNode* change = (ChangeNode*) node->GetData();
-        if (change->changeid == namechange && change->whichlayer == cloneptr)
+        if (change->whichlayer == cloneptr) {
             change->whichlayer = NULL;
+        }
+        if (change->startinfo) {
+            change->startinfo->RemoveClone(cloneptr);
+        }
         node = node->GetNext();
     }
     
     node = redolist.GetFirst();
     while (node) {
         ChangeNode* change = (ChangeNode*) node->GetData();
-        if (change->changeid == namechange && change->whichlayer == cloneptr)
+        if (change->whichlayer == cloneptr) {
             change->whichlayer = NULL;
+        }
+        if (change->startinfo) {
+            change->startinfo->RemoveClone(cloneptr);
+        }
         node = node->GetNext();
     }
 }
@@ -1258,10 +1329,9 @@ void UndoRedo::UndoChange()
     if (!CanUndo()) return;
     
     if (mainptr->generating) {
-        // terminate generating loop and set command_pending flag
-        mainptr->Stop();
         mainptr->command_pending = true;
         mainptr->cmdevent.SetId(ID_UNDO);
+        mainptr->Stop();
         return;
     }
     
@@ -1327,10 +1397,9 @@ void UndoRedo::RedoChange()
     
     /* can't redo while generating -- redo list will be empty
     if (mainptr->generating) {
-        // terminate generating loop and set command_pending flag
-        mainptr->Stop();
         mainptr->command_pending = true;
         mainptr->cmdevent.SetId(ID_REDO);
+        mainptr->Stop();
         return;
     }
     */
@@ -1467,11 +1536,14 @@ void UndoRedo::ClearUndoRedo()
         startcount = 0;
     }
     
+    // set flag so ChangeNode::~ChangeNode() can delete all temporary files
+    delete_all_temps = true;
+
     // clear the undo/redo lists (and delete each node's data)
     WX_CLEAR_LIST(wxList, undolist);
     WX_CLEAR_LIST(wxList, redolist);
     
-    fixsetgen = false;
+    delete_all_temps = false;
     
     if (inscript) {
         // script has called a command like new() so add a scriptstart node
@@ -1490,50 +1562,70 @@ void UndoRedo::ClearUndoRedo()
 
 bool CopyTempFiles(ChangeNode* srcnode, ChangeNode* destnode, const wxString& tempstart1)
 {
-    // if srcnode has any existing temporary files then create new
+    // if srcnode has any existing temporary files then, if necessary, create new
     // temporary file names in the destnode and copy each file
     bool allcopied = true;
     
     if ( !srcnode->oldfile.IsEmpty() && wxFileExists(srcnode->oldfile) ) {
-        destnode->oldfile = wxFileName::CreateTempFileName(tempdir + wxT("golly_dupe1_"));
+        destnode->oldfile = wxFileName::CreateTempFileName(tempdir + dupe1_prefix);
         if ( !wxCopyFile(srcnode->oldfile, destnode->oldfile, true) )
             allcopied = false;
     }
     
     if ( !srcnode->newfile.IsEmpty() && wxFileExists(srcnode->newfile) ) {
-        destnode->newfile = wxFileName::CreateTempFileName(tempdir + wxT("golly_dupe2_"));
+        destnode->newfile = wxFileName::CreateTempFileName(tempdir + dupe2_prefix);
         if ( !wxCopyFile(srcnode->newfile, destnode->newfile, true) )
             allcopied = false;
     }
     
+    if ( !srcnode->oldcurrfile.IsEmpty() && wxFileExists(srcnode->oldcurrfile) ) {
+        if (srcnode->oldcurrfile == currlayer->tempstart) {
+            // the file has already been copied to tempstart1 by Layer::Layer()
+            destnode->oldcurrfile = tempstart1;
+        } else if (srcnode->oldcurrfile.StartsWith(tempdir)) {
+            destnode->oldcurrfile = wxFileName::CreateTempFileName(tempdir + dupe3_prefix);
+            if ( !wxCopyFile(srcnode->oldcurrfile, destnode->oldcurrfile, true) )
+                allcopied = false;
+        }
+    }
+    
+    if ( !srcnode->newcurrfile.IsEmpty() && wxFileExists(srcnode->newcurrfile) ) {
+        if (srcnode->newcurrfile == srcnode->oldcurrfile) {
+            // use destnode->oldcurrfile set above or earlier in DuplicateHistory
+            destnode->newcurrfile = destnode->oldcurrfile;
+        } else if (srcnode->newcurrfile == currlayer->tempstart) {
+            // the file has already been copied to tempstart1 by Layer::Layer()
+            destnode->newcurrfile = tempstart1;
+        } else if (srcnode->newcurrfile.StartsWith(tempdir)) {
+            destnode->newcurrfile = wxFileName::CreateTempFileName(tempdir + dupe4_prefix);
+            if ( !wxCopyFile(srcnode->newcurrfile, destnode->newcurrfile, true) )
+                allcopied = false;
+        }
+    }
+    
     if ( !srcnode->oldtempstart.IsEmpty() && wxFileExists(srcnode->oldtempstart) ) {
         if (srcnode->oldtempstart == currlayer->tempstart) {
             // the file has already been copied to tempstart1 by Layer::Layer()
             destnode->oldtempstart = tempstart1;
-        } else {
-            destnode->oldtempstart = wxFileName::CreateTempFileName(tempdir + wxT("golly_dupe3_"));
+        } else if (srcnode->oldtempstart.StartsWith(tempdir)) {
+            destnode->oldtempstart = wxFileName::CreateTempFileName(tempdir + dupe5_prefix);
             if ( !wxCopyFile(srcnode->oldtempstart, destnode->oldtempstart, true) )
                 allcopied = false;
         }
-        if (srcnode->oldstartfile == srcnode->oldtempstart)
-            destnode->oldstartfile = destnode->oldtempstart;
-        if (srcnode->oldcurrfile == srcnode->oldtempstart)
-            destnode->oldcurrfile = destnode->oldtempstart;
     }
     
     if ( !srcnode->newtempstart.IsEmpty() && wxFileExists(srcnode->newtempstart) ) {
-        if (srcnode->newtempstart == currlayer->tempstart) {
+        if (srcnode->newtempstart == srcnode->oldtempstart) {
+            // use destnode->oldtempstart set above or earlier in DuplicateHistory
+            destnode->newtempstart = destnode->oldtempstart;
+        } else if (srcnode->newtempstart == currlayer->tempstart) {
             // the file has already been copied to tempstart1 by Layer::Layer()
             destnode->newtempstart = tempstart1;
-        } else {
-            destnode->newtempstart = wxFileName::CreateTempFileName(tempdir + wxT("golly_dupe4_"));
+        } else if (srcnode->newtempstart.StartsWith(tempdir)) {
+            destnode->newtempstart = wxFileName::CreateTempFileName(tempdir + dupe6_prefix);
             if ( !wxCopyFile(srcnode->newtempstart, destnode->newtempstart, true) )
                 allcopied = false;
         }
-        if (srcnode->newstartfile == srcnode->newtempstart)
-            destnode->newstartfile = destnode->newtempstart;
-        if (srcnode->newcurrfile == srcnode->newtempstart)
-            destnode->newcurrfile = destnode->newtempstart;
     }
     
     return allcopied;
@@ -1549,7 +1641,7 @@ void UndoRedo::DuplicateHistory(Layer* oldlayer, Layer* newlayer)
     // a scriptstart node to undolist if inscript is true, but we don't
     // want that here because the old layer's history will already have one
     WX_CLEAR_LIST(wxList, undolist);
-    WX_CLEAR_LIST(wxList, redolist);
+    WX_CLEAR_LIST(wxList, redolist);    // should already be empty but play safe
     
     // safer to do our own shallow copy (avoids setting undolist/redolist)
     savecellchanges = history->savecellchanges;
@@ -1567,11 +1659,10 @@ void UndoRedo::DuplicateHistory(Layer* oldlayer, Layer* newlayer)
     prevexpo = history->prevexpo;
     prevsel = history->prevsel;
     startcount = history->startcount;
-    fixsetgen = history->fixsetgen;
     
     // copy existing temporary file to new name
     if ( !prevfile.IsEmpty() && wxFileExists(prevfile) ) {
-        prevfile = wxFileName::CreateTempFileName(tempdir + temp_prefix);
+        prevfile = wxFileName::CreateTempFileName(tempdir + genchange_prefix);
         if ( !wxCopyFile(history->prevfile, prevfile, true) ) {
             Warning(_("Could not copy prevfile!"));
             return;
@@ -1619,6 +1710,19 @@ void UndoRedo::DuplicateHistory(Layer* oldlayer, Layer* newlayer)
             memcpy(newchange->cellinfo, change->cellinfo, bytes);
         }
         
+        if (change->startinfo) {
+            newchange->startinfo = new StartingInfo(change->startinfo, oldlayer, newlayer);
+        }
+        
+        // if node is a name change then update whichlayer
+        if (newchange->changeid == namechange) {
+            if (change->whichlayer == oldlayer) {
+                newchange->whichlayer = newlayer;
+            } else {
+                newchange->whichlayer = NULL;
+            }
+        }
+        
         // copy any existing temporary files to new names
         if (!CopyTempFiles(change, newchange, newlayer->tempstart)) {
             Warning(_("Failed to copy temporary file in undolist!"));
@@ -1626,11 +1730,6 @@ void UndoRedo::DuplicateHistory(Layer* oldlayer, Layer* newlayer)
             return;
         }
         
-        // if node is a name change then update whichlayer to point to new layer
-        if (newchange->changeid == namechange) {
-            newchange->whichlayer = newlayer;
-        }
-        
         undolist.Append(newchange);
         node = node->GetNext();
     }
@@ -1662,6 +1761,19 @@ void UndoRedo::DuplicateHistory(Layer* oldlayer, Layer* newlayer)
             memcpy(newchange->cellinfo, change->cellinfo, bytes);
         }
         
+        if (change->startinfo) {
+            newchange->startinfo = new StartingInfo(change->startinfo, oldlayer, newlayer);
+        }
+        
+        // if node is a name change then update whichlayer
+        if (newchange->changeid == namechange) {
+            if (change->whichlayer == oldlayer) {
+                newchange->whichlayer = newlayer;
+            } else {
+                newchange->whichlayer = NULL;
+            }
+        }
+        
         // copy any existing temporary files to new names
         if (!CopyTempFiles(change, newchange, newlayer->tempstart)) {
             Warning(_("Failed to copy temporary file in redolist!"));
@@ -1669,11 +1781,6 @@ void UndoRedo::DuplicateHistory(Layer* oldlayer, Layer* newlayer)
             return;
         }
         
-        // if node is a name change then update whichlayer to point to new layer
-        if (newchange->changeid == namechange) {
-            newchange->whichlayer = newlayer;
-        }
-        
         redolist.Append(newchange);
         node = node->GetNext();
     }
