@@ -25,13 +25,16 @@
 #ifndef _WXVIEW_H_
 #define _WXVIEW_H_
 
-#include "bigint.h"     // for bigint
-#include "lifealgo.h"   // for lifealgo
-#include "wxselect.h"   // for Selection
+#include "wx/glcanvas.h"    // for wxGLCanvas, wxGLContext
 
-// Define a child window for viewing and editing patterns:
+#include "bigint.h"         // for bigint
+#include "lifealgo.h"       // for lifealgo
+#include "wxselect.h"       // for Selection
+#include "wxlayer.h"        // for Layer
 
-class PatternView : public wxWindow
+// OpenGL is used for viewing and editing patterns:
+
+class PatternView : public wxGLCanvas
 {
 public:
     PatternView(wxWindow* parent, wxCoord x, wxCoord y, int wd, int ht, long style);
@@ -42,7 +45,7 @@ public:
     void CopySelection();
     void ClearSelection();
     void ClearOutsideSelection();
-    bool GetClipboardPattern(lifealgo** tempalgo, bigint* t, bigint* l, bigint* b, bigint* r);
+    bool GetClipboardPattern(Layer* templayer, bigint* t, bigint* l, bigint* b, bigint* r);
     void PasteClipboard(bool toselection);
     void AbortPaste();
     void CyclePasteLocation();
@@ -89,7 +92,7 @@ public:
     void ToggleGridLines();
     void ToggleCellIcons();
     void ToggleCellColors();
-    void ToggleBuffering();
+    void ToggleSmarterScaling();
     void UpdateScrollBars();         // update thumb positions
     void CheckCursor(bool active);   // make sure cursor is up to date
     int GetMag();                    // get magnification (..., -1=2:1, 0=1:1, 1=1:2, ...)
@@ -148,8 +151,7 @@ private:
     void OnEraseBackground(wxEraseEvent& event);
     
     // edit functions
-    void ShowDrawing();
-    void DrawOneCell(wxDC& dc, int cx, int cy, int oldstate, int newstate);
+    void RememberOneCellChange(int cx, int cy, int oldstate, int newstate);
     void StartDrawingCells(int x, int y);
     void DrawCells(int x, int y);
     void PickCell(int x, int y);
@@ -180,9 +182,8 @@ private:
     int BigScroll(int xysize);
     
     // data
-    wxBitmap* viewbitmap;         // viewport bitmap used in OnPaint
-    int viewbitmapwd;             // width of viewport bitmap
-    int viewbitmapht;             // height of viewport bitmap
+    wxGLContext* glcontext;       // OpenGL context for this canvas
+    bool initgl;                  // need to initialize GL state?
     wxTimer* dragtimer;           // timer used while dragging mouse
     int cellx, celly;             // current cell's 32-bit position
     bigint bigcellx, bigcelly;    // current cell's position
@@ -192,7 +193,6 @@ private:
     bigint anchorx, anchory;      // anchor cell of current selection
     Selection prevsel;            // previous selection
     int drawstate;                // new cell state (0..255)
-    wxBrush* cellbrush;           // brush used to draw live cells
     int pastex, pastey;           // where user wants to paste clipboard pattern
     int hthumb, vthumb;           // current thumb box positions
     int realkey;                  // key code set by OnKeyDown
